
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 ************************************************************************/

/*
 * asn_any.c - BER encode, decode, print, free, type set up and installation
 *             routines for the ASN.1 ANY and ANY DEFINED BY types.
 *
 * MS 92
 * Copyright (C) 1992 Michael Sample and the University of British Columbia
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * $Header$
 * $Log$
 */

#include "asn_conf.h"
#include "asn_len.h"
#include "asn_tag.h"
#include "asn_oid.h"
#include "asn_int.h"
#include "asn_any.h"

/*
 * 2 hash tables. 1 for INTEGER to type mappings the other
 * for OBJECT IDENTIFER to type mappings.
 */
Table* anyOidHashTblG = NULL;
Table* anyIntHashTblG = NULL;

/*
 * given an ANY type value and a integer hash key, this defines
 * this any values type (gets ptr to hash tbl entry from int key).
 * The hash table entry contains ptrs to the encode/decode etc. routines.
 */
void
SetAnyTypeByInt PARAMS((v, id),
AsnAny* v _AND_
AsnInt id)
{
    Hash hash;
    void* anyInfo;

    /* use int as hash string */
    hash = MakeHash((char*)&id, sizeof(id));
    if (CheckForAndReturnValue(anyIntHashTblG, hash, &anyInfo))
        v->ai = (AnyInfo*) anyInfo;
    else
        v->ai = NULL; /* indicates failure */

} /* SetAnyTypeByInt */


/*
 * Same as SetAnyTypeByInt except that the hash key is an OBJECT IDENTIFER.
 */
void SetAnyTypeByOid PARAMS((v, id),
AsnAny* v _AND_
AsnOid* id)
{
    Hash hash;
    void* anyInfo;

    /* use encoded oid as hash string */
    hash = MakeHash(id->octs, id->octetLen);
    if (CheckForAndReturnValue(anyOidHashTblG, hash, &anyInfo))
        v->ai = (AnyInfo*) anyInfo;
    else
        v->ai = NULL; /* indicates failure */

} /* SetAnyTypeByOid */


/*
 * Creates an entry in the hash table that contains the 
 * type's size, encode, decode, free, and print routines and anyId.
 * The given intId is used as the hash key so future calls to
 * SetAnyTypeByInt with that intId as the id will reference this entry.
 * The anyId is stored in the hash tbl entry as well so the user can
 * figure out the type with a simple integer comparison.
 *
 * This routine is usually called from the AnyInit routine that
 * the compiler generates from MACRO info.  Call this routine
 * once for each possible ANY type to set up the hash table.
 * Future calls to SetAnyTypeByInt/Oid will reference this table.
 */
void 
InstallAnyByInt PARAMS((anyId, intId, size, Encode, Decode, Free, Print),
int anyId _AND_
AsnInt intId _AND_
unsigned int size _AND_
EncodeFcn Encode _AND_
DecodeFcn Decode _AND_
FreeFcn Free _AND_
PrintFcn Print)
{
    AnyInfo* a;
    Hash h;

    a = (AnyInfo*) RescIR_Zmalloc(IR_FILE, sizeof(AnyInfo));
    a->anyId = anyId;
    a->oid.octs = NULL;
    a->oid.octetLen = 0;
    a->intId = intId;
    a->size = size;
    a->Encode = Encode;
    a->Decode = Decode;
    a->Free = Free;
    a->Print = Print;

    if (anyIntHashTblG == NULL)
        anyIntHashTblG = InitHash();

    h = MakeHash((char*)&intId, sizeof(intId));
    Insert(anyIntHashTblG, a, h);

}  /* InstallAnyByOid */


/*
 * Same as InstallAnyByInt except the oid is used as the hash key
 */
void 
InstallAnyByOid PARAMS((anyId, oid, size, Encode, Decode, Free, Print),
int anyId _AND_
AsnOid* oid _AND_
unsigned int size _AND_
EncodeFcn Encode _AND_
DecodeFcn Decode _AND_
FreeFcn Free _AND_
PrintFcn Print)
{
    AnyInfo* a;
    Hash h;

    a = (AnyInfo*) RescIR_Zmalloc(IR_FILE, sizeof(AnyInfo));
    a->anyId = anyId;
    a->oid.octs = oid->octs;
    a->oid.octetLen = oid->octetLen;
    a->size = size;
    a->Encode = Encode;
    a->Decode = Decode;
    a->Free = Free;
    a->Print = Print;

    h = MakeHash(oid->octs, oid->octetLen);

    if (anyOidHashTblG == NULL)
        anyOidHashTblG = InitHash();

    Insert(anyOidHashTblG, a, h);

}  /* InstallAnyByOid */


/*
 * Calls the free routine in this type's any info.
 * If the routine ptr is NULL, nothing is done 
 * (This is the case for INTEGERs, BOOLEANs and other simple
 * values)
 */
void
FreeAsnAny PARAMS((v),
AsnAny* v)
{
    if ((v->ai != NULL) && (v->ai->Free != NULL))
        v->ai->Free(v->value);
} /* FreeAsnAny */


/*
 * Calls the Encode routine pointed to in the given type's
 * Any Info.  If the routine ptr is NULL nothing is encoded
 * (This should set some type of error).
 * Note: this calls the BEncFoo not BEncFooContent routine form
 * since the tags are needed too.
 */
AsnLen
BEncAsnAny PARAMS((b, v),
BUF_TYPE b _AND_
AsnAny* v)
{
    if ((v->ai != NULL) && (v->ai->Encode != NULL))
        return(v->ai->Encode(b, v->value));
    else
        return (0);
} /* BEncAsnAny */


/*
 * Calls the Decode routine pointed to in the given type's
 * Any Info.  If the routine ptr is NULL any error is flagged.
 * Note: this calls the BDecFoo not BDecFooContent routine form
 * since the tags are needed too.
 */
void BDecAsnAny PARAMS((b, result, bytesDecoded, env),
BUF_TYPE b _AND_
AsnAny*  result _AND_
AsnLen* bytesDecoded _AND_
ENV_TYPE env)
{
    if ((result->ai != NULL) && (result->ai->Decode != NULL))
    {
        result->value = (void*)RescIR_Zmalloc(IR_FILE, result->ai->size);
        result->ai->Decode(b, result->value, bytesDecoded, env);
    }
    else
    {
        Asn1Error("ERROR - ANY Decode routine is NULL\n");
        longjmp(env, -44);
    }
}

/*
 * Calls the print routine pointed to from the given type's
 * Any Info.  Prints an error if the type does not have
 * any 'AnyInfo' or if the AnyInfo has a NULL Print routine ptr.
 */
void PrintAsnAny PARAMS((f, v, indent),
FILE* f _AND_
AsnAny* v _AND_
unsigned short indent)
{
    if ((v->ai != NULL) && (v->ai->Print != NULL))
        v->ai->Print(f, v->value);
    else
        fprintf(f," -- ERROR: malformed ANY value --");
}



