
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: CF_Assoc
 *
 * DESCRIPTION:
 *	This unit contains functions used to control the associations
 *	maintained during a session.
 *
 * CONTENTS:
 *	CsCf_AssocInit
 *	CsCf_AssocGet
 *	CsCf_AssocDel
 *	CsCf_AssocGetFirst
 *	CsCf_AssocGetNext
 *	newSock
 *	getSock
 *	delSock
 *
 * NOTES:
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 *
/***************************************************************************/

#include <ir.h>
#include <memory.h>
#include <ir_pli.h>
#include <ir_cs.h>
#include <resc.h>
#include <resclist.h>

static IR_LIST CSCF_AssocList;


static CS_SOCK_ITEM 	*newSock(IR_ASSOC_ID assocId);
static CS_SOCK_ITEM	*getSock(IR_ASSOC_ID assocId);
static IR_STATUS 	delSock(IR_ASSOC_ID assocId);
static SHORT		getSockIndex(IR_ASSOC_ID assocId);
static void		sockItemDestroy(CHAR *src, void *);

/****************************************************************************
 * FUNCTION: CsCf_AssocInit
 *
 * PURPOSE:
 *	Initialize the association list and the mode of operation.
 *	event, polled.
 *
 * INPUT:
 *	commMode		- mode of operation.
 *
 *
 * OUTPUT:
 *	void
 *
 *
 * NOTES:
 *
 ***************************************************************************/

void CsCf_AssocInit(IRC_INIT_REQ *initReq )
{
	
	RESC_ASN_NO_MALLOC(&CSCF_AssocList);
	RESC_LIST_INIT(&CSCF_AssocList);

}


/****************************************************************************
 * FUNCTION: CsCf_AssocNew
 *
 * PURPOSE:
 *	This function will create a new association entry in the association
 *	list.
 *
 * INPUT:
 *	request		- IRC Assoc request.
 *	hostent		- pointer to hostent structure. (net stuff)
 *
 *
 * OUTPUT:
 *	void
 *
 *
 * NOTES:
 *
 ***************************************************************************/

CS_SOCK_ITEM *CsCf_AssocNew(IR_ASSOC_ID assocId)
{
	CS_SOCK_ITEM *curSock;

	if ((curSock = newSock(assocId)) != (CS_SOCK_ITEM *)NULL)
	{
		curSock->state = CS_SOCK_ST_INIT;
	}
	return(curSock);
}

/****************************************************************************
 * FUNCTION: CsCf_AssocGet
 *
 * PURPOSE:
 *	This function will return information maintained for an
 *	association.
 *
 * INPUT:
 *	assocId
 *
 *
 * OUTPUT:
 *	CS_SOCK_ITEM		- pointer to current item.
 *
 *
 * NOTES:
 *
 ***************************************************************************/

CS_SOCK_ITEM *CsCf_AssocGet(IR_ASSOC_ID assocId)
/*
 * This function will return fail if the association cannot be found.
 */
{
	return(getSock(assocId));
}

/****************************************************************************
 * FUNCTION: CsCf_AssocDel
 *
 * PURPOSE:
 *	This function will remove an association from the list.
 *
 * INPUT:
 *	assocId
 *
 *
 * OUTPUT:
 *
 *
 * NOTES:
 *
 ***************************************************************************/

IR_STATUS CsCf_AssocDel(IR_ASSOC_ID assocId)
/*
 * This function will return fail if the association cannot be found.
 */
{
	return (delSock(assocId));
}

/****************************************************************************
 * FUNCTION: CsCf_AssocFirst
 *
 * PURPOSE:
 *	This function will return the first association id in the list.
 *	If no associations in the list then an unknown id will be
 *	returned. This is a special function used originally for polling.
 *
 * INPUT:
 *	none.
 *
 *
 * OUTPUT:
 *	assocId
 *
 *
 * NOTES:
 *
 ***************************************************************************/

IR_ASSOC_ID CsCf_AssocGetFirst()
/*
 * This function will return the first assocId.
 */
{
	CS_SOCK_ITEM	*item;

	if (RESC_LIST_SIZE(&CSCF_AssocList) > 0)
	{
		RESC_LIST_SET_FIRST(&CSCF_AssocList);
		item = RESC_LIST_GET(&CSCF_AssocList);
		return(item->assocId);
	}
	else
	{
		return ((IR_ASSOC_ID)IR_UNKNOWN_ID_TYPE);
	}
}

/****************************************************************************
 * FUNCTION: CsCf_AssocGetNext
 *
 * PURPOSE:
 *	Given an association identifier this function will return the next one.
 *	If there is no next one then this function will return the first
 *	one in the list.
 *
 * INPUT:
 *	none.
 *
 *
 * OUTPUT:
 *	assocId
 *
 *
 * NOTES:
 *
 ***************************************************************************/

IR_ASSOC_ID CsCf_AssocGetNext(IR_ASSOC_ID assocId)
/*
 * This function will return the first assocId.
 */
{
	
	SHORT		index;
	CS_SOCK_ITEM	*sockPtr;

	if (RESC_LIST_SIZE(&CSCF_AssocList) == 0)
	{
		return((IR_ASSOC_ID)IR_UNKNOWN_ID_TYPE);
	}
	index = getSockIndex(assocId);
	if ((index == IR_UNKNOWN_ID_TYPE) || 
	    RESC_LIST_IS_LAST(&CSCF_AssocList))
	{
		RESC_LIST_SET_FIRST(&CSCF_AssocList);
	}
	else
	{
		RESC_LIST_SET_NEXT(&CSCF_AssocList);
	}
	sockPtr = RESC_LIST_GET(&CSCF_AssocList);
	return(sockPtr->assocId);
}

/****************************************************************************
 * LOCAL FUNCTION: newSock 
 *
 * PURPOSE:
 *	This function will create a new entry in the association list and
 *	return a pointer to the new item.
 *
 * INPUT:
 *	assocId		- unique identifier for the new item.
 *
 *
 * OUTPUT:
 *	pointer to a sockItem. Null if assocId already exists.
 *
 *
 * NOTES:
 *
 ***************************************************************************/

static CS_SOCK_ITEM	*newSock(IR_ASSOC_ID assocId)
/*
 * Returns pointer to new item or NULL if assocId already exists or if
 * error in malloc.
 */
{
	CS_SOCK_ITEM	*sockPtr;

	/*
 	 * malloc the space for the new entry.
	 */
	if ((sockPtr = (CS_SOCK_ITEM *)RescIR_Malloc(
	    IR_FILE, sizeof(CS_SOCK_ITEM))) == (CS_SOCK_ITEM *)NULL)
	{
		CS_LOG_PRINTS(IR_FILE, "Memory error detected allocating sock entry\n");
		return((CS_SOCK_ITEM *)NULL);
	}

	RESC_ASN_MALLOCED(sockPtr);

	/* Initialize that baby! */
	sockPtr->assocId = assocId;
	sockPtr->socketFd = IR_UNKNOWN_ID_TYPE;
	sockPtr->state = 0;
	sockPtr->port = 0;
	RESC_ASN_NO_MALLOC(&sockPtr->readList);
	RESC_LIST_INIT(&sockPtr->readList);
	RESC_ASN_NO_MALLOC(&sockPtr->sendList);
	RESC_LIST_INIT(&sockPtr->sendList);

	RescList_Append(IR_FILE, &CSCF_AssocList, sockPtr,
   	    sockItemDestroy, NULL);

	/* Return the new socket item entry */
	return(sockPtr);
}

/****************************************************************************
 * LOCAL FUNCTION: getSock 
 *
 * PURPOSE:
 *	This function will extract the entry in the association list and
 *	return a pointer to the item.
 *
 * INPUT:
 *	assocId		- unique identifier for the new item.
 *
 *
 * OUTPUT:
 *	pointer to a sockItem. Null if assocId already exists.
 *
 *
 * NOTES:
 *
 ***************************************************************************/

static CS_SOCK_ITEM	*getSock(IR_ASSOC_ID assocId)
/*
 * Returns pointer to a matching item or NULL if assocId not found.
 */
{
	CS_SOCK_ITEM	*sockPtr;
	SHORT		i;

	i = getSockIndex(assocId);
	
	if (i == IR_UNKNOWN_ID_TYPE)
	{
		return((CS_SOCK_ITEM *)NULL);
	}
	else
	{
		RESC_LIST_SET_CUR(&CSCF_AssocList, i);
		sockPtr = RESC_LIST_GET(&CSCF_AssocList);
		return(sockPtr);
	}
}

/****************************************************************************
 * LOCAL FUNCTION: delSock 
 *
 * PURPOSE:
 *	This function will delete an entry in the association list.
 *
 * INPUT:
 *	assocId		- unique identifier for the new item.
 *
 *
 * OUTPUT:
 *	none
 *
 *
 * NOTES:
 *
 ***************************************************************************/

static IR_STATUS delSock(IR_ASSOC_ID assocId)
{
	CS_SOCK_ITEM	*sockPtr;
	CS_SOCK_ITEM	*lastSock;
	SHORT		index;

	index = getSockIndex(assocId);

	if (index == IR_UNKNOWN_ID_TYPE)
	{
		return(IR_SUCCESS);
	}
	else
	{
		RescList_Delete(IR_FILE, &CSCF_AssocList, index);
		return(IR_SUCCESS);
	}

}

static SHORT	getSockIndex(IR_ASSOC_ID assocId)
/*
 * Returns index of a matching item or -1 if assocId not found.
 */
{
	CS_SOCK_ITEM	*sockPtr;
	SHORT		i;

	if (RESC_LIST_SIZE(&CSCF_AssocList) == 0)
	{
		return(IR_UNKNOWN_ID_TYPE);
	}

	RESC_LIST_SET_FIRST(&CSCF_AssocList);
	for (i = 0; i < RESC_LIST_SIZE(&CSCF_AssocList); i++)
	{
		sockPtr = RESC_LIST_GET(&CSCF_AssocList);
		if (sockPtr->assocId == assocId)
		{
			break;
		}
		else
		{
			RESC_LIST_SET_NEXT(&CSCF_AssocList);
		}
	}

	if (sockPtr->assocId == assocId)
	{
		/* Found association entry */
		return (i);
	}
	else
	{
		return(IR_UNKNOWN_ID_TYPE);
	}
}

static void sockItemDestroy(CHAR *srcFile, void *item)
{
	CS_SOCK_ITEM	*sockPtr;

	if (item == NULL)
	{
		return;
	}
	sockPtr = (CS_SOCK_ITEM *)item;

	RescList_Destroy(srcFile, &sockPtr->readList);
	RescList_Destroy(srcFile, &sockPtr->sendList);

	if (sockPtr->hostp)
	{
		RescIR_Free(srcFile, (CHAR *)sockPtr->hostp);
	}
	if (RESC_ASN_IS_MALLOCED(sockPtr))
	{
		RescIR_Free(srcFile, (CHAR *)sockPtr);
	}
}
