
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: CF_Poll
 *
 * DESCRIPTION:
 *	This unit contains functions used to poll associations for
 *	input/output activity.
 *
 * CONTENTS:
 *	CsCf_Poll
 *	CsCf_SendData
 *	CsCf_ReadData
 *	checkForAPDU
 *	apduTagLen
 *
 * NOTES:
 *
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
/***************************************************************************/

#include <ir.h>
#ifdef WINSOCKET
#include <stdlib.h>
#endif
#include <memory.h>
#include <ir_pli.h>
#include <ir_cs.h>
#include <resc.h>
#include <ir_net.h>
#include <resclist.h>
#include <debug.h>

/* apduTagLen return error code for buffer overrun */
#define LEN_ERROR	(-1)

static IR_COMM_EVENT_STATUS checkForAPDU(CS_SOCK_ITEM *sockItem);
static USHORT apduTagLen(CHAR *buf, USHORT lenOfBuf, 
	    AsnTag *tag, USHORT *lenOfTag );
static IR_STATUS CsCf_FindIdefApdu(CS_SOCK_ITEM *);

/****************************************************************************
 * FUNCTION: CsCf_Poll
 *
 * PURPOSE:
 *	Poll all associations for read, write and exceptions.
 *
 * INPUT:
 *	CS_COMM_EVENT		- communication event information.
 *
 *
 * OUTPUT:
 *	IR_COMM_EVENT_STATUS
 *
 *
 * NOTES:
 *
 ***************************************************************************/

IR_COMM_EVENT_STATUS CsCf_Poll(IR_ASSOC_ID assocId, CS_SOCK_EVENT event)
/*
 * This function will return an IR_COMM_EVENT_STATUS indicating one of the
 * PLI event options only. 
 */
{
	CS_SOCK_ITEM 		*sockItem;
#ifdef WINPATHWAY
	int			writeFds;
	int			readFds;
	int			exceptFds;
#else 
	fd_set			writeFds;
	fd_set			readFds;
	fd_set			exceptFds;
#endif
	int			nFound;
	IR_COMM_EVENT_STATUS	retStatus;
	struct timeval		timeout;

	/* Get the association's state information. */
	sockItem = CsCf_AssocGet(assocId);
	if (!sockItem)
	{
		CsCf_ReportError(IR_FILE, CS_ERR_NO_SOCK_ITEM, NO_STRING,
		    (int)assocId);
		return(IR_COMM_DATA_ERROR);
	}

	/* Default return status. */
	retStatus = IR_COMM_DATA_NO_DATA;

	/* Check for association to able to handle events */
	if (sockItem->state != CS_SOCK_ST_OPER)
	{
		/* 
		 * No need to poll this sucker. It is not in a valid state.
		 */
		CS_LOG_PRINTS(IR_FILE, "Polling assoc that is not in oper state\n");
		return(retStatus);
	}

	FD_ZERO(&writeFds);
	FD_ZERO(&readFds);
	FD_ZERO(&exceptFds);

	switch (event)
	{
	case CS_SOCK_EV_RECV:
		FD_SET(sockItem->socketFd, &readFds);
		break;
	case CS_SOCK_EV_SEND:
		FD_SET(sockItem->socketFd, &writeFds);
		break;
	case CS_SOCK_EV_LOST_CONNECT:
		FD_SET(sockItem->socketFd, &readFds);
		FD_SET(sockItem->socketFd, &exceptFds);
		break;
	default:
		/* Polled event */
		FD_SET(sockItem->socketFd, &exceptFds);
		FD_SET(sockItem->socketFd, &writeFds);
		FD_SET(sockItem->socketFd, &readFds);
		break;
	}

	/* Wait 10 micro seconds */
	timeout.tv_sec = 0;
	timeout.tv_usec = 10;

	nFound = select( sockItem->socketFd+1, &readFds, 
	    &writeFds, &exceptFds, &timeout);

	if (nFound < 0 )
	{
		CsCf_ReportError(IR_FILE, CS_ERR_SELECT, "Select Error",
		    CsCf_CommErrorNo());
	}
	else if (nFound > 0)
	{
		if (FD_ISSET(sockItem->socketFd, &exceptFds))
		{
			/*
			 * The peer has aborted. P-Abort.
			 */
			sockItem->state = CS_SOCK_ST_ABORT;
			CsCf_Notify(assocId, CS_SOCK_EV_LOST_CONNECT);
			retStatus = IR_COMM_DATA_NO_DATA;
			CS_LOG_PRINTS(IR_FILE, "P_Abort\n");
		}
		else
		{
			/* If something to send then send it */
			if (FD_ISSET(sockItem->socketFd, &writeFds))
			{
				/*
				 * Check to see if something to 
				 * send. Otherwise, ignore it.
				 */
				if (RESC_LIST_SIZE(&sockItem->sendList) > 0)
				{
					retStatus = CsCf_SendData(sockItem);
				} 
			}

			/* If something to read then read it. */
			if (FD_ISSET(sockItem->socketFd, &readFds))
			{
				retStatus = CsCf_ReadData(sockItem);
			}
		}
	}
	return(retStatus);
}

/****************************************************************************
 * FUNCTION: CsCf_SendData
 *
 * PURPOSE:
 *	This function is used to send data out on a socket.
 *
 * INPUT:
 *	CS_SOCK_ITEM		- socket Item.
 *
 * OUTPUT:
 *	IR_COMM_DATA_NO_DATA	- always.
 *
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

IR_COMM_EVENT_STATUS CsCf_SendData(CS_SOCK_ITEM *sockItem)
/*
 * This function will return an IR_COMM_DATA_NO_DATA status. 
 */
{
	CS_BUFFER	*sendBuf;
	int 		numSent;
	int		numToSend;
	SHORT		errorCode;

	if (RESC_LIST_SIZE(&sockItem->sendList) == 0)
	{
#ifdef WINSOCKET
		/* Remove WRITE to select option */
		if (WSAAsyncSelect((SOCKET)sockItem->socketFd,
		    (HWND)sockItem->assocId,
		    (unsigned int)CS_SOCK_EV_RWC,
		    (long)(FD_READ|FD_CLOSE)) != 0)
		{
			   errorCode = WSAGetLastError();
			CS_LOG_PRINTV(IR_FILE, 
			   "WSAAsynSelect Error. Error = %d\n",
			   errorCode);
			sockItem->state = CS_SOCK_ST_ABORT;
			CsCf_Notify(sockItem->assocId, 
			    CS_SOCK_EV_ERROR_ABORT);
		}
#endif
		return(IR_COMM_DATA_NO_DATA);
	}

	RESC_LIST_SET_FIRST(&sockItem->sendList);
	sendBuf = RESC_LIST_GET(&sockItem->sendList);

	numToSend = sendBuf->bufSize - sendBuf->numSent;
	numSent = send (sockItem->socketFd, 
	    (char *)&sendBuf->buf[sendBuf->numSent], numToSend, 0);

	if (numSent == SOCKET_ERROR)
	{
		CsCf_ReportError(IR_FILE, CS_ERR_IRC_SEND_ERROR, 
		    "Error sending pdata via socket",
		    CsCf_CommErrorNo());
		sockItem->state = CS_SOCK_ST_ABORT;
		CsCf_Notify(sockItem->assocId, CS_SOCK_EV_ERROR_ABORT);
		return(IR_COMM_DATA_NO_DATA);
	}

	if (numSent >= 0) 
	{
		CS_LOG_PRINTV(IR_FILE, "Sent %d bytes ", numSent);
		CS_LOG_PRINTV("", "on assoc %d\n", sockItem->assocId);
		if (numSent != numToSend)
		{
			/* Adjust offsets in buffer. */
			sendBuf->numSent += numSent;
			CsCf_Notify(sockItem->assocId, CS_SOCK_EV_SEND);
		}
		else
		{
			/* Shift next buffer to top of list. */
			RescList_Delete(IR_FILE, &sockItem->sendList, 0);
			CsCf_BuffFree(sendBuf);
		}
#ifndef WINSOCKET
		/* Event for next send event */
		CsCf_Notify(sockItem->assocId, CS_SOCK_EV_SEND);
#endif
				  
	}
	return(IR_COMM_DATA_NO_DATA);
}
	    
/****************************************************************************
 * FUNCTION: CsCf_ReadData
 *
 * PURPOSE:
 *	This function is used to read data from a socket.
 *
 * INPUT:
 *	CS_SOCK_ITEM		- socket Item.
 *
 * OUTPUT:
 *	IR_COMM_DATA_NO_DATA	- always.
 *	IR_COMM_DATA_APDU	- apdu.
 *
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

IR_COMM_EVENT_STATUS CsCf_ReadData(CS_SOCK_ITEM *sockItem)
/*
 * This function will return an IR_COMM_DATA_NO_DATA or 
 * IR_COMM_DATA_APDU status. 
 */
{
	CS_BUFFER		*readBuf;
	CS_BUFFER		*lastBuf;
	int 			numRead;
	IR_COMM_EVENT_STATUS	retStatus;

	retStatus = IR_COMM_DATA_NO_DATA;

	/* Allocate a buffer to read into. */
	readBuf = CsCf_BuffAlloc();
	numRead = recv(sockItem->socketFd, readBuf->buf, CS_MAX_RD_BUF_SIZE, 0);

	/* Check for data read. */
	if (numRead <= 0)
	{
		/*
		 * The peer has aborted. P-Abort.
		 */
		sockItem->state = CS_SOCK_ST_ABORT;
		CsCf_Notify(sockItem->assocId, CS_SOCK_EV_LOST_CONNECT);
		CsCf_BuffFree(readBuf);
		CS_LOG_PRINTS(IR_FILE, "P_Abort\n");
		retStatus = IR_COMM_DATA_NO_DATA;
		return(retStatus);
	}

	CS_LOG_PRINTV(IR_FILE, "Read (%d) bytes from socket ", numRead);
	CS_LOG_PRINTV("", "on assoc %d\n", sockItem->assocId);

	readBuf->bufSize = numRead;

	/* Attach this buffer to the end of the list. */
	RescList_Append(IR_FILE, &sockItem->readList, readBuf, NULL, NULL);

	/*
	 * Determine whether an apdu has been completely read.
	 * (ie. An APDU may be spread over multiple buffers.
	 */
	retStatus = checkForAPDU(sockItem);

	return(retStatus);
}

/****************************************************************************
 * LOCAL FUNCTION: checkForAPDU
 *
 * PURPOSE:
 *	This function is used to check for an APDU to be complete.
 *
 * INPUT:
 *	CS_SOCK_ITEM		- pointer to assoc info containing byte stream.
 *
 * OUTPUT:
 *	IR_COMM_EVENT_STATUS	- returns no data or APDU.
 *
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

static IR_COMM_EVENT_STATUS checkForAPDU(CS_SOCK_ITEM *sockItem)
{
	CS_BUFFER		*tmpBuf;
	CS_BUFFER		*firstBuf;
	USHORT			sizeOfAPDU;
	USHORT			numTagLen;
	AsnTag			tag;
	USHORT			intermCount;
	IR_COMM_EVENT_STATUS	retStatus;
	
	if (RESC_LIST_SIZE(&sockItem->readList) == 0)
	{
		return(IR_COMM_DATA_NO_DATA);
	}

	RESC_LIST_SET_FIRST(&sockItem->readList);
	firstBuf = RESC_LIST_GET(&sockItem->readList);

	if (firstBuf->bufState == CS_BUF_ST_EMPTY)
	{
		/* 
		 * Decode the buffer and check for a tag and length values.
		 * If a LEN_ERROR is returned then the a buffer over-run
		 * was most likely detected. This will only occur if the
		 * target is really screwed up.
		 */
		numTagLen = apduTagLen( firstBuf->buf, firstBuf->bufSize,
	 	    &tag, &sizeOfAPDU);
		if (numTagLen == LEN_ERROR)
		{
			CS_LOG_PRINTS(IR_FILE, "ApduTagLen error.\n");
			sockItem->state = CS_SOCK_ST_ABORT;
			CsCf_Notify(sockItem->assocId, 
			    CS_SOCK_EV_ERROR_ABORT);
			retStatus = IR_COMM_DATA_NO_DATA;
			return(retStatus);
		}

		/* Start of new apdu */
		firstBuf->bufState |= CS_BUF_ST_START_APDU;
		
		if (sizeOfAPDU == (USHORT)INDEFINITE_LEN)
		{
			firstBuf->bufState |= CS_BUF_ST_IDEF_APDU;
			/* Must be indefinite length. Yikes! */
			CS_LOG_PRINTV(IR_FILE, 
			    "Read Indefinite Length Tag %2lx in CsCf_CheckForAPDU.\n", tag);
		}
		else
		{	
			sizeOfAPDU += numTagLen;
			firstBuf->sizeOfAPDU = sizeOfAPDU;
			firstBuf->bufState |= CS_BUF_ST_DEF_APDU;
		}
	}
	
	/* 
	 * Check for end of APDU. It may span many buffers. 
	 * May have to Chain buffers to make one APDU.
	 * This uses the definite length encoded rules only.
	 */
	retStatus = IR_COMM_DATA_NO_DATA;
	if (firstBuf->bufState & CS_BUF_ST_DEF_APDU)
	{
		/* Check the length of the buffers */
		intermCount = firstBuf->bufSize;
		tmpBuf = firstBuf;
		while ((intermCount < firstBuf->sizeOfAPDU) &&
		    (!RESC_LIST_IS_LAST(&sockItem->readList)))
		{
			RESC_LIST_SET_NEXT(&sockItem->readList);
			tmpBuf = RESC_LIST_GET(&sockItem->readList);
			intermCount += tmpBuf->bufSize;
		}

		/* Check for complete APDU */
		if (intermCount == firstBuf->sizeOfAPDU)
		{
			CS_LOG_PRINTS(IR_FILE, "Built Recv Block.\n");
			tmpBuf->bufState |= CS_BUF_ST_END_APDU;
			retStatus = IR_COMM_DATA_APDU;
		}
		else if (intermCount > firstBuf->sizeOfAPDU)
		{
			/* More than one APDU. */
			tmpBuf->bufState |= CS_BUF_ST_END_APDU;

			/* Move the next APDU to another buffer */
			CS_LOG_PRINTV(IR_FILE, "HELP! MORE THAN ONE APDU. APDU size = %d\n", firstBuf->sizeOfAPDU);

			retStatus = IR_COMM_DATA_APDU;
		}
		else
		{
			/* Still build the buffer chain. */
			CS_LOG_PRINTS(IR_FILE, "Still building the chain.\n");
			retStatus = IR_COMM_DATA_NO_DATA;
		}
	}
	else
	{
		/* Build indefinite length chain */
		if (CsCf_FindIdefApdu(sockItem) == IR_FAIL)
		{
			/* Major problem in IDEF apdu */
			CS_LOG_PRINTS(IR_FILE, 
			    "Failed Idef apdu chain building\n");
			sockItem->state = CS_SOCK_ST_ABORT;
			CsCf_Notify(sockItem->assocId, 
			    CS_SOCK_EV_ERROR_ABORT);
			retStatus = IR_COMM_DATA_NO_DATA;
		}
		else
		{
			RESC_LIST_SET_LAST(&sockItem->readList);
			firstBuf = RESC_LIST_GET(&sockItem->readList);
			if (firstBuf->bufState & CS_BUF_ST_END_APDU)
			{
				retStatus = IR_COMM_DATA_APDU;
			}
			else
			{
				retStatus = IR_COMM_DATA_NO_DATA;
			}
		}
	}
	return(retStatus);
}

/****************************************************************************
 * LOCAL FUNCTION: apduTagLen
 *
 * PURPOSE:
 *	This function is used to calculate the length of an apdu.
 *
 * INPUT:
 *	CHAR *buf		- byte stream
 *
 * OUTPUT:
 *	AsnTag *tag		- apdu tag
 *	USHORT *length		- apdu tag length
 *
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

static USHORT apduTagLen(CHAR *buf, USHORT bufLen, AsnTag *tag, 
	USHORT *lenOfTag)
{
	AsnLen			bytesDecoded;
	AsnLen			length;
	jmp_buf			env;
	int			val;
	SBuf			dataBuf;


	length = 0;
	SBufInstallData(&dataBuf, buf, bufLen);
	/* Set jmp vector in case decode detects a buffer over run */
	if ((val = setjmp(env)) == 0)
	{
		bytesDecoded = 0;
    		*tag = BDecTag(&dataBuf, &bytesDecoded, env);
		if (*tag == EOC_TAG_ID)
		{
			BDEC_2ND_EOC_OCTET(&dataBuf, &bytesDecoded, env)
		}
		else
		{
    			length = BDecLen(&dataBuf, &bytesDecoded, env);
		}
	}
	else
	{
		/* Decode error. Most likely a buffer overflow. */
		return(LEN_ERROR);
	}
	*lenOfTag = (USHORT)length;
	return((USHORT)bytesDecoded);
}

static IR_STATUS CsCf_FindIdefApdu(CS_SOCK_ITEM *sockItem)
{
	CHAR	*apduBuf;
	ULONG	apduBufSize;
	CS_BUFFER	*firstBuf;
	CS_BUFFER	*tmpBuf;
	SHORT		i;
	SHORT		index;
	AsnTag		tag;
	USHORT		tagLen;
	USHORT		bytesDecoded;
	SHORT		idefCnt;

	RESC_LIST_SET_FIRST(&sockItem->readList);
	firstBuf = RESC_LIST_GET(&sockItem->readList);

	apduBufSize = (RESC_LIST_SIZE(&sockItem->readList) * 
	    firstBuf->bufMallocSize);
	if (apduBufSize > (ULONG)IR_MAX_MALLOC_SIZE)
	{
		/* Cannot malloc a buffer this size */
		CS_LOG_PRINTV(IR_FILE, "Cannot malloc block of %ld size\n",
		    apduBufSize);
		return(IR_FAIL);
	}

	apduBuf = (CHAR *)RescIR_Malloc(IR_FILE, apduBufSize);

	/* Copy all buffers in chain into this buffer */
	index = 0;
	RESC_LIST_SET_FIRST(&sockItem->readList);
	for (i = 0; i < RESC_LIST_SIZE(&sockItem->readList); i++)
	{

		tmpBuf = RESC_LIST_GET(&sockItem->readList);
		memcpy(&apduBuf[index], tmpBuf->buf, tmpBuf->bufSize);
		index = index + tmpBuf->bufSize;
		RESC_LIST_SET_NEXT(&sockItem->readList);
	}
	apduBufSize = (ULONG)(index);

	/* Scan entire buffer looking for a complete apdu */
	index = 0;
	idefCnt = 0;
	do 
	{
		bytesDecoded = apduTagLen(&apduBuf[index], 
		    (USHORT)((USHORT)(apduBufSize+1) - index), &tag, &tagLen);
		index = index + bytesDecoded;
		if (tag == EOC_TAG_ID)
		{
			idefCnt--;
		}
		else if (tagLen == (USHORT)INDEFINITE_LEN)
		{
			idefCnt++;
		}
		else
		{
			index = index + tagLen;
		}
	} while (((ULONG)index < apduBufSize) && (idefCnt > 0));

	if (idefCnt == 0)
	{
		/* Found entire apdu */
		tmpBuf->bufState |= CS_BUF_ST_END_APDU;
		firstBuf->sizeOfAPDU = (USHORT)index+1;
		CS_LOG_PRINTV(IR_FILE, 
		    "Detected end of indefinite length apdu at %d\n", index+1);
	}
	else
	{
		CS_LOG_PRINTV(IR_FILE, 
		    "Still looking for endof APDU. Length = %d\n", index+1);
	}

	RescIR_Free(IR_FILE, (CHAR *)apduBuf);
	return(IR_SUCCESS);
}
