
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/***************************************************************************
 * UNIT: assoc.c
 *
 * DESCRIPTION: This unit contains the functionality required by the Origin
 *	Kernel ACSE to connect, release and abort associations.
 *
 * CONTENTS: IRK_Association
 *	     OkAcse_Association
 *	     OkAcse_Terminate
 *	     OkAcse_AbortAssoc
 *	     OkAcse_AAbort
 *	     OkAcse_Release
 *	     OkAcse_StopConnectWait
 *	     OkAcse_ARelease
 *
 * NOTES: 
 *	Nothing of note.
 *
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
***************************************************************************/
#include <stdio.h>
#include <ir.h>
#include <ir_ali.h>
#include <ir_pli.h>
#include <resc.h>
#include <ok/ok.h>


static WAIT_FOR_CON_TYPE *Acse_waitForConListPtr;
static WAIT_FOR_CON_TYPE *lastWaitForConPtr = NULL;
IR_STATUS OkAcse_StopConnectWait(IR_ASSOC_ID assocId);

/****************************************************************************
 * FUNCTION: IRK_Association
 *
 * PURPOSE: Support the A-Associate service.
 *
 * INPUT: 
 *	IRK_ASSOC_REQ_TYPE	reqType
 *	IRK_ASSOC_REQ		*request
 *	IRK_ASSOC_REPLY		*reply
 *
 * OUTPUT: IR_STATUS
 *
 * NOTES:
 *	Nothing of Note.
 *
 ***************************************************************************/
IR_STATUS IRK_Association(
 	IRK_ASSOC_REQ		*request,
 	IRK_ASSOC_REPLY		*reply)
{
	IR_STATUS	  status;
	WAIT_FOR_CON_TYPE *waitForConPtr;
	IRC_ASSOC_REQ		ircRequest;


	if (!OkCf_IsIRKInitialized())
	{
		OkCf_UpdateErrorReport(IR_ERROR_WARNING, 
		    IRK_PLI_ERROR, IR_FILE);
		return(IR_FAIL);
	}
	
	/* Check that association does not already exist, or is
	   not already waiting for a connection. */
	status = OkAse_CheckAssoc(request->assocId);
	if (status != IR_FAIL)
	{
		/* Error - association already exists. */
		OkCf_UpdateErrorReport(IR_ERROR_WARNING, 
		    IRK_ASSOC_ALREADY_EXISTS, IR_FILE);
		return (IR_FAIL);
	}
	waitForConPtr = Acse_waitForConListPtr;
	while ((waitForConPtr != NULL) && 
	    (waitForConPtr->assocId != request->assocId))
	{
		waitForConPtr = waitForConPtr->nextAssocPtr;
	}
	if (waitForConPtr != NULL) 
	{
		/* Error - association already waiting for connection. */
		OkCf_UpdateErrorReport(IR_ERROR_WARNING, 
		    IRK_ASSOC_ALREADY_EXISTS, IR_FILE);
		return (IR_FAIL);
	}

	RESC_ASN_NO_MALLOC(&ircRequest.hostName);
	RESC_ASN_INIT_VS(&ircRequest.hostName);
	RESC_ASN_NO_MALLOC(&ircRequest.hostAddr);
	RESC_ASN_INIT_VS(&ircRequest.hostAddr);
	RescASN_CopyVS(IR_FILE, &ircRequest.hostName, &request->hostName);
	RescASN_CopyVS(IR_FILE, &ircRequest.hostAddr, &request->hostAddr);
	ircRequest.portNum = request->portNum;
	ircRequest.assocId = request->assocId;

	/* Invoke PLI primitive to connect association. */
	status = IRC_Connect(&ircRequest);
	if (status == IR_FAIL)
	{
		OkCf_UpdateErrorReport(IR_ERROR_WARNING, 
		    IRK_PLI_ERROR, IR_FILE);
		RescASN_DestroyVS(IR_FILE, &ircRequest.hostName);
		RescASN_DestroyVS(IR_FILE, &ircRequest.hostAddr);
		return (IR_FAIL);
	}
	RescASN_DestroyVS(IR_FILE, &ircRequest.hostName);
	RescASN_DestroyVS(IR_FILE, &ircRequest.hostAddr);

	/* Create a waiting for connect table. */
	waitForConPtr = (WAIT_FOR_CON_TYPE *)RescIR_Malloc( 
	    IR_FILE, sizeof(WAIT_FOR_CON_TYPE));
	if (waitForConPtr == NULL)
	{
		/* Error - unable to allocate memory */
		OkCf_UpdateErrorReport(IR_ERROR_FATAL, 
		    IRK_MEMORY_ERROR, IR_FILE);
		return (IR_FAIL);
	}
	if (Acse_waitForConListPtr == NULL)
	{
		Acse_waitForConListPtr = waitForConPtr;
	}
	else
	{
		lastWaitForConPtr->nextAssocPtr = waitForConPtr;
	}
	lastWaitForConPtr = waitForConPtr;
	waitForConPtr->assocId = request->assocId;
	waitForConPtr->nextAssocPtr = NULL;

	return (IR_SUCCESS);
}

/****************************************************************************
 * FUNCTION: OkAcse_Association
 *
 * PURPOSE: Create and initialize the internal structures required to
 *	represent an association.
 *
 * INPUT: 
 *	IR_ASSOC_ID	assocId
 *
 * OUTPUT: IR_STATUS
 *
 * NOTES:
 *	Nothing of Note.
 *
 ***************************************************************************/
IR_STATUS OkAcse_Association(
 	IR_ASSOC_ID	assocId)
{
	IR_STATUS	status;

	/* Create a state table. */
	status = OkAcse_StopConnectWait(assocId);
	if (status == IR_FAIL)
	{
		/* Error - Association not waiting for connection. */
		OkCf_UpdateErrorReport(IR_ERROR_WARNING, 
		    IRK_PROTOCOL_ERROR, IR_FILE);
		return (IR_FAIL);
	}
	status = OkAse_CreateAssoc(assocId);
	if (status == IR_FAIL)
	{
		return (IR_FAIL);
	}

	return (IR_SUCCESS);
}

/****************************************************************************
 * FUNCTION: OkAcse_Terminate 
 *
 * PURPOSE: Clean up all internal association structures.
 *
 * INPUT: 
 *	nothing
 *
 * OUTPUT: IR_STATUS
 *
 * NOTES:
 *	Nothing of Note.
 *
 ***************************************************************************/
IR_STATUS OkAcse_Terminate()
{
	IR_STATUS	  status;
	WAIT_FOR_CON_TYPE *waitForConPtr;

	/* Delete all waiting for response tables. */
	while (Acse_waitForConListPtr != NULL)
	{
		waitForConPtr = Acse_waitForConListPtr;
		Acse_waitForConListPtr = Acse_waitForConListPtr->nextAssocPtr;
		waitForConPtr = (WAIT_FOR_CON_TYPE *)RescIR_Free
		    (IR_FILE, (CHAR *)waitForConPtr);
	}

	/* Delete all state tables. */
	status = OkAse_DeleteAllAssocs();
	return (status);
}

/****************************************************************************
 * FUNCTION: OkAcse_AbortAssoc 
 *
 * PURPOSE: Clean up the internal structures of an association.
 *
 * INPUT: 
	IR_ASSOC_ID assocId;
 *
 * OUTPUT: IR_STATUS
 *
 * NOTES:
 *	Nothing of Note.
 *
 ***************************************************************************/
IR_STATUS OkAcse_AbortAssoc(
	IR_ASSOC_ID assocId)
{

	/* Stop wait for association connection. */
	OkAcse_StopConnectWait(assocId);

	/* Delete association state table. */
	OkAse_DeleteAssoc(assocId);
	return (IR_SUCCESS);
}

/****************************************************************************
 * FUNCTION: OkAcse_AAbort 
 *
 * PURPOSE: Support A-Abort service.
 *
 * INPUT: 
	IR_ASSOC_ID assocId;
 *
 * OUTPUT: IR_STATUS
 *
 * NOTES:
 *	Nothing of Note.
 *
 ***************************************************************************/
IR_STATUS OkAcse_AAbort(
	IR_ASSOC_ID assocId)
{
	IR_STATUS	status;

	/* Invoke PLI primitive to abort an association. */
	status = IRC_AbortAssoc(assocId);
	if (status == IR_FAIL)
	{
		OkCf_UpdateErrorReport(IR_ERROR_WARNING, 
		    IRK_PLI_ERROR, IR_FILE);
		return (IR_FAIL);
	}

	return (IR_SUCCESS);
}

/****************************************************************************
 * FUNCTION: OkAcse_ReleaseAssoc 
 *
 * PURPOSE: Clean up internal structures for an association.
 *
 * INPUT: 
	IR_ASSOC_ID assocId;
 *
 * OUTPUT: IR_STATUS
 *
 * NOTES:
 *	Nothing of Note.
 *
 ***************************************************************************/
IR_STATUS OkAcse_ReleaseAssoc(
	IR_ASSOC_ID assocId)
{
	/* Stop wait for association connection. */
	OkAcse_StopConnectWait(assocId);

	/* Delete association state table. */
	OkAse_DeleteAssoc(assocId);
	return (IR_SUCCESS);
}

/****************************************************************************
 * FUNCTION: OkAcse_StopConnectWait 
 *
 * PURPOSE: Remove association from wait for connection list.
 *
 * INPUT: 
	IR_ASSOC_ID assocId;
 *
 * OUTPUT: IR_STATUS
 *
 * NOTES:
 *	Nothing of Note.
 *
 ***************************************************************************/
IR_STATUS OkAcse_StopConnectWait(
	IR_ASSOC_ID assocId)
{
	WAIT_FOR_CON_TYPE *waitForConPtr;
 	WAIT_FOR_CON_TYPE *prevConPtr;

	/* Delete association from waiting for connection list. */
	prevConPtr = NULL;
	waitForConPtr = Acse_waitForConListPtr;
	while ((waitForConPtr != NULL) && (waitForConPtr->assocId != assocId))
	{
		prevConPtr = waitForConPtr;
		waitForConPtr = waitForConPtr->nextAssocPtr;
	}
	if (waitForConPtr == NULL)
	{
		/* association not waiting. Error */
		return (IR_FAIL);
	}
	if (prevConPtr != NULL)
	{
		/* link previous to next */
		prevConPtr->nextAssocPtr = waitForConPtr->nextAssocPtr;
	}
	if (waitForConPtr == lastWaitForConPtr)
	{
		/* Set last pointer to previous entry */
		lastWaitForConPtr = prevConPtr;
	}

	if (waitForConPtr == Acse_waitForConListPtr)
	{
		/* Set first pointer to next */
		Acse_waitForConListPtr = waitForConPtr->nextAssocPtr;
	}

	/* Free this sucker! */
	RescIR_Free (IR_FILE, (CHAR *)waitForConPtr);
	
	return (IR_SUCCESS);
}

/****************************************************************************
 * FUNCTION: OkAcse_ARelease 
 *
 * PURPOSE: Support A-Release service.
 *
 * INPUT: 
	IR_ASSOC_ID assocId;
 *
 * OUTPUT: IR_STATUS
 *
 * NOTES:
 *	Nothing of Note.
 *
 ***************************************************************************/
IR_STATUS OkAcse_ARelease(
	IR_ASSOC_ID assocId)
{
	IR_STATUS	status;

	/* Invoke PLI primitive to release association. */
	status = IRC_Release(assocId);
	if (status == IR_FAIL)
	{
		OkCf_UpdateErrorReport(IR_ERROR_WARNING, 
		    IRK_PLI_ERROR, IR_FILE);
		return (IR_FAIL);
	}
	return (IR_SUCCESS);
}

void OkAcse_AssocDebug()
{
	WAIT_FOR_CON_TYPE *waitForConPtr;
 	WAIT_FOR_CON_TYPE *prevConPtr;

	/* List all associations currently waiting for a connection */
	prevConPtr = NULL;
	waitForConPtr = Acse_waitForConListPtr;
	while (waitForConPtr != NULL)
	{
		prevConPtr = waitForConPtr;
		waitForConPtr = waitForConPtr->nextAssocPtr;
	}
}
