

/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: OpAm_InitialService
 *
 * DESCRIPTION:
 *	This unit contains the functions used to service the initialize
 *	IR service apdu.
 *
 * CONTENTS:
 *	OpAm_InitService
 *	OpAm_InitServiceFree
 *
 * NOTES:
 *
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
/***************************************************************************/
#include <ir.h>
#include <memory.h>
#include <string.h>
#include <ir_api.h>
#include <ir_ali.h>
#include <resc.h>
#include <op/ir_op.h>
#include <asn_tool.h>

#define BITS_IN_BYTE		8
#define N_OPTION_BITS		7
#define SEARCH_MASK		0x80
#define PRESENT_MASK		0x40
#define DELETE_MASK		0x20
#define RSRC_RPT_MASK		0x10
#define TRIGGER_MASK		0x08
#define RSRC_CNTL_MASK		0x04
#define ACCESS_CNTL_MASK	0x02

static void OpAm_GetSnaccUserInfo(IRO_COMM_INIT_RESP *, InitializeResp *);

/****************************************************************************
 * FUNCTION:  OpAm_InitService
 *
 * PURPOSE:
 *	This function is used to process the initialize service apdu
 *	that was received from the target. If the assocId is valid then
 *	the init apdu data will be saved in the assoc entry and returned
 *	to the origin process. If the init failed then this function is
 *	responsible for terminating the association from the origin's 
 *	point of view.
 *
 * INPUT:
 *	IR_ASSOC_ID			- assoc list key
 *	APDU				- pointer to init info
 *
 * OUTPUT:
 *	IRO_COMM_EVENT_REPLY 		- pointer pointer to reply
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

IR_STATUS	OpAm_InitService(IR_ASSOC_ID assocId, APDU *apdu,
		    IRO_COMM_INIT_RESP *commResp)
{
	OP_ASSOC_ENTRY	*assocEntry;
	CHAR		currentByte;
	CHAR		*serviceMask;
	CHAR		*protocolMask;
	USHORT		protocol;
	CHAR		tmpBuf[100];
	CHAR		tmpBuf1[20];
	USHORT		byteCnt;
	USHORT		bitCnt;
	SHORT		stat;
	InitializeResp	*initResp;
	USHORT		i;
	UCHAR		bitMask;
	VISIBLE_STRING	*refId;

	/* Verify that the association exists */
	assocEntry = OpAm_AssocGet(assocId);
	if (assocEntry == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING, 
		    IRO_ASSOC_DOES_NOT_EXIST, IR_FILE);
		return (IR_FAIL);
	}

	/* Use a local initResp pointer */
	initResp = apdu->a.initResp;

	/* Verify the referenceId against the one stored in assocEntry */
	refId = RescASN_CreateVS(IR_FILE);
	RescASN_SnaccToIRVS(IR_FILE, refId, &initResp->referenceId);

	if (strncmp(refId->sPtr, assocEntry->wsRefId.sPtr, refId->len) != 0)
	{
		/* Reference Id does not correspond. Z3950 protocol no no! */
		CS_LOG_PRINTS(IR_FILE, "Mismatched referenceId on init.\n");
	}

	/* Destroy the refId tmp variable */
	RescASN_DestroyVS(IR_FILE, refId);
	/* Reset the assocEntry wsRefId */
	RescASN_DestroyVS(IR_FILE, &assocEntry->wsRefId);

	if (initResp->result != TRUE)
	{
		/* It failed */
		FreeAPDU(apdu);

		/* Abort the association */
		IRK_SendRequest((IRK_REQ_SERVICE_TYPE)IRK_ABORT_REQ_TYPE, 
		    (IR_ASSOC_ID)assocId, 
		    (IRK_SEND_REQUEST_REQ *)NULL);	
		return (IR_FAIL);
	}

	/* Get the response options */
	stat = ASN_BitsGet(&serviceMask, &byteCnt, &bitCnt, 
	    &initResp->options);
	/* Adjust the bitCnt to the total number of bits */
	bitCnt = (byteCnt * BITS_IN_BYTE) + bitCnt;

	/* 
	 * Check if the number of Option bits fit inside one byte.
	 * If not, then get option bits from first byte, set
	 * currentByte to serviceMask[1], get options bits, and
	 * repeat until all bytes have been processed.
	 */
	if (N_OPTION_BITS <= BITS_IN_BYTE)
	{
		/* Set all options to false */
		commResp->search = FALSE;
		commResp->present = FALSE;
		commResp->delSet = FALSE;
		commResp->resourceReport = FALSE;
		commResp->triggerResourceCtrl = FALSE;
		commResp->resourceCtrl = FALSE;
		commResp->accessCtrl = FALSE;

		/* Set only the options supported to true */
		currentByte = serviceMask[0];
		if (currentByte & SEARCH_MASK)
		{
			commResp->search = TRUE;
		}
		if (currentByte & PRESENT_MASK)
		{
			commResp->present = TRUE;
		}
		if (currentByte & DELETE_MASK)
		{
			commResp->delSet = TRUE;
		}
		if (currentByte & RSRC_RPT_MASK)
		{
			commResp->resourceReport = TRUE;
		}
		if (currentByte & TRIGGER_MASK)
		{
			commResp->triggerResourceCtrl = TRUE;
		}
		if (currentByte & RSRC_CNTL_MASK)
		{
			commResp->resourceCtrl = TRUE;
		}
		if (currentByte & ACCESS_CNTL_MASK)
		{
			commResp->accessCtrl = TRUE;
		}
	}
		
	/* Convert each protocol bit to a char representation */
	ASN_BitsGet(&protocolMask, &byteCnt, &bitCnt, 
	    &(apdu->a.initResp->protocolVersion));
	bitMask = (UCHAR)0x80;
	protocol = 0;
	tmpBuf[0] = EOS;
	for (i = 0; i < ((byteCnt * BITS_IN_BYTE) + bitCnt); i++)
	{
		protocol++;
		if (*protocolMask & bitMask)
		{
			if (tmpBuf[0] == EOS)
			{
				sprintf(tmpBuf1, "%1d", protocol);
			}
			else
			{
				sprintf(tmpBuf1, ",%1d", protocol);
			}
			strcat(tmpBuf, tmpBuf1);
		}
		bitMask = bitMask >> 1;
		if (bitMask == 0)
		{
			bitMask = (UCHAR)0x80;
			protocolMask++;
		}
	}
		
	RESC_ASN_NO_MALLOC(&commResp->protocolVersion);
	RescASN_AddVS(IR_FILE, &commResp->protocolVersion, tmpBuf,
		LENPLUSNULL(tmpBuf));


	/* Copy Message sizes */
	commResp->preferredMessageSize = 
	    (LONG)initResp->preferredMessageSize;
	commResp->maximumRecordSize =
	    (LONG)initResp->maximumRecordSize;

	/* Get the implementation Id */
	RESC_ASN_NO_MALLOC(&commResp->implementationId);
	RescASN_SnaccToIRVS(IR_FILE, &commResp->implementationId,
	    &initResp->implementationId);

	/* Get the implementation Name */
	RESC_ASN_NO_MALLOC(&commResp->implementationName);
	RescASN_SnaccToIRVS(IR_FILE, &commResp->implementationName,
	    &initResp->implementationName);

	/* Get the implementation version */
	RESC_ASN_NO_MALLOC(&commResp->implementationVersion);
	RescASN_SnaccToIRVS(IR_FILE, &commResp->implementationVersion,
	    &initResp->implementationVersion);

	/* Initialize the user information fields. */
	RESC_ASN_NO_MALLOC(&commResp->userInfo);
	RESC_ASN_INIT_OS(&commResp->userInfo);

	/* 
	 * If the user information field is present then convert the 
	 * supplied external into a visible string (this should work
	 * for all cases, but you never know with the way some targets
	 * have been implemented). 
	 */
	OpAm_GetSnaccUserInfo(commResp, initResp);
	RescUT_DebugVS(IR_FILE, &commResp->userInfo);

	/* Free the snacc apdu */
	FreeAPDU(apdu);
	RescIR_Free(IR_FILE, (CHAR *)apdu);

	/* Add a copy of the initialize response to the assoc entry */
	assocEntry->initResp = (IRO_COMM_INIT_RESP *)RescIR_Zmalloc(IR_FILE,
	    sizeof(IRO_COMM_INIT_RESP));
	if (assocEntry->initResp == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
		    IRO_MEMORY_ERROR, IR_FILE);
		return (IR_FAIL);
	}

	OpAm_AssocInitInfo(commResp, assocEntry->initResp);

	/* Change the association state from initialize to idle */
	assocEntry->state = OP_ASSOC_IDLE;

	return (IR_SUCCESS);
}

/****************************************************************************
 * FUNCTION:  OpAm_InitServiceFree
 *
 * PURPOSE:
 *	This function is used to free an IRO_COMM_INIT_RESP structure.
 *
 * INPUT:
 *	IRO_COMM_INIT_RESP		- pointer to init response
 *
 * OUTPUT:
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

void	OpAm_InitServiceFree(IRO_COMM_INIT_RESP *resp)
{
	RescASN_DestroyVS(IR_FILE, &resp->protocolVersion);
	RescASN_DestroyVS(IR_FILE, &resp->implementationId);
	RescASN_DestroyVS(IR_FILE, &resp->implementationName);
	RescASN_DestroyVS(IR_FILE, &resp->implementationVersion);
	RescASN_DestroyOS(IR_FILE, &resp->userInfo);

	if (RESC_ASN_IS_MALLOCED(resp))
	{
		RescIR_Free(IR_FILE, (CHAR *)resp);
	}
}

static void OpAm_GetSnaccUserInfo(IRO_COMM_INIT_RESP *resp, 
    InitializeResp *initResp)
{
	IR_EXTERNAL	*userInfo;
	CHAR		*cannedMsg;


	if (initResp->userInfoField == (UserInfoField *)NULL)
	{
		/* No user info supplied. */
		cannedMsg = "No user info supplied by target";
		RescASN_AddVS(IR_FILE, &resp->userInfo, cannedMsg,
		    LENPLUSNULL(cannedMsg));
		return;
	}

	/* Create the userInfo external structure. */
	userInfo = RescASN_CreateExternal(IR_FILE);

	/* Convert from snacc to local external structure. */
	RescASN_SnaccToIRExternal(IR_FILE, userInfo, initResp->userInfoField);

	/* Extract user info information into a visible string */
	if (userInfo->encType == IR_EXT_ENC_ALIGNED) 
	{
		RescASN_CopyOS(IR_FILE, &resp->userInfo, 
		    &userInfo->octetAligned);
	}
	else
	{
		cannedMsg = 
		    "Unknown external format in userInfo field of init response";
		RescASN_AddVS(IR_FILE, &resp->userInfo, cannedMsg,
		    LENPLUSNULL(cannedMsg));
	}
	RescASN_DestroyExternal(IR_FILE, userInfo);
}
