
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: OpAm Association Maintenance
 *
 * DESCRIPTION:
 *	This unit contains functions used for the maintenance of the
 *	association entries which are maintained for the entire life-span
 *	of an association.
 *
 * CONTENTS:
 *	OpAm_AssocInit
 *	OpAm_AssocCreate
 *	OpAm_AssocGet
 *	OpAm_AssocDelete
 *	local OpAm_AssocFind
 *	local OpAm_AssocDestroy
 *
 * NOTES:
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 *
/***************************************************************************/

#include <ir.h>
#include <ir_api.h>
#include <ir_ali.h>
#include <resc.h>
#include <resclist.h>
#include <op/ir_op.h>

IR_LIST	*OpAm_AssocList;

static SHORT OpAm_AssocFind(IR_ASSOC_ID assocId);
static void  OpAm_AssocDestroy(CHAR *srcFile, OP_ASSOC_ENTRY *assocPtr);

/****************************************************************************
 * FUNCTION:  OpAm_AssocInit
 *
 * PURPOSE:
 *	Initialize the association list to no entries and set all pointers
 *	to null.
 *
 * INPUT:
 *	none
 *
 * OUTPUT:
 *	none
 *
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

IR_STATUS OpAm_AssocInit()
{
	OpAm_AssocList = RescList_Create(IR_FILE);
	if (OpAm_AssocList == (IR_LIST *)NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL, IRO_MEMORY_ERROR,
		    IR_FILE);
		return (IR_FAIL);
	}
}


/****************************************************************************
 * FUNCTION:  OpAm_AssocCreate
 *
 * PURPOSE:
 *	Add a new entry to the end of the association list. If the given
 *	association id already exists then return a null pointer. Otherwise,
 *	return a pointer to the new entry.
 *
 * INPUT:
 *	IR_ASSOC_ID			- new assoc Id.
 *
 * OUTPUT:
 *	OP_ASSOC_ENTRY	*		- pointer to new entry.
 *
 *
 * NOTES:
 *	There are no notes. 
 *
 ***************************************************************************/

OP_ASSOC_ENTRY *OpAm_AssocCreate(IR_ASSOC_ID assocId)
{
	OP_ASSOC_ENTRY *assocPtr;

	assocPtr = (OP_ASSOC_ENTRY *)RescIR_Zmalloc(IR_FILE,
	    sizeof(OP_ASSOC_ENTRY));
	if (assocPtr == NULL)
	{
		return ((OP_ASSOC_ENTRY *)NULL);
	}

	RESC_ASN_MALLOCED(assocPtr);

	/* Set new parameters */
	assocPtr->assocId = assocId;
	assocPtr->state = OP_ASSOC_CONNECTING;
	assocPtr->initResp = NULL;
	assocPtr->resultNaming = FALSE;
	assocPtr->elementSetNaming = FALSE;
	assocPtr->userId = IR_UNKNOWN_ID_TYPE;
	assocPtr->targetId = IR_UNKNOWN_ID_TYPE;
	assocPtr->securityParameterType = 0;
	assocPtr->rsrcCntlRespReq = FALSE;
	

	RESC_ASN_NO_MALLOC(&assocPtr->hostName);
	RESC_ASN_NO_MALLOC(&assocPtr->address);
	RESC_ASN_NO_MALLOC(&assocPtr->localName);
	RESC_ASN_NO_MALLOC(&assocPtr->refId);
	RESC_ASN_NO_MALLOC(&assocPtr->wsRefId);
	RESC_ASN_INIT_VS(&assocPtr->hostName);
	RESC_ASN_INIT_VS(&assocPtr->address);
	RESC_ASN_INIT_VS(&assocPtr->localName);
	RESC_ASN_INIT_OS(&assocPtr->refId);
	RESC_ASN_INIT_VS(&assocPtr->wsRefId);

	RescList_Append(IR_FILE, OpAm_AssocList, assocPtr, 
	    OpAm_AssocDestroy, NULL);

	assocPtr->resultList = (IR_LIST *)NULL;
	assocPtr->delFunc = IR_UNKNOWN_ID_TYPE;

	return (assocPtr);
}

/****************************************************************************
 * FUNCTION:  OpAm_AssocGet
 *
 * PURPOSE:
 *	This function will search for the assocId in the list and return
 *	a pointer to the matching entry. If an entry does not exist for the
 *	requested assocId then return NULL.
 *
 * INPUT:
 *	IR_ASSOC_ID			- entry key
 *
 * OUTPUT:
 *	OP_ASSOC_ENTRY *		- pointer to assoc entry
 *
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

OP_ASSOC_ENTRY	*OpAm_AssocGet(IR_ASSOC_ID assocId)
{
	SHORT		pos;
	OP_ASSOC_ENTRY *assocPtr;

	pos = OpAm_AssocFind(assocId);

	if (pos == IR_UNKNOWN_ID_TYPE)
	{
		return((OP_ASSOC_ENTRY *)NULL);
	}

	RESC_LIST_SET_CUR(OpAm_AssocList, pos);

	assocPtr = RESC_LIST_GET(OpAm_AssocList);

	return (assocPtr);
}

/****************************************************************************
 * FUNCTION:  OpAm_AssocDelete
 *
 * PURPOSE:
 *	This function is used to remove an association entry from the
 *	association list.
 *
 * INPUT:
 *	IR_ASSOC_ID			- entry to remove
 *
 * OUTPUT:
 *	IR_STATUS
 *
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

IR_STATUS 	OpAm_AssocDelete(IR_ASSOC_ID assocId)
{
	SHORT	pos;

	pos = OpAm_AssocFind(assocId);

	if (pos == IR_UNKNOWN_ID_TYPE)
	{
		return(IR_SUCCESS);
	}

	/* Remove from link */
	RescList_Delete(IR_FILE, OpAm_AssocList, pos);

	return (IR_SUCCESS);
}

/****************************************************************************
 * FUNCTION:  OpAm_AssocListFree
 *
 * PURPOSE:
 *	Free assoc entries and assoc list
 *
 * INPUT:
 *	none
 *
 * OUTPUT:
 *	none
 *
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

void OpAm_AssocListFree()
{

	RescList_Destroy(IR_FILE, OpAm_AssocList);

}


/****************************************************************************
 * LOCAL FUNCTION:  OpAm_AssocFind
 *
 * PURPOSE:
 *	This local function searches for the assoc entry that corresponds
 *	to the assocId supplied. The list index of the matching entry
 *	is returned.
 *
 * INPUT:
 *	assocId
 *
 * OUTPUT:
 *	index into list. (IR_UNKNOWN_ID_TYPE) if not found.
 *
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

static SHORT	OpAm_AssocFind(IR_ASSOC_ID assocId)
{
	SHORT		i;
	OP_ASSOC_ENTRY	*assocPtr;

	if (RESC_LIST_SIZE(OpAm_AssocList) == 0)
	{
		return(IR_UNKNOWN_ID_TYPE);
	}

	RESC_LIST_SET_FIRST(OpAm_AssocList);
	for (i = 0; i < RESC_LIST_SIZE(OpAm_AssocList); i++)
	{
		assocPtr = RESC_LIST_GET(OpAm_AssocList);

		if (assocPtr->assocId == assocId)
		{
			/* This is the one. */
			break;
		}
		RESC_LIST_SET_NEXT(OpAm_AssocList);
	} 

	if (i == RESC_LIST_SIZE(OpAm_AssocList))
	{
		/* assocId not found in list */
		return(IR_UNKNOWN_ID_TYPE);
	}
	else
	{
		/* IR List counts start at zero. */
		return(i);
	}
}

/****************************************************************************
 * LOCAL FUNCTION:  OpAm_AssocDestroy
 *
 * PURPOSE:
 *	This local function will destroy an assoc entry. It will free all
 *	malloced data.
 *
 * INPUT:
 *	pointer to assoc entry.
 *
 * OUTPUT:
 *	void
 *
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

static void  OpAm_AssocDestroy(CHAR *srcFile, OP_ASSOC_ENTRY *assocPtr)
{

	if (assocPtr == (OP_ASSOC_ENTRY *)NULL)
	{
		return;
	}
	
	RescASN_DestroyVS(IR_FILE, &assocPtr->hostName);
	RescASN_DestroyVS(IR_FILE, &assocPtr->address);
	RescASN_DestroyVS(IR_FILE, &assocPtr->localName);
	RescASN_DestroyVS(IR_FILE, &assocPtr->refId);
	RescASN_DestroyVS(IR_FILE, &assocPtr->wsRefId);

	if (assocPtr->initResp != NULL)
	{
		OpAm_InitServiceFree(assocPtr->initResp);
	}

	if (assocPtr->resultList != NULL)
	{
		RescList_Destroy(IR_FILE, assocPtr->resultList);
	}
	assocPtr->resultList = (IR_LIST *)NULL;

	if (RESC_ASN_IS_MALLOCED(assocPtr))
	{
		RescIR_Free(IR_FILE, (CHAR *)assocPtr);
	}
}

BOOLEAN	OpAm_AssocServiceAllowed(SHORT serviceType, IR_ASSOC_ID assocId)
{
	OP_ASSOC_ENTRY	*assocPtr;
	BOOLEAN		supported;
	IRO_COMM_INIT_RESP *initResp;

	if ((assocPtr = OpAm_AssocGet(assocId)) == NULL)
	{
		/* Obviously the option is not supported */
		return(FALSE);
	}

	if (assocPtr->initResp == NULL)
	{
		/* Init response not yet received. */
		return(FALSE);
	}
	else
	{
		initResp = assocPtr->initResp;
	}

	supported = FALSE;
	switch (serviceType)
	{
	case OPAM_SEARCH:
		supported = initResp->search;
		break;
	case OPAM_PRESENT:
		supported = initResp->present;
		break;
	case OPAM_DELETE:
		supported = initResp->delSet;
		break;
	case OPAM_ACCESS:
		supported = initResp->accessCtrl;
		break;
	case OPAM_RSRC_CTRL:
		supported = initResp->resourceCtrl;
		break;
	case OPAM_RSRC_REPORT:
		supported = initResp->resourceReport;
		break;
	case OPAM_TRIGGER:
		supported = initResp->triggerResourceCtrl;
		break;
	case OPAM_RESULT_NAMING:
		supported = assocPtr->resultNaming;
		break;
	case OPAM_ELEM_SET_NAMING:
		supported = assocPtr->elementSetNaming;
		break;
	default:
		supported = FALSE;
		break;
		
	}

	return(supported);
}
