

/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: OpArc_AccessService
 *
 * DESCRIPTION:
 *	This unit contains the functions used to service the access 
 *	control IR service apdu.
 *
 * CONTENTS:
 *	IRO_AccessControl
 *	OpArc_AccessService
 *	OpArc_AccessServiceFree

 * NOTES:
 *
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
/***************************************************************************/
#include <ir.h>
#include <ir_api.h>
#include <ir_ali.h>
#include <resc.h>
#include <op/ir_op.h>
#include <asn_tool.h>
#include <ir_oid.h>


/****************************************************************************
 * FUNCTION:  IRO_AccessControl
 *
 * PURPOSE:
 *	This function creates an access control response apdu and then
 *	invokes the IRK interface to send it.
 *
 * INPUT:
 *	IRO_ACCESS_CNTRL_RESP		- access control struct.
 *
 * OUTPUT:
 *	IR_STATUS			
 *
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

IR_STATUS	IRO_AccessControl(IR_USER_ID userId, IRO_ACCESS_CNTRL_REQ *resp)
{
	OP_ASSOC_ENTRY	*assocEntry;
	IR_STATUS	status;
	IRK_SEND_REQUEST_REQ	accResp;
	OP_SESSION_INFO	*sessionInfo;
	IR_EXTERNAL	*tmpExt;

	/* Check if user exists */
	if ((sessionInfo = OpTm_UserSessionInfo(userId)) == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING, 
		    IRO_UNKNOWN_USER_ID, IR_FILE);
		return (IR_FAIL);
	}

	/* Check if assoc exists */
	assocEntry = OpAm_AssocGet(resp->assocId);
	if (assocEntry == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING, 
		    IRO_ASSOC_DOES_NOT_EXIST, IR_FILE);
		return (IR_FAIL);
	}
	
	/* Create access response apdu */
	accResp.irApdu.choiceId = APDU_ACCESSCONTROLRESP;
	accResp.irApdu.a.accessControlResp = (AccessControlResp *)
	    RescIR_Zmalloc(IR_FILE, sizeof(AccessControlResp));

	if (accResp.irApdu.a.accessControlResp == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
		    IRO_MEMORY_ERROR, IR_FILE);
		FreeAPDU(&(accResp.irApdu));
		return (IR_FAIL);
	}
		
	/* Check if reference id accompanied the access request */
	if (assocEntry->refId.len > 0)
	{
		/* Return reference Id with response */
		RescASN_IRToSnaccOS(IR_FILE, 
		    &accResp.irApdu.a.accessControlResp->referenceId,
		    &assocEntry->refId);
		RescASN_DestroyVS(IR_FILE, &assocEntry->refId);
	}

	/* create access control resp choice structure */
	accResp.irApdu.a.accessControlResp->securityChallengeResponse = 
	    (AccessControlRespChoice *)RescIR_Zmalloc(
	    IR_FILE, sizeof(AccessControlRespChoice));
	if (accResp.irApdu.a.accessControlResp->securityChallengeResponse == NULL)
	{			
		OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
		    IRO_MEMORY_ERROR, IR_FILE);
		FreeAPDU(&(accResp.irApdu));
		return (IR_FAIL);
	}
	accResp.irApdu.a.accessControlResp->securityChallengeResponse->choiceId = 
	    assocEntry->securityParameterType;

	/* use same type as in access request */
	switch (assocEntry->securityParameterType)
	{
		case ACCESSCONTROLRESPCHOICE_OLDWAY:
			accResp.irApdu.a.accessControlResp->securityChallengeResponse->
			    a.oldway = ASN_OctsCreate();
			if (accResp.irApdu.a.accessControlResp->
			    securityChallengeResponse->a.oldway == NULL)
			{			
				OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
				    IRO_MEMORY_ERROR, IR_FILE);
				FreeAPDU(&(accResp.irApdu));
				return (IR_FAIL);
			}
			
			if (resp->accessResponse.len == 0)
			{
				OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
				    IRO_PROTOCOL_ERROR, IR_FILE);
				FreeAPDU(&(accResp.irApdu));
				return (IR_FAIL);
			}

			RescASN_IRToSnaccOS(IR_FILE,
			    accResp.irApdu.a.accessControlResp->
			    securityChallengeResponse->a.oldway,
			    &resp->accessResponse);

			break;
		case ACCESSCONTROLRESPCHOICE_NEWWAY:
			/* Convert the response into an external */
	 		accResp.irApdu.a.accessControlResp-> 
			    securityChallengeResponse->a.newway = (EXTERNAL *)
			    RescIR_Zmalloc(IR_FILE, sizeof(EXTERNAL));

			tmpExt = RescASN_CreateExternal(IR_FILE);

			RescASN_AddOI(IR_FILE, &tmpExt->direct,
			    OID_ACCESS_CTRL_SKL,
			    strlen(OID_ACCESS_CTRL_SKL));
			tmpExt->referenceType = IR_EXT_REF_DIRECT;
			tmpExt->encType = IR_EXT_ENC_ALIGNED;
			RescASN_AddOS(IR_FILE, &tmpExt->octetAligned,
			    resp->accessResponse.sPtr,
			    resp->accessResponse.len);

			RescASN_IRToSnaccExternal(IR_FILE,
		 	    accResp.irApdu.a.accessControlResp-> 
			    securityChallengeResponse->a.newway,
			    tmpExt);
			RescASN_DestroyExternal(IR_FILE, tmpExt);
			
			break;
		default:
			OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
			    IRO_PROTOCOL_ERROR, IR_FILE);
			FreeAPDU(&(accResp.irApdu));
			return (IR_FAIL);
	}

	/* send the access control response */
	status = IRK_SendRequest(IRK_ACCESS_RES_TYPE, resp->assocId, &accResp);
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
		    IRO_ALI_ERROR, IR_FILE);
		FreeAPDU(&(accResp.irApdu));
		return (IR_FAIL);
	}
	FreeAPDU(&(accResp.irApdu));
	return (IR_SUCCESS);
}

/****************************************************************************
 * FUNCTION:  OpArc_AccessService
 *
 * PURPOSE:
 *	This function is used to process the access control apdu 
 *	received from the target. The apdu information is
 *	copied to the IR return structure and then the apdu
 *	is destroyed.
 *
 * INPUT:
 *	IR_ASSOC_ID			- assoc list key
 *	APDU				- pointer to access info
 *
 * OUTPUT:
 *	IRO_COMM_EVENT_REPLY		- pointer pointer to reply
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

IR_STATUS	OpArc_AccessService(IR_ASSOC_ID assocId, APDU *apdu,
		    IRO_COMM_ACCS_CNTL_REQ *accessReq)
{
	OP_ASSOC_ENTRY		*assocEntry;
	AccessControlReqChoice	*secParm;
	IR_EXTERNAL		*ext;

	/* Check if assoc exists */
	assocEntry = OpAm_AssocGet(assocId);
	if (assocEntry == NULL)
	{
		CS_LOG_PRINTS(IR_FILE, 
		    "Received Access Control Request for unknown Assoc\n");
		OpCf_UpdateErrorReport(IR_ERROR_WARNING, 
		    IRO_ASSOC_DOES_NOT_EXIST, IR_FILE);
		FreeAPDU(apdu);
		return (IR_FAIL);
	}

	/* Make sure there is not an old assocId hangin around */
	RescASN_DestroyOI(IR_FILE, &assocEntry->refId);
	/* save reference id */
	RescASN_SnaccToIROS(IR_FILE, &assocEntry->refId,
	    &apdu->a.accessControlReq->referenceId);


	/* decode security challenge */
	secParm = apdu->a.accessControlReq->securityChallenge;
	assocEntry->securityParameterType = secParm->choiceId;

	switch (apdu->a.accessControlReq->securityChallenge->choiceId)
	{
		case (ACCESSCONTROLREQCHOICE_OLDWAY):
			RESC_ASN_NO_MALLOC(&accessReq->accessControlReq);
			RESC_ASN_INIT_OS(&accessReq->accessControlReq);
			if (secParm->a.oldway)
			{
				RescASN_SnaccToIROS(IR_FILE, 
				    &accessReq->accessControlReq,
				    secParm->a.oldway);
			}

			break;
		case (ACCESSCONTROLREQCHOICE_NEWWAY):
			/* Convert the snacc external to IR external */
			ext = RescASN_CreateExternal(IR_FILE);
			RescASN_SnaccToIRExternal(IR_FILE,
			     ext, secParm->a.newway);
			/* check the direct reference */
			RESC_ASN_NO_MALLOC(&ext->octetAligned);
			RESC_ASN_INIT_OS(&ext->octetAligned);
			if ((ext->referenceType == IR_EXT_REF_DIRECT) &&
			    (ext->direct.len > 0))
			{
				if (strncmp(ext->direct.sPtr,
				    OID_ACCESS_CTRL_SKL,
				    ext->direct.len) == 0)
				{
					RescASN_CopyOS(IR_FILE,
					    &accessReq->accessControlReq,
					    &ext->octetAligned);
				}
			}
			RescASN_DestroyExternal(IR_FILE, ext);
			break;
		default:
			OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
			    IRO_PROTOCOL_ERROR, IR_FILE);
			FreeAPDU(apdu);
			return (IR_FAIL);
	}

	FreeAPDU(apdu);
	return (IR_SUCCESS);
}


/****************************************************************************
 * FUNCTION:  OpArc_AccessServiceFree 
 *
 * PURPOSE:
 *	This function is used to free the data returned to the user
 *	application.
 *
 * INPUT: IRO_COMM_ACCS_CNTL_REQ	- pointer to access request
 *	
 *
 * OUTPUT:
 *
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/


void OpArc_AccessServiceFree( IRO_COMM_ACCS_CNTL_REQ *req)
{
	if (req)
	{
		RescASN_DestroyOS(IR_FILE, &req->accessControlReq);
		if (RESC_ASN_IS_MALLOCED(req))
		{
			RescIR_Free(IR_FILE, (CHAR *)req);
		}
	}
}
