

/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: OpArc_TriggerResourceControl
 *
 * DESCRIPTION:
 *	This unit provides functions to support the IRO trigger resource
 *	control request.
 *
 * CONTENTS:
 *	IRO_TriggerControl
 *
 * NOTES:
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 *
/***************************************************************************/
#include <ir.h>
#include <ir_api.h>
#include <ir_ali.h>
#include <resc.h>
#include <op/ir_op.h>
#include <asn_tool.h>
#include <ir_oid.h>


/****************************************************************************
 * FUNCTION:  IRO_TriggerControl
 *
 * PURPOSE:
 *	This function will take a user application supplied trigger
 *	control structure and convert it to an APDU trigger resource control
 * 	request structure. The irApdu will then be sent to the associated target.
 *
 * INPUT:
 *	IRO_TRIGGER_CNTRL_REQ 		- trigger control request
 *
 * OUTPUT:
 *	IR_STATUS
 *
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

IR_STATUS	IRO_TriggerControl(IR_USER_ID userId, 
	    	     IRO_TRIGGER_CNTRL_REQ *req)
{
	OP_ASSOC_ENTRY 	*assocEntry;
	IRK_SEND_REQUEST_REQ	sendReq;
	OP_SESSION_INFO	*sessionInfo;
	TriggerResourceReq	*trigReq;
	IR_STATUS 	status;

	/* Check if assoc exists */
	assocEntry = OpAm_AssocGet(req->assocId);
	if (assocEntry == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING,
		    IRO_ASSOC_DOES_NOT_EXIST, IR_FILE);
		return (IR_FAIL);
	}

	/* Verify the user Id */
	if ((sessionInfo = OpTm_UserSessionInfo(userId)) == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING, IRO_UNKNOWN_USER_ID,
		    IR_FILE);
		return(IR_FAIL);
	}

	/* Check to see whether the association is busy or not */
	if (assocEntry->state == OP_ASSOC_IDLE)
	{
		/* Can only send trigger if busy */
		OpCf_UpdateErrorReport(IR_ERROR_WARNING,
		    IRO_ASSOC_NOT_BUSY, IR_FILE);
		return(IR_FAIL);
	}

	/* Initialize APDU */
	/* Create trigger request apdu */
	sendReq.irApdu.choiceId = APDU_TRIGGERRESOURCEREQ;

	sendReq.irApdu.a.triggerResourceReq = (TriggerResourceReq *)
	    RescIR_Zmalloc(IR_FILE, sizeof(TriggerResourceReq));
	if (sendReq.irApdu.a.triggerResourceReq == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL,
		    IRO_MEMORY_ERROR, IR_FILE);
		return (IR_FAIL);
	}
	
	trigReq = sendReq.irApdu.a.triggerResourceReq;

	/* Get the current wsRefId to use for the reference Id */
	RescUT_ShortToVS(IR_FILE, &assocEntry->wsRefId, 
	    (SHORT)req->assocId);
	RescASN_IRToSnaccOS(IR_FILE, &trigReq->referenceId, 
	    &assocEntry->wsRefId);

	/* Set apdu trigger Resource request values */
	switch (req->operation)
	{
		case IRO_TRIGGER_RSRC_RPRT:
			trigReq->requestedOperation = RESOURCEREPORT;
			break;
		case IRO_TRIGGER_RSRC_CNTL:
			trigReq->requestedOperation = RESOURCECONTROL;
			break;
		case IRO_TRIGGER_CANCEL:
			trigReq->requestedOperation = CANCEL;
			break;
		default:
			OpCf_UpdateErrorReport(IR_ERROR_WARNING,
			    IRO_DATA_ERROR, IR_FILE);
			FreeAPDU(&(sendReq.irApdu));
			return (IR_FAIL);
	}

	
	if (OpArc_VerifyRsrcRprtSyntaxOI(&req->resourceReportFmt) == IR_FAIL)
	{
			OpCf_UpdateErrorReport(IR_ERROR_WARNING,
			    IRO_DATA_ERROR, IR_FILE);
			FreeAPDU(&(sendReq.irApdu));
			return (IR_FAIL);
	}

	RescASN_IRToSnaccOI(IR_FILE, &trigReq->preferredRsrcRprtFmt,
	    &req->resourceReportFmt);

	trigReq->resultSetWanted =
	    (AsnBool *)RescIR_Malloc(IR_FILE, sizeof(AsnBool));
	if (trigReq->resultSetWanted == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL,
		    IRO_MEMORY_ERROR, IR_FILE);
		FreeAPDU(&(sendReq.irApdu));
		return (IR_FAIL);
	}
	*trigReq->resultSetWanted = (AsnBool)req->resultSetWanted;
	
	/* send trigger resource request */
	status = IRK_SendRequest(IRK_TRIG_REQ_TYPE, 
	    req->assocId, &sendReq);
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING,
		    IRO_ALI_ERROR, IR_FILE);
		FreeAPDU(&(sendReq.irApdu));
		return (IR_FAIL);
	}
	FreeAPDU(&(sendReq.irApdu));
	return (IR_SUCCESS);
}
