
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: OpCf_CommEvent
 *
 * DESCRIPTION:
 *	This interface function will be used to service all communication
 *	events.
 *
 * CONTENTS:
 *	IRO_CommEvent
 *
 * NOTES:
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 *
/***************************************************************************/

#include <ir.h>
#include <memory.h>
#include <ir_api.h>
#include <ir_ali.h>
#include <resc.h>
#include <op/ir_op.h>

static IRO_COMM_EVENT_REPLY	staticIROReply;

/****************************************************************************
 * FUNCTION:  IRO_CommEvent
 *
 * PURPOSE:
 *	This function is used by the User Application to service any
 *	communications related event.
 *
 *	This function will invoke the IRK_CommEvent function to pass
 *	on the event information for the Communication Services to
 *	service the event. This function will examine the return code
 *	from the IRK_CommEvent function and if extra processing is
 *	required then this function will invoke the appropriate OP
 *	CSC to act on the returned event information. It is the 
 *	responsibilty of the invoked OP CSC to establish the reply 
 *	information.
 *
 * INPUT:
 *	IRO_COMM_EVENT_REQ		- assoc Id, event info.
 *
 * OUTPUT:
 *	IRO_COMM_EVENT_REPLY		- apdu info, status, assoc Id.
 *
 * NOTES:
 *
 ***************************************************************************/

IR_STATUS IRO_CommEvent(IRO_COMM_EVENT_REQ *req, 
	      IRO_COMM_EVENT_REPLY **reply)
{
	IR_STATUS status;
	IRK_COMM_EVENT_REPLY	irkReply;
	IRK_COMM_EVENT_REQ	irkReq;
	IRO_COMM_EVENT_REPLY	*iroReply;


	/* Check to see if interface is initialized */
	if (!OpCf_IsOpInitialized())
	{
		CS_LOG_PRINTS(IR_FILE, "Tried to service commevent when API not initialized.\n");
		return (IR_FAIL);
	}

	/* Free any left over data */
	OpCf_IROFree();

	/* Invoke comm event in kernel layer */
	irkReq.assocId = req->assocId;
	irkReq.event = req->eventInfo;
	status = IRK_CommEvent(&irkReq, &irkReply);
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING, IRO_ALI_ERROR,
		    IR_FILE);
		return (IR_FAIL);
	}
	
	/* Use a static IRO reply for efficiency */
	iroReply = (IRO_COMM_EVENT_REPLY *)&staticIROReply;
	(void)memset((char *)iroReply, (CHAR)NULL, 
	    sizeof(IRO_COMM_EVENT_REPLY)); 
		 

	iroReply->assocId = irkReply.assocId;
	iroReply->commReplyType = irkReply.replyType;

	if (irkReply.replyType != IR_COMM_DATA_NO_DATA)
	{
		CS_LOG_PRINTV(IR_FILE, "OP CommReply = %d\n", irkReply.replyType);
	}
	status = IR_SUCCESS;
	switch (irkReply.replyType)
	{
	case IR_COMM_DATA_ERROR:		
		OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
		    IRO_COMMS_ERROR, IR_FILE);
		break;
	case IR_COMM_DATA_PROTOCOL_ERR:
		break;
	case IR_COMM_DATA_CONN_REFUSED:
	case IR_COMM_DATA_UNKNOWN_HOST:
	case IR_COMM_DATA_HOST_NOT_FOUND:
		if (OpAm_AssocDelete(irkReply.assocId) != IR_SUCCESS)
		{
			/* Remove the assoc entry from all lists */
			OpAm_AbortAssoc(irkReply.assocId);
			irkReply.replyType = IR_COMM_DATA_REL_FAIL;
		}
		break;
	case IR_COMM_DATA_NO_DATA:
		break;
	case IR_COMM_DATA_ABORT:
		if (OpAm_AbortAssoc(irkReply.assocId) != IR_SUCCESS)
		{
			irkReply.replyType = IR_COMM_DATA_ABORT_FAIL;
		}
		break;
	case IR_COMM_DATA_RELEASE:
		if (OpAm_ReleaseAssoc(irkReply.assocId) != IR_SUCCESS)
		{
			irkReply.replyType = IR_COMM_DATA_REL_FAIL;
		}
		break;
	case IR_COMM_DATA_CONNECT:
		if (OpAm_ConnectAssoc(irkReply.assocId) != IR_SUCCESS)
		{
			irkReply.replyType = IR_COMM_DATA_CONN_FAIL;
		}
		break;
	case IR_COMM_DATA_INIT_SRVC:
		if (OpAm_InitService(irkReply.assocId,
		    irkReply.apdu, &iroReply->commData.initResponse) != IR_SUCCESS)
		{
			iroReply->commReplyType = IR_COMM_DATA_INIT_FAIL;
		}
		break;
	case IR_COMM_DATA_ACCS_SRVC:
		status = OpArc_AccessService(irkReply.assocId, 
		    irkReply.apdu, &iroReply->commData.accessCntlReq);
		break;

	case IR_COMM_DATA_RSRC_CNTL_SRVC:
		status = OpArc_RsrcCntlService(irkReply.assocId, 
		    irkReply.apdu, &iroReply->commData.resourceCntlReq);
		break;

	case IR_COMM_DATA_RSRC_RPT_SRVC:
		status = OpArc_RsrcReportService(irkReply.assocId, 
		    irkReply.apdu, &iroReply->commData.resourceReport);
		break;

	case IR_COMM_DATA_SEARCH_SRVC:
		if (OpSp_SearchService(irkReply.assocId,
		    irkReply.apdu, &iroReply->commData.searchResponse) 
		    != IR_SUCCESS)
		{
			iroReply->commReplyType = IR_COMM_DATA_SRCH_FAIL;
		}
		break;
	case IR_COMM_DATA_PRESENT_SRVC:
		if (OpPm_PresentService(irkReply.assocId,
		    irkReply.apdu, &iroReply->commData.presentResponse) 
		    != IR_SUCCESS)
		{
			iroReply->commReplyType = IR_COMM_DATA_PRSNT_FAIL;
		}
		break;
	case IR_COMM_DATA_DELETE_SRVC:
		if (OpWs_DeleteService(irkReply.assocId,
		    irkReply.apdu, &iroReply->commData.deleteResponse) 
		    != IR_SUCCESS)
		{
			iroReply->commReplyType = IR_COMM_DATA_SRCH_FAIL;
		}
		break;
	default:
		OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
		    IRO_PROTOCOL_ERROR, IR_FILE);
		status = IR_FAIL;
		CS_LOG_PRINTV(IR_FILE, "Unknown comm reply data type %d\n",
		    irkReply.replyType);
	}

	/* Copy the return values to the reply structure */
	*reply = iroReply;
	OpCf_IROMark(OpCf_CommReplyFree, iroReply);

	return (status);
}

/****************************************************************************
 * FUNCTION: OpCf_CommReplyFree 
 *
 * PURPOSE:
 *
 * INPUT:
 *	IRO_COMM_EVENT_REPLY		- apdu info, status, assoc Id.
 *
 * OUTPUT:
 *
 * NOTES:
 *
 ***************************************************************************/

void OpCf_CommReplyFree(IRO_COMM_EVENT_REPLY *reply)
{
	switch (reply->commReplyType)
	{
	case IR_COMM_DATA_INIT_SRVC:
		OpAm_InitServiceFree(&reply->commData.initResponse);
		break;
	case IR_COMM_DATA_ACCS_SRVC:
		OpArc_AccessServiceFree(&reply->commData.accessCntlReq);
		break;

	case IR_COMM_DATA_RSRC_CNTL_SRVC:
		OpArc_RsrcControlServiceFree(
		    &reply->commData.resourceCntlReq);
		break;

	case IR_COMM_DATA_RSRC_RPT_SRVC:
		OpArc_RsrcReportServiceFree(
		    &reply->commData.resourceReport);
		break;
	case IR_COMM_DATA_SEARCH_SRVC:
		OpSp_SearchServiceFree(
		    &reply->commData.searchResponse);
		break;
	case IR_COMM_DATA_PRESENT_SRVC:
		OpPm_PresentServiceFree(
		    &reply->commData.presentResponse);
		break;
	case IR_COMM_DATA_DELETE_SRVC:
		OpWs_DeleteServiceFree(
		    &reply->commData.deleteResponse);
		break;
	default:
		break;
		/* do nothing */
	}
	(void)memset((void *)reply, (CHAR)NULL, sizeof(IRO_COMM_EVENT_REPLY));
}
