

/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: OpPm_PresentService
 *
 * DESCRIPTION:
 *	This unit contains the functions used to service the present
 *	IR service apdu.
 *
 * CONTENTS:
 *	OpPm_PresentService
 *	OpPm_PresentServiceFree
 *
 * NOTES:
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 *
/***************************************************************************/
#include <ir.h>
#include <ir_api.h>
#include <ir_ali.h>
#include <resc.h>
#include <resclist.h>
#include <op/ir_op.h>
#include <asn_tool.h>
#include <stdlib.h>


/****************************************************************************
 * FUNCTION:  OpPm_PresentService
 *
 * PURPOSE:
 *	This function is used to process the search service apdu
 *	that was received from the target. If the assocId is valid then
 *	the results of the search will be saved in the workspace.
 *	The status of the search will be returned to the application
 *
 * INPUT:
 *	IR_ASSOC_ID			- assoc list key
 *	APDU				- pointer to search info
 *
 * OUTPUT:
 *	IRO_COMM_EVENT_REPLY 		- pointer pointer to reply
 *
 * NOTES:
 *	All decoded data is copied from the apdu to the search response
 *	and then it is copied to the work space. The only exception to
 *	this is the record list which will be copied to the workspace
 *	only. 
 *
 ***************************************************************************/

IR_STATUS	OpPm_PresentService(IR_ASSOC_ID assocId, APDU *apdu,
		    IRO_COMM_PRSNT_RESP *presentResp)
{
	OP_ASSOC_ENTRY	*assocEntry;
	PresentResp	*apduResp;
	IR_WS_ID		wsId;
	VISIBLE_STRING		*refId;
	OP_WS_INFO		*wsInfo;
	IR_LIST			*recList;

	/* Verify that association exists. */
	assocEntry = OpAm_AssocGet(assocId);
	if (assocEntry == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING, 
		    IRO_ASSOC_DOES_NOT_EXIST, IR_FILE);
		return (IR_FAIL);
	}

	apduResp = apdu->a.presentResp;

	/* Verify the referenceId against the one stored in assocEntry */
	refId = RescASN_CreateVS(IR_FILE);
	RescASN_SnaccToIRVS(IR_FILE, refId, &apduResp->referenceId);

	if (strncmp(refId->sPtr, assocEntry->wsRefId.sPtr, refId->len) != 0)
	{
		/* Reference Id does not correspond. Z3950 protocol no no! */
		CS_LOG_PRINTS(IR_FILE, "Mismatched referenceId on present.\n");
	}
	/* Destroy the refId tmp variable */
	RescASN_DestroyVS(IR_FILE, refId);

	/* Use the assocEntry to find the corresponding ws */
	wsId = (IRO_WORK_SPACE_ID)RescUT_VSToShort(IR_FILE,
	    &assocEntry->wsRefId);

	/* Reset the assocEntry wsRefId */
	RescASN_DestroyVS(IR_FILE, &assocEntry->wsRefId);

	OpWs_WsSetPresentFlag(wsId, FALSE);
	wsInfo = OpWs_WsGet(wsId);
	
	/* If no work space then build records in response */
	if (wsInfo == NULL)
	{
		CS_LOG_PRINTV(IR_FILE, "Couldn't get ws info for wsId %d\n",
		    wsId);
		presentResp->wsId = IR_UNKNOWN_ID_TYPE;
		FreeAPDU(apdu);
		CS_LOG_PRINTS(IR_FILE, "Cannot process response without work space\n");
		return(IR_FAIL);
	}
	else
	{
		CS_LOG_PRINTV(IR_FILE, "ws Id %d found for search\n",
		    wsId);
		recList = &wsInfo->records;
		presentResp->wsId = wsInfo->wsId;
	}

	presentResp->nextPosOnTarget = (LONG)apduResp->nextResultSetPosition;
	presentResp->numRecRetrieved = apduResp->numRecordsReturned;

	/* Copy the present status */
	if (apduResp->presentStatus)
	{
		presentResp->presentStatus = (LONG)*apduResp->presentStatus;
	}
	else
	{
		/* No present status. . */
		CS_LOG_PRINTS(IR_FILE, "This should never occur\n");
		presentResp->presentStatus = IR_PRSNT_DELETE_FAILURE;
		FreeAPDU(apdu);
		return(IR_FAIL);
	}


	
	/* Records are optional. */
	if (apduResp->dbOrDiagnosticRec)
	{
		/* Check for a diagnostic record */
		if (apduResp->dbOrDiagnosticRec->choiceId == 
		    RECORDS_NONSURDIAGNOSTICS)
		{
			/* One diagnostic record to return. */
			if (apduResp->dbOrDiagnosticRec->a.nonSurDiagnostics)
			{
				/* Convert the diagnostic record to IR struct */
				OpSp_ConvertDiagRecToIR(
				    &presentResp->diagnostic,
			    	    apduResp->dbOrDiagnosticRec->a.nonSurDiagnostics);
			}
			    
		}
		else
		{
			/* 
			 * One or more db or surrogate diag records 
			 * Free the allocated ir record and used the
			 * Convert function to convert the entire list.
			 */
			if (apduResp->dbOrDiagnosticRec->a.dbOrSurDiagnostics)
			{
				OpSp_ConvertRecordList(recList,
			    	    apduResp->dbOrDiagnosticRec->a.dbOrSurDiagnostics);
			}
		}
	}
			
	/* update association state */
	assocEntry->state = OP_ASSOC_IDLE;

	/* Copy the resulting search info into the work space. */
	wsInfo->presentStatus = presentResp->presentStatus;
	wsInfo->numWSRecords = RESC_LIST_SIZE(recList);
	presentResp->numRecOnTarget = wsInfo->totalTargetRecords;

	wsInfo->nextTargetRec = presentResp->nextPosOnTarget;

	if (wsInfo->presentStatus == FAILURE)
	{
		/* Copy the diagnostic record to the work space */
		RescASN_CopyOI(IR_FILE, 
		    &wsInfo->nonSurrDiag.diagnosticSetId,
	    	    &presentResp->diagnostic.diagnosticSetId);
		RescASN_CopyVS(IR_FILE, &wsInfo->nonSurrDiag.addInfo,
	    	    &presentResp->diagnostic.addInfo);
		wsInfo->nonSurrDiag.condition = 
	    	    presentResp->diagnostic.condition;
	
	}

	/* Check for inconsistencies */
	if (RESC_LIST_SIZE(&wsInfo->records) 
	     != (SHORT)wsInfo->numWSRecords)
	{
		CS_LOG_PRINTV(IR_FILE, 
	           "Work space records and returned records are different. ws = %d\n",
	           RESC_LIST_SIZE(&wsInfo->records));
	}

	presentResp->numRecInWs = RESC_LIST_SIZE(recList);

	FreeAPDU(apdu);
	return (IR_SUCCESS);
}

void OpPm_PresentServiceFree(IRO_COMM_PRSNT_RESP *presentResp)
{
	if (presentResp)
	{
		OpSp_DestroyDiagRec(&presentResp->diagnostic);
	}
	if (RESC_ASN_IS_MALLOCED(presentResp))
	{
		RescIR_Free(IR_FILE, (CHAR *)presentResp);
	}
}
