
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: OpWs  IRO  Present 
 *
 * DESCRIPTION:
 *
 * CONTENTS:
 *	IRO_Present
 *
 * NOTES:
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 *
/***************************************************************************/

#include <ir.h>
#include <ir_api.h>
#include <ir_ali.h>
#include <resc.h>
#include <resclist.h>
#include <op/ir_op.h>
#include <asn_tool.h>

void OpPm_ElemSetNameToSnacc(VISIBLE_STRING *irElem, 
    ElementSetNames *snaccElem);
void OpPm_ElemSetListToSnacc(IR_LIST *irElemList, 
    ElementSetNames *snaccElemList);

/****************************************************************************
 * FUNCTION:  IRO_Present
 *
 * PURPOSE:
 *	This function supports the IRO API Present functions. In this case 
 *	there is only one function which is to create and send a present
 *	APDU to a remote target. The userId is validated against the current
 *	user. The present request is checked against the local work spaces
 *	to ensure that a work space exists for a remote present.
 *
 * INPUT:
 *	IR_USER_ID
 *	IRO_PRESENT_REQ
 *
 * OUTPUT:
 *	IR_STATUS
 *
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

IR_STATUS	IRO_Present(IR_USER_ID userId, 
		    IRO_PRESENT_REQ *req)
{
	OP_SESSION_INFO	*sessionInfo;
	OP_WS_INFO	*ws;
	OP_ASSOC_ENTRY	 	*assocEntry;
	IRK_SEND_REQUEST_REQ	pSend;
	PresentReq		*pReq;
	IR_STATUS		status;


	/* Verify the user Id */
	if ((sessionInfo = OpTm_UserSessionInfo(userId)) == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING, 
		    IRO_UNKNOWN_USER_ID, IR_FILE);
		return (IR_FAIL);
	}

	if ((ws = OpWs_WsGet(req->wsId)) == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING, 
	    	    IRO_WORKSPACE_DOES_NOT_EXIST, IR_FILE);
		return (IR_FAIL);
	}


	if (ws->assocId != req->assocId)
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING, 
	    	    IRO_ASSOC_DOES_NOT_EXIST, IR_FILE);
		return (IR_FAIL);
	}

	assocEntry = OpAm_AssocGet(req->assocId);
	if (assocEntry == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING,
		     IRO_ASSOC_DOES_NOT_EXIST, IR_FILE);
		return (IR_FAIL);
	}

	/* Check to see if the operation is supported */
	if (OpAm_AssocServiceAllowed(OPAM_PRESENT, req->assocId) 
	    == FALSE)
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING,
		    IRO_REQ_SERVICE_NOT_SUPPORTED, IR_FILE);
		return(IR_FAIL);
	}


	/* Initialize APDU */

	/* Create the present request */
	pSend.irApdu.choiceId = APDU_PRESENTREQ;
	pSend.irApdu.a.presentReq = (PresentReq *)
	    RescIR_Zmalloc(IR_FILE, sizeof(PresentReq));
	if (pSend.irApdu.a.presentReq == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL,
		     IRO_MEMORY_ERROR, IR_FILE);
		return (IR_FAIL);
	}

	pReq = pSend.irApdu.a.presentReq;

	/* Create the reference Id based on the assocId */
	RescUT_ShortToVS(IR_FILE, &assocEntry->wsRefId, (SHORT)req->wsId);
	RescASN_IRToSnaccOS(IR_FILE, &pReq->referenceId, 
	    &assocEntry->wsRefId);
	
	RescASN_IRToSnaccVS(IR_FILE, &pReq->resultSetId,
	    &req->resultSetName);

	pReq->resultSetStartPoint = (AsnInt)req->startRec;
	pReq->numRecordsRequested = (AsnInt)req->numRecords;

	/* Copy the element set name list to the apdu. */
	if (OpAm_AssocServiceAllowed(OPAM_ELEM_SET_NAMING, req->assocId))
	{
		switch(req->elemSetNameType)
		{
		case IR_ELEM_SET_NAME_NONE:
			break;
		case IR_ELEM_SET_NAME_GENERIC:
			pReq->elementSetNames = (ElementSetNames *)
		    	    RescIR_Zmalloc(IR_FILE, sizeof(ElementSetNames));
			OpPm_ElemSetNameToSnacc(&req->elemSetList.generic, 
		    	    pReq->elementSetNames);
			break;
		case IR_ELEM_SET_NAME_SPECIFIC:
			pReq->elementSetNames = (ElementSetNames *)
		    	    RescIR_Zmalloc(IR_FILE, sizeof(ElementSetNames));
			OpPm_ElemSetListToSnacc(&req->elemSetList.dbSpecific, 
		    	    pReq->elementSetNames);
			break;
		default:
			break;
		}
	}
	
	/* Convert the preferred syntax type to an object identifier. */
	RescASN_IRToSnaccOI(IR_FILE, &pReq->preferredRecordSyntax,
	    &req->recordSyntax);

	status = IRK_SendRequest(IRK_PRESENT_REQ_TYPE,
	    req->assocId, &pSend);
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL,
		     IRO_ALI_ERROR, IR_FILE);
		FreeAPDU(&(pSend.irApdu));
		return (IR_FAIL);
	}	

	assocEntry->state = OP_ASSOC_PRESENTING;
	FreeAPDU(&(pSend.irApdu));
	OpWs_WsSetPresentFlag(req->wsId, TRUE);
	return(status);
}
