

/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: OpSp_SearchService
 *
 * DESCRIPTION:
 *	This unit contains the functions used to service the search 
 *	IR service apdu.
 *
 * CONTENTS:
 *	OpSp_SearchService
 *	OpSp_SearchServiceFree
 *
 * NOTES:
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 *
/***************************************************************************/
#include <ir.h>
#include <ir_api.h>
#include <ir_ali.h>
#include <resc.h>
#include <resclist.h>
#include <op/ir_op.h>
#include <asn_tool.h>
#include <stdlib.h>


/****************************************************************************
 * FUNCTION:  OpSp_SearchService
 *
 * PURPOSE:
 *	This function is used to process the search service apdu
 *	that was received from the target. If the assocId is valid then
 *	the results of the search will be saved in the workspace.
 *	The status of the search will be returned to the application
 *
 * INPUT:
 *	IR_ASSOC_ID			- assoc list key
 *	APDU				- pointer to search info
 *
 * OUTPUT:
 *	IRO_COMM_EVENT_REPLY 		- pointer pointer to reply
 *
 * NOTES:
 *	All decoded data is copied from the apdu to the search response
 *	and then it is copied to the work space. The only exception to
 *	this is the record list which will be copied to the workspace
 *	only. 
 *
 ***************************************************************************/

IR_STATUS	OpSp_SearchService(IR_ASSOC_ID assocId, APDU *apdu,
		    IRO_COMM_SRCH_RESP *searchResp)
{
	OP_ASSOC_ENTRY	*assocEntry;
	SearchResp	*apduResp;
	IR_WS_ID		wsId;
	VISIBLE_STRING		*refId;
	OP_WS_INFO		*wsInfo;
	IR_LIST			*recList;

	/* Verify that association exists. */
	assocEntry = OpAm_AssocGet(assocId);
	if (assocEntry == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING, 
		    IRO_ASSOC_DOES_NOT_EXIST, IR_FILE);
		return (IR_FAIL);
	}

	apduResp = apdu->a.searchResp;

	/* Verify the referenceId against the one stored in assocEntry */
	refId = RescASN_CreateVS(IR_FILE);
	RescASN_SnaccToIRVS(IR_FILE, refId, &apduResp->referenceId);

	if (strncmp(refId->sPtr, assocEntry->wsRefId.sPtr, refId->len) != 0)
	{
		/* Reference Id does not correspond. Z3950 protocol no no! */
		CS_LOG_PRINTS(IR_FILE, "Mismatched referenceId on search.\n");
	}
	/* Destroy the refId tmp variable */
	RescASN_DestroyVS(IR_FILE, refId);

	/* Use the assocEntry to find the corresponding ws */
	wsId = (IRO_WORK_SPACE_ID)RescUT_VSToShort(IR_FILE,
	    &assocEntry->wsRefId);

	/* Reset the assocEntry wsRefId */
	RescASN_DestroyVS(IR_FILE, &assocEntry->wsRefId);

	wsInfo = OpWs_WsGet(wsId);
	
	/* If no work space then build records in iro response message */
	if (wsInfo == NULL)
	{
		/* Error condition */
		CS_LOG_PRINTV(IR_FILE, "Couldn't get ws info for wsId %d\n",
		    wsId);
		searchResp->wsId = IR_UNKNOWN_ID_TYPE;
		FreeAPDU(apdu);
		return(IR_FAIL);
	}
	else
	{
		recList = &wsInfo->records;
		searchResp->wsId = wsInfo->wsId;
	}

	/* Copy the search status */
	searchResp->searchStatus = apduResp->searchStatus;

	searchResp->numRecInWs = 0;
	searchResp->numRecOnTarget = (LONG)apduResp->resultCount;
	searchResp->nextPosOnTarget = (LONG)apduResp->nextResultSetPosition;

	/* Copy the result set status */
	if (apduResp->resultSetStatus)
	{
		switch(*apduResp->resultSetStatus)
		{
		case SUBSET1:
			searchResp->resultSetStatus = IR_RESULT_SET_SUBSET;
			break;
		case INTERIM1:
			searchResp->resultSetStatus = IR_RESULT_SET_INTERIM;
			break;
		case NONE1:
		default:
			searchResp->resultSetStatus = IR_RESULT_SET_NONE;
			break;
		}
	}
	else
	{
		/* No result set status. . */
		searchResp->resultSetStatus = IR_RESULT_SET_NO_STATUS;
	}

	/* Copy the present status */
	if (apduResp->presentStatus)
	{
		searchResp->presentStatus = (LONG)*apduResp->presentStatus;
	}
	else
	{
		/* No present status. . */
		searchResp->presentStatus = IR_PRSNT_DELETE_FAILURE;
	}


	
	/* Records are optional. */
	if (apduResp->dbOrDiagnosticRec)
	{
		/* Check for a diagnostic record */
		if (apduResp->dbOrDiagnosticRec->choiceId == 
		    RECORDS_NONSURDIAGNOSTICS)
		{
			/* One diagnostic record to return. */
			if (apduResp->dbOrDiagnosticRec->a.nonSurDiagnostics)
			{
				/* Convert the diagnostic record to IR struct */
				OpSp_ConvertDiagRecToIR(
				    &searchResp->diagnostic,
			    	    apduResp->dbOrDiagnosticRec->a.nonSurDiagnostics);
			}
			    
		}
		else
		{
			/* 
			 * One or more db or surrogate diag records 
			 * Free the allocated ir record and used the
			 * Convert function to convert the entire list.
			 */
			if (apduResp->dbOrDiagnosticRec->a.dbOrSurDiagnostics)
			{
				OpSp_ConvertRecordList(recList,
			    	    apduResp->dbOrDiagnosticRec->a.dbOrSurDiagnostics);
			}
		}
	}
			
	/* update association state */
	assocEntry->state = OP_ASSOC_IDLE;

	/* Update the work space info */
	if (wsInfo != NULL)
	{
		/* Copy the resulting search info into the work space. */
		wsInfo->searchStatus = searchResp->searchStatus;
		wsInfo->resultSetStatus = searchResp->resultSetStatus;
		wsInfo->presentStatus = searchResp->presentStatus;
		wsInfo->totalTargetRecords = searchResp->numRecOnTarget;
		wsInfo->numWSRecords = RESC_LIST_SIZE(recList);

		wsInfo->nextTargetRec = searchResp->nextPosOnTarget;

		if (wsInfo->searchStatus == FALSE)
		{
			/* Copy the diagnostic record to the work space */
			RescASN_CopyOI(IR_FILE, 
			    &wsInfo->nonSurrDiag.diagnosticSetId,
		    	    &searchResp->diagnostic.diagnosticSetId);
			RescASN_CopyVS(IR_FILE, &wsInfo->nonSurrDiag.addInfo,
		    	    &searchResp->diagnostic.addInfo);
			wsInfo->nonSurrDiag.condition = 
		    	    searchResp->diagnostic.condition;
		
		}

		/* Check for inconsistencies */
		if (RESC_LIST_SIZE(&wsInfo->records) 
		     != (SHORT)wsInfo->numWSRecords)
		{
			CS_LOG_PRINTV(IR_FILE, 
		           "Work space records and returned records are different. ws = %d\n",
		           RESC_LIST_SIZE(&wsInfo->records));
		}

		searchResp->numRecInWs = RESC_LIST_SIZE(recList);

	}

	/* Check for a remote count of 0. */
	if (searchResp->numRecOnTarget == 0)
	{ 
		/* No need for local one either */
		(void)OpWs_WsDelete(searchResp->wsId);
		searchResp->wsId = IR_UNKNOWN_ID_TYPE;
	}

	FreeAPDU(apdu);
	return (IR_SUCCESS);
}

void OpSp_ConvertRecordList(IR_LIST *recList, RecordsSeqOf *apduList)
{
	IR_RECORD_STRUCT	*irRec;
	NamePlusRecord		*apduRec;

	if (recList == NULL)
	{
		CS_LOG_PRINTS(IR_FILE, "Invalid IR record List\n");
		return;
	}
	if (apduList == NULL)
	{
		CS_LOG_PRINTS(IR_FILE, "Invalid apdu record list\n");
		return;
	}

	/* 
	 * Foreach element in the apdu list create a corresponding one
	 * in the IR_LIST.
	 */
	FOR_EACH_LIST_ELMT(apduRec, apduList)
	{
		irRec = (IR_RECORD_STRUCT *)RescIR_Zmalloc(IR_FILE, 
		    sizeof(IR_RECORD_STRUCT));
		if (irRec == NULL)
		{
			CS_LOG_PRINTS(IR_FILE, "Malloc error\n");
			break;
		}

		RESC_ASN_MALLOCED(irRec);
		RESC_ASN_NO_MALLOC(&irRec->databaseName);
		RESC_ASN_INIT_VS(&irRec->databaseName);

		RESC_ASN_NO_MALLOC(&irRec->dbRecord);
		RESC_ASN_NO_MALLOC(&irRec->dbRecord.dbRecSyntax);
		RESC_ASN_INIT_OI(&irRec->dbRecord.dbRecSyntax);
		RESC_ASN_NO_MALLOC(&irRec->dbRecord.record);
		RESC_ASN_INIT_OS(&irRec->dbRecord.record);

		RESC_ASN_NO_MALLOC(&irRec->diagRecord);
		RESC_ASN_NO_MALLOC(&irRec->diagRecord.diagnosticSetId);
		RESC_ASN_INIT_OI(&irRec->diagRecord.diagnosticSetId);
		RESC_ASN_NO_MALLOC(&irRec->diagRecord.addInfo);
		RESC_ASN_INIT_VS(&irRec->diagRecord.addInfo);


		RescASN_SnaccToIRVS(IR_FILE, &irRec->databaseName,
		    &apduRec->databaseName);
		if (apduRec->namePlusRecordChoice)
		{
			if (apduRec->namePlusRecordChoice->choiceId ==
			    NAMEPLUSRECORDCHOICE_DATABASERECORD)
			{
				irRec->recordType = IR_REC_TYPE_DB_RECORD;
				OpSp_ConvertDbRecToIR(
				    &irRec->dbRecord,
			    	    apduRec->namePlusRecordChoice->a.databaseRecord);
			}
			else
			{
				irRec->recordType = IR_REC_TYPE_SURR_DIAG;
				OpSp_ConvertDiagRecToIR(
				    &irRec->diagRecord,
			    	    apduRec->namePlusRecordChoice->a.surrogateDiagnostic);
			}
		}
		else
		{
			irRec->recordType = IR_REC_TYPE_NONE;
		}

		/* Append the record to the list. */
		RescList_Append(IR_FILE, recList, irRec,
		    OpSp_RecordFree, OpSp_RecordCopy);

	}

}

void OpSp_ConvertDiagRecToIR(IR_DIAG_RECORD *rec, DiagRec *apduDiag)
{

	
	RESC_ASN_NO_MALLOC(rec);
	RESC_ASN_NO_MALLOC(&rec->diagnosticSetId);
	RESC_ASN_INIT_OI(&rec->diagnosticSetId);
	RESC_ASN_NO_MALLOC(&rec->addInfo);
	RESC_ASN_INIT_OI(&rec->addInfo);

	if (apduDiag == NULL)
	{
		/* For some reason this diagnostic may not be there. */
		return;
	}
	RescASN_SnaccToIROI(IR_FILE, &rec->diagnosticSetId,
	    &apduDiag->diagnosticSetId);

	rec->condition = (LONG)apduDiag->condition;

	RescASN_SnaccToIRVS(IR_FILE, &rec->addInfo, &apduDiag->addInfo);
}

void OpSp_ConvertDbRecToIR(IR_DB_RECORD *rec, DatabaseRecord *apduRec)
{
	IR_EXTERNAL	*dbExt;

	RESC_ASN_NO_MALLOC(rec);
	RESC_ASN_NO_MALLOC(&rec->dbRecSyntax);
	RESC_ASN_INIT_OI(&rec->dbRecSyntax);
	RESC_ASN_NO_MALLOC(&rec->record);
	RESC_ASN_INIT_OS(&rec->record);

	if (apduRec == NULL)
	{
		/* For some reason the data may not be there. */
		return;
	}

	dbExt = RescASN_CreateExternal(IR_FILE);
	RescASN_SnaccToIRExternal(IR_FILE, dbExt, apduRec);

	if (dbExt->encType == IR_EXT_ENC_ALIGNED)
	{
		RescASN_CopyOI(IR_FILE, &rec->dbRecSyntax,
		    &dbExt->direct);
		RescASN_CopyOS(IR_FILE, &rec->record, &dbExt->octetAligned);
	}
	else
	{
		CS_LOG_PRINTV(IR_FILE, "Unknown external for a record. %d\n",
		    dbExt->referenceType);
	}
	RescASN_DestroyExternal(IR_FILE, dbExt);
}

void OpSp_RecordFree(CHAR *srcFile, void *data)
{
	IR_RECORD_STRUCT *rec;

	rec = (IR_RECORD_STRUCT *)data;

	if (!rec)
	{
		return;
	}

	RescASN_DestroyVS(IR_FILE, &rec->databaseName);
	RescASN_DestroyOI(IR_FILE, &rec->dbRecord.dbRecSyntax);
	RescASN_DestroyOS(IR_FILE, &rec->dbRecord.record);
	RescASN_DestroyOI(IR_FILE, &rec->diagRecord.diagnosticSetId);
	RescASN_DestroyVS(IR_FILE, &rec->diagRecord.addInfo);

	if (RESC_ASN_IS_MALLOCED(rec))
	{
		RescIR_Free(IR_FILE, (CHAR *)rec);
	}
}

void *OpSp_RecordCopy(CHAR *srcFile, void *srcElem)
{
	IR_RECORD_STRUCT	*rec;
	IR_RECORD_STRUCT	*srcRec;

	/* Clone the input record. */

	if (srcElem == NULL)
	{
		return((void *)NULL);
	}

	rec = (IR_RECORD_STRUCT *)RescIR_Zmalloc(IR_FILE,
	    sizeof(IR_RECORD_STRUCT));

	srcRec = (IR_RECORD_STRUCT *)srcElem;

	if (rec == NULL)
	{
		return((void *)NULL);
	}
	RESC_ASN_MALLOCED(rec);
	RESC_ASN_NO_MALLOC(&rec->databaseName);
	RESC_ASN_INIT_VS(&rec->databaseName);
	RescASN_CopyVS(IR_FILE, &rec->databaseName, &srcRec->databaseName);

	rec->recordType = srcRec->recordType;


	RESC_ASN_NO_MALLOC(&rec->dbRecord);
	RESC_ASN_NO_MALLOC(&rec->dbRecord.dbRecSyntax);
	RESC_ASN_INIT_OI(&rec->dbRecord.dbRecSyntax);
	RESC_ASN_NO_MALLOC(&rec->dbRecord.record);
	RESC_ASN_INIT_OS(&rec->dbRecord.record);

	RescASN_CopyOI(srcFile, &rec->dbRecord.dbRecSyntax,
	  &srcRec->dbRecord.dbRecSyntax);

	RescASN_CopyOS(srcFile, 
	    &rec->dbRecord.record,
	    &srcRec->dbRecord.record);


	RESC_ASN_NO_MALLOC(&rec->diagRecord);
	RESC_ASN_NO_MALLOC(&rec->diagRecord.diagnosticSetId);
	RESC_ASN_INIT_OS(&rec->diagRecord.diagnosticSetId);
	RESC_ASN_NO_MALLOC(&rec->diagRecord.addInfo);
	RESC_ASN_INIT_OS(&rec->diagRecord.addInfo);

	RescASN_CopyOI(srcFile, 
	    &rec->diagRecord.diagnosticSetId,
	    &srcRec->diagRecord.diagnosticSetId);

	RescASN_CopyVS(srcFile, 
	    &rec->diagRecord.addInfo,
	    &srcRec->diagRecord.addInfo);

	rec->diagRecord.condition = srcRec->diagRecord.condition;

	return(rec);
}

void OpSp_SearchServiceFree(IRO_COMM_SRCH_RESP *searchResp)
{
	if (searchResp)
	{
		OpSp_DestroyDiagRec(&searchResp->diagnostic);
	}
	if (RESC_ASN_IS_MALLOCED(searchResp))
	{
		RescIR_Free(IR_FILE, (CHAR *)searchResp);
	}
}

void OpSp_DestroyDiagRec(IR_DIAG_RECORD *diagRec)
{
	if (diagRec == NULL)
	{
		return;
	}

	RescASN_DestroyOI(IR_FILE, &diagRec->diagnosticSetId);
	RescASN_DestroyVS(IR_FILE, &diagRec->addInfo);

	if (RESC_ASN_IS_MALLOCED(diagRec))
	{
		RescIR_Free(IR_FILE, (CHAR *)diagRec);
	}
}
