
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/***************************************************************************
 * UNIT: OpSp_Search
 *
 * DESCRIPTION:
 *	This unit is used to perform an IRO_Search request
 *
 * CONTENTS:
 *	IRO_Search
 * 	OpSp_ConvertRPNSearchQuery
 *	OpSp_GetOperand
 *	OpSp_GetOperator
 *	
 *
 * NOTES:
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 *
/***************************************************************************/

#include <ir.h>
#include <string.h>
#include <ir_api.h>
#include <ir_ali.h>
#include <resc.h>
#include <op/ir_op.h>
#include <asn_tool.h>
#include <ir_oid.h>
#include <resclist.h>

void OpSp_IRToSnaccRPNOperand(RPNStructure *snaccElem,
	      IR_RPN_SIMPLE_NODE *irElem);
void OpSp_IRToSnaccRPNStructSeq(RPNStructure *snaccElem,
	      IR_RPN_COMPLEX_NODE *irElem);
void OpSp_ElemSetNameToSnacc(VISIBLE_STRING *irElem, 
    ElementSetNames *snaccElem);
void OpSp_ElemSetListToSnacc(IR_LIST *irElemList, 
    ElementSetNames *snaccElemList);
IRO_SEARCH_REQ	*OpSp_DupSearchReq(IRO_SEARCH_REQ *req);

/****************************************************************************
 * FUNCTION:  IRO_Search
 *
 * PURPOSE:
 *	This function is used to service the API search interface
 *	function.
 *
 * INPUT:
 *	IRO_SEARCH_REQ			- request info
 * OUTPUT:
 *	IR_STATUS			- status of operation
 *
 * NOTES:
 *	The calling function should check the IRO_ErrorReport if the
 *	return status is IR_FAIL.
 *
 ***************************************************************************/

IR_STATUS	IRO_Search(IR_USER_ID userId, IRO_SEARCH_REQ *req)
{
	IR_STATUS 	 	status;
	OP_ASSOC_ENTRY	 	*assocEntry;
	SearchReq		*searchReq;
	IRK_SEND_REQUEST_REQ	sendReq;
	VISIBLE_STRING		*vsPtr;
	AsnOcts			*octs;
	OP_SESSION_INFO		*sessionInfo;
	OP_WS_INFO		*wsInfo;

	/* Verify that interface has been initialized. */
	if (!OpCf_IsOpInitialized())
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL, IRO_PROTOCOL_ERROR,
		    IR_FILE);
		return (IR_FAIL);
	}

	/* Verify that association exists. */
	assocEntry = OpAm_AssocGet(req->assocId);
	if (assocEntry == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING,
		     IRO_ASSOC_DOES_NOT_EXIST, IR_FILE);
		return (IR_FAIL);
	}

	/* Verify the user Id */
	if ((sessionInfo = OpTm_UserSessionInfo(userId)) == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING,
		     IRO_UNKNOWN_USER_ID, IR_FILE);
		return (IR_FAIL);
	}

	/* Check to see whether the association is busy or not */
	if (assocEntry->state != OP_ASSOC_IDLE)
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING,
		    IRO_ASSOC_BUSY, IR_FILE);
		return(IR_FAIL);
	}

	/* Check to see if the operation is supported */
	if (OpAm_AssocServiceAllowed(OPAM_SEARCH, req->assocId) 
	    == FALSE)
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING,
		    IRO_REQ_SERVICE_NOT_SUPPORTED, IR_FILE);
		return(IR_FAIL);
	}

	/* Initialize APDU */
	sendReq.irApdu.choiceId = APDU_SEARCHREQ;
	sendReq.irApdu.a.searchReq = (SearchReq *)RescIR_Zmalloc(IR_FILE,
	    sizeof(SearchReq));
	if (sendReq.irApdu.a.searchReq == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL,
		     IRO_MEMORY_ERROR, IR_FILE);
		return (IR_FAIL);
	}

	searchReq = sendReq.irApdu.a.searchReq;

	/* Generate reference id to match up with search response. */
	wsInfo = OpWs_WsCreate(userId);
	RescUT_ShortToVS(IR_FILE, &assocEntry->wsRefId, 
	    (SHORT)wsInfo->wsId);
	RescASN_IRToSnaccOS(IR_FILE, &searchReq->referenceId, 
	    &assocEntry->wsRefId);

	wsInfo->assocId = req->assocId;

	/* Copy values from search req to apdu */
	searchReq->smallSetUpperBound = (AsnInt)req->smallSetUpperBound;
	searchReq->largeSetLowerBound = (AsnInt)req->largeSetLowerBound;
	searchReq->mediumSetPresentNum = (AsnInt)req->mediumSetPresentNum;
	searchReq->replaceIndicator = (AsnBool)req->replaceIndicator;
	
	/* Copy result set name to apdu. */

	RescASN_IRToSnaccVS(IR_FILE, &searchReq->resultSetName,
	    &req->resultSetName);

	/* 
	 * According to the Z39.50 standard: if the result set name
	 * is default then the replace indicator must be on.
	 */
	if (strncmp(req->resultSetName.sPtr, "default",
	    strlen("default")) == 0)
	{
		searchReq->replaceIndicator = (AsnBool)TRUE;
	}

	/* 
	 * If this result set name was used for a previous search
	 * on this association then remove the reference to this
	 * result set name from the local work space.
 	 */
	OpWs_WsNewResultName(req->assocId, &req->resultSetName, 
	    searchReq->replaceIndicator);

	/* Copy new result set name to work space */
	RescASN_CopyVS(IR_FILE, &wsInfo->resultSetName,
	    &req->resultSetName);

	/* Copy list of databases to apdu. */
	RESC_LIST_SET_FIRST(&req->dbNameList);
	searchReq->databaseNames = ASN_ListCreate(sizeof(AsnOcts));
	while (RESC_LIST_CUR(&req->dbNameList))
	{
		vsPtr = RESC_LIST_GET(&req->dbNameList);
		RescUT_DebugVS(IR_FILE, vsPtr);
		octs = ASN_OctsCreate();
		RescASN_IRToSnaccVS(IR_FILE, octs, vsPtr);
		ASN_ListAppend(searchReq->databaseNames, octs);
		RESC_LIST_SET_NEXT(&req->dbNameList);
	}

	/* Copy the small and medium set name lists to the apdu. */
	if (OpAm_AssocServiceAllowed(OPAM_ELEM_SET_NAMING, req->assocId))
	{
		switch(req->smallSetNameType)
		{
		case IR_ELEM_SET_NAME_NONE:
			break;
		case IR_ELEM_SET_NAME_GENERIC:
			searchReq->smallSetElementSetNames = (ElementSetNames *)
		    	    RescIR_Zmalloc(IR_FILE, sizeof(ElementSetNames));
			OpSp_ElemSetNameToSnacc(&req->smallSetList.generic, 
		    	    searchReq->smallSetElementSetNames);
			break;
		case IR_ELEM_SET_NAME_SPECIFIC:
			searchReq->smallSetElementSetNames = (ElementSetNames *)
		    	    RescIR_Zmalloc(IR_FILE, sizeof(ElementSetNames));
			OpSp_ElemSetListToSnacc(&req->smallSetList.dbSpecific, 
		    	    searchReq->smallSetElementSetNames);
			break;
		default:
			break;
		}

		switch(req->mediumSetNameType)
		{
		case IR_ELEM_SET_NAME_NONE:
			break;
		case IR_ELEM_SET_NAME_GENERIC:
			searchReq->mediumSetElementSetNames = (ElementSetNames *)
		      	    RescIR_Zmalloc(IR_FILE, sizeof(ElementSetNames));
			OpSp_ElemSetNameToSnacc(&req->mediumSetList.generic, 
		    	    searchReq->mediumSetElementSetNames);
			break;
		case IR_ELEM_SET_NAME_SPECIFIC:
			searchReq->mediumSetElementSetNames = (ElementSetNames *)
		    	    RescIR_Zmalloc(IR_FILE, sizeof(ElementSetNames));
			OpSp_ElemSetListToSnacc(&req->mediumSetList.dbSpecific, 
		    	    searchReq->mediumSetElementSetNames);
			break;
		default:
			break;
		}
	}

	/* Convert the preferred syntax type to an object identifier. */
	RescASN_IRToSnaccOI(IR_FILE, &searchReq->preferredRecordSyntax,
	    &req->recordSyntax);

	searchReq->query = (Query *)RescIR_Zmalloc(IR_FILE, sizeof(Query));
	if (searchReq->query == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL,
		     IRO_MEMORY_ERROR, IR_FILE);
		FreeAPDU(&(sendReq.irApdu));
		return (IR_FAIL);
	}
	switch (req->queryType)
	{
		case IR_QUERY_TYPE_0:
			searchReq->query->choiceId = QUERY_TYPE_0;
			OpCf_UpdateErrorReport(IR_ERROR_WARNING,
			    IRO_QUERY_TYPE_NOT_SUPPORTED, IR_FILE);
			FreeAPDU(&(sendReq.irApdu));
			return (IR_FAIL);
			break;
		case IR_QUERY_TYPE_1:
			searchReq->query->choiceId = QUERY_TYPE_1;
			status = OpSp_ConvertRPNSearchQuery(&req->query,
			   &(searchReq->query->a.type_1));
			if (status == IR_FAIL)
			{
				FreeAPDU(&(sendReq.irApdu));
				return (IR_FAIL);
			}
			break;
		case IR_QUERY_TYPE_2:
			searchReq->query->choiceId = QUERY_TYPE_2;
			OpCf_UpdateErrorReport(IR_ERROR_WARNING,
			    IRO_QUERY_TYPE_NOT_SUPPORTED, IR_FILE);
			FreeAPDU(&(sendReq.irApdu));
			return (IR_FAIL);
			break;
		case IR_QUERY_TYPE_100:
			searchReq->query->choiceId = QUERY_TYPE_100;
			OpCf_UpdateErrorReport(IR_ERROR_WARNING,
			    IRO_QUERY_TYPE_NOT_SUPPORTED, IR_FILE);
			FreeAPDU(&(sendReq.irApdu));
			return (IR_FAIL);
			break;
		case IR_QUERY_TYPE_101:
			searchReq->query->choiceId = QUERY_TYPE_101;
			OpCf_UpdateErrorReport(IR_ERROR_WARNING,
			    IRO_QUERY_TYPE_NOT_SUPPORTED, IR_FILE);
			FreeAPDU(&(sendReq.irApdu));
			return (IR_FAIL);
			break;
		default:
			OpCf_UpdateErrorReport(IR_ERROR_WARNING,
			    IRO_DATA_ERROR, IR_FILE);
			FreeAPDU(&(sendReq.irApdu));
			return (IR_FAIL);
	}


	/* Issue ALI search request generated. */
	status = IRK_SendRequest(IRK_SEARCH_REQ_TYPE,
	    req->assocId, &sendReq);
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL,
		     IRO_ALI_ERROR, IR_FILE);
		FreeAPDU(&(sendReq.irApdu));
		return (IR_FAIL);
	}	

	/* Update association state. */
	assocEntry->state = OP_ASSOC_SEARCHING;

	FreeAPDU(&(sendReq.irApdu));
	return (IR_SUCCESS);
}

/****************************************************************************
 * FUNCTION:  OpSp_ConvertRPNSearchQuery
 *
 * PURPOSE:
 *	This function is used to copy an IR RPN query to an apdu 
 *	type 1 search query
 *
 * INPUT:
 *	IR_RPN_QUERY_STRUCT - RPN search query 
 *
 * OUTPUT:
 *	RPNQuery*			- pointer to an apdu type 1 query
 *	IR_STATUS
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/
IR_STATUS OpSp_ConvertRPNSearchQuery(IR_RPN_QUERY_STRUCT *reqQuery,
					RPNQuery **apduQueryPtr)
{
	RPNQuery	*apduQuery;

	/* Allocate space for query. */
	apduQuery = (RPNQuery *)RescIR_Zmalloc(IR_FILE, sizeof(RPNQuery));	
	if (apduQuery == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL,
		     IRO_MEMORY_ERROR, IR_FILE);
		return (IR_FAIL);
	}
	*apduQueryPtr = apduQuery;

	/* Copy the attribute set id object identifier to snacc format */
	RescASN_IRToSnaccOI(IR_FILE, &apduQuery->attributeSetId,
	    &reqQuery->attributeSetId);
	    
	/* 
	 * Start with the first element and perform a recursive conversion
	 * of all elements.
	 */
	if ((apduQuery->rPNStructure = 
	    (RPNStructure *)RescIR_Zmalloc(IR_FILE,
	    sizeof(RPNStructure))) != NULL)
	{

		if (reqQuery->rpnElem.rpnElemType == IR_RPN_NODE_SIMPLE)
		{
			OpSp_IRToSnaccRPNOperand(apduQuery->rPNStructure,
		    	    reqQuery->rpnElem.rpnOperand);
		}
		else
		{
			OpSp_IRToSnaccRPNStructSeq(apduQuery->rPNStructure,
		    	    reqQuery->rpnElem.rpnStructSeq);
		}
	}
	return (IR_SUCCESS);
}

/****************************************************************************
 * FUNCTION:  OpSp_IRToSnaccRPNOperand
 *
 * PURPOSE:
 *	This function is used to copy an IR RPN query operand to an apdu 
 *	type 1 search query operand
 *
 * INPUT:
 *	IR_RPN_ELEMENT	- RPN search query element
 *
 * OUTPUT:
 *	RPNStructure		- pointer to an apdu type 1 query operand
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/
void OpSp_IRToSnaccRPNOperand(RPNStructure *snaccElem,
	      IR_RPN_SIMPLE_NODE *irElem)
{
	Operand		*snaccOperand;
	BOOLEAN		done;
	IR_ATTR_ELEM	*irAttribute;
	AttributeElement *snaccAttribute;
	AttributeList	*snaccList;

	snaccElem->choiceId = RPNSTRUCTURE_OPERAND;


	if ((snaccElem->a.operand = (Operand *)RescIR_Zmalloc(IR_FILE,
	    sizeof(Operand))) == NULL)
	{
		CS_LOG_PRINTS(IR_FILE, "Cannot malloc snacc operand\n");
		return;
	}

	snaccOperand = snaccElem->a.operand;

	if (irElem->simpleType == IR_RPN_SIMPLE_ATTR)
	{
		/* Attribute plus term */
		snaccOperand->choiceId = OPERAND_ATTRIBUTEPLUSTERM;

		if ((snaccOperand->a.attributePlusTerm = 
		     (AttributePlusTerm *)RescIR_Zmalloc(IR_FILE,
		     sizeof(AttributePlusTerm))) != NULL)
		{
			RESC_LIST_SET_FIRST(
			    &irElem->attrPlusTerm.attrList);
			done = FALSE;
			/* Convert each attribute */
			snaccList = ASN_ListCreate(sizeof(AttributeElement));
			snaccOperand->a.attributePlusTerm->attributeList =
			    snaccList;
			while ((!done) && (snaccList != NULL))
			{
				irAttribute = RESC_LIST_GET(
				    &irElem->attrPlusTerm.attrList);
				snaccAttribute = (AttributeElement *)
			  	    RescIR_Malloc(IR_FILE,
				    sizeof(AttributeElement));

				if (snaccAttribute != NULL)
				{
					snaccAttribute->attributeType =
					    irAttribute->attrType;
					snaccAttribute->attributeValue =
					    irAttribute->attrValue;
					ASN_ListAppend(snaccList,
					    snaccAttribute);
				}
				if (RESC_LIST_IS_LAST(
				    &irElem->attrPlusTerm.attrList))
				{
					done = TRUE;
				}
				else
				{
					RESC_LIST_SET_NEXT(
					    &irElem->attrPlusTerm.attrList);
				}
			}
				

			/* Copy the term */
			RescASN_IRToSnaccOS(IR_FILE,
			    &snaccOperand->a.attributePlusTerm->term,
			    &irElem->attrPlusTerm.term);
		}
	}
	else
	{
		/* Just a result set Id */
		snaccOperand->choiceId = OPERAND_RESULTSETID;

		if ((snaccOperand->a.resultSetId = ASN_OctsCreate()) 
		    != NULL)
		{
			RescASN_IRToSnaccOS(IR_FILE, snaccOperand->a.resultSetId,
			    &irElem->resultSetId);
		}
		
	}
}

/****************************************************************************
 * FUNCTION:  OpSp_IRToSnaccRPNStruct
 *
 * PURPOSE:
 *	This function is used to copy an IR RPN query structure to an apdu 
 *	type 1 search query operand
 *
 * INPUT:
 *	IR_RPN_ELEMENT	- RPN search query element
 *
 * OUTPUT:
 *	RPNStructure		- pointer to an apdu type 1 query operand
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/
void OpSp_IRToSnaccRPNStructSeq(RPNStructure *snaccElem,
	      IR_RPN_COMPLEX_NODE *irElem)
{
	RPNStructureSeq 	*snaccRPNSeq; 
	IR_RPN_ELEMENT		*irRPNElem;

	snaccElem->choiceId = RPNSTRUCTURE_RPNSTRUCTURESEQ;

	/* Malloc the rpn structure sequence and its sub fields */
	if ((snaccElem->a.rPNStructureSeq = 
	    (RPNStructureSeq *)RescIR_Zmalloc(IR_FILE,
	    sizeof(RPNStructureSeq))) != NULL)
	{
		snaccRPNSeq = snaccElem->a.rPNStructureSeq;
		
		if ((snaccRPNSeq->rPNStructure = 
		    (RPNStructure *)RescIR_Zmalloc(IR_FILE,
		    sizeof(RPNStructure))) != NULL)
		{

			irRPNElem = (IR_RPN_ELEMENT *)irElem->rpnElem1;

			if (irRPNElem->rpnElemType == IR_RPN_NODE_SIMPLE)
			{
				OpSp_IRToSnaccRPNOperand(
				    snaccRPNSeq->rPNStructure,
			    	    irRPNElem->rpnOperand);
			}
			else
			{
				OpSp_IRToSnaccRPNStructSeq(
				    snaccRPNSeq->rPNStructure,
			    	    irRPNElem->rpnStructSeq);
			}
		}
		/* 
	 	 * Continue with the second element and perform a recursive 
	 	 * conversion of all sub RPN elements.
	 	 */
		if ((snaccRPNSeq->rPNStructure1 = 
	    	    (RPNStructure *)RescIR_Zmalloc(IR_FILE,
	    	    sizeof(RPNStructure))) != NULL)
		{
			irRPNElem = (IR_RPN_ELEMENT *)irElem->rpnElem2;
			if (irRPNElem->rpnElemType == IR_RPN_NODE_SIMPLE)
			{
				OpSp_IRToSnaccRPNOperand(
				    snaccRPNSeq->rPNStructure1,
			    	    irRPNElem->rpnOperand);
			}
			else
			{
				OpSp_IRToSnaccRPNStructSeq(
				    snaccRPNSeq->rPNStructure1,
			    	    irRPNElem->rpnStructSeq);
			}
		}

		/* Convert the operator */
		if ((snaccRPNSeq->operator =
	    	    (Operator *)RescIR_Malloc(IR_FILE,
		    sizeof(Operator))) != NULL)
		{
			switch (irElem->operator)
			{
			case IR_QUERY_OPER_AND:
				snaccRPNSeq->operator->choiceId = OPERATOR_AND;
				break;
			case IR_QUERY_OPER_OR:
				snaccRPNSeq->operator->choiceId = OPERATOR_OR;
				break;
			case IR_QUERY_OPER_AND_NOT:
				snaccRPNSeq->operator->choiceId = OPERATOR_AND_NOT;
				break;
			default:
				break;
			}
		}
	}
}
