

/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: tm_user.c
 *
 * DESCRIPTION:
 *	This unit contains the functions used get user target info
 *
 * CONTENTS:
 *	IRO_UserInfo	
 *	OpTm_UserLoginInfoFree
 *	OpTm_UserTargetInfoFree
 *	OpTm_UserTargetInfoClone
 *	OpTm_UserGenInfoFree
 *	OpTm_UserGenInfoClone
 *	OpTm_GetUserInfo
 *	OpTm_GetNextUserInfo
 *	OpTm_CloseUserInfoFile
 *	OpTm_UserTargetInfoElemFree
 *	OpTm_UserGenInfoElemFree
 *	OpTm_ConvertUserInfoToRecord
 *	OpTm_ConvertRecordToUserInfo
 *	OpTm_GetTargetFileName
 *	OpTm_UserSessionInfo
 *	OpTm_GetElem
 *	OpTm_PutElem
 *	OpTm_UserInfoInit
 *
 * NOTES:
 *
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
/***************************************************************************/
#include <stdlib.h>
#include <ir.h>
#include <ir_api.h>
#include <resc.h>
#include <rescio.h>
#include <resclist.h>
#include <asn_tool.h>
#include <op/ir_op.h>


static OP_SESSION_INFO *currentUserInfo=(OP_SESSION_INFO *)NULL;
static USHORT	userCount = 1;
static RESC_IO_FILE_STRUCT	filePtr;

/****************************************************************************
 * FUNCTION: IRO_UserInfo 
 *
 * PURPOSE:
 *	This function is used to return user information.
 *
 * INPUT:
 *	IRO_USER_INFO_TYPE	type of info required
 * 	IRO_USER_INFO_REQ*	pointer to request
 *
 * OUTPUT:
 *	IRO_USER_INFO_REPLY*	pointer to return structure
 *	IR_STATUS
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

IR_STATUS	IRO_UserInfo(IRO_USER_INFO_TYPE infoType,
		    IRO_USER_INFO_REQ *req, IRO_USER_INFO_REPLY *reply)
{
	IR_STATUS	status;
	OP_TARGET_INFO_STRUCT	*targetInfo;
	RESC_IO_FILE_STRUCT	targetFilePtr;
	IR_USER_TARGET_INFO	*tgInfo;

	switch (infoType)
	{
		case IRO_UI_LOGIN:
			/* Verify user name */

			/* Verify no other users currently logged in */
			if (currentUserInfo != NULL)
			{
				OpCf_UpdateErrorReport(IR_ERROR_WARNING,
				    IRO_USER_CURRENTLY_LOGGED_IN, IR_FILE);
				return (IR_FAIL);
			}

			/* Get user info from table */	
			status = OpTm_GetUserInfo(&(req->loginName), 
			    &currentUserInfo);
			if (status == IR_FAIL)
			{
				return (IR_FAIL);
			}

			/* Copy user login info to reply */
			reply->loginInfo = (IRO_USER_LOGIN *)RescIR_Zmalloc(
			    IR_FILE, sizeof(IRO_USER_LOGIN));
			if (reply->loginInfo == NULL)
			{
				OpCf_UpdateErrorReport(IR_ERROR_FATAL,
				    IRO_MEMORY_ERROR, IR_FILE);
				return (IR_FAIL);
			}
			RESC_ASN_MALLOCED(reply->loginInfo);
			currentUserInfo->userId = userCount++;
			reply->loginInfo->userId = currentUserInfo->userId;
			reply->loginInfo->privilege = 
			    currentUserInfo->privilege;
			reply->loginInfo->prefLang = currentUserInfo->prefLang;
			reply->loginInfo->isMalloced = TRUE;	

			/* Close file */
			status = OpTm_CloseUserInfoFile();
			if (status == IR_FAIL)
			{
				OpTm_UserLoginInfoFree(reply->loginInfo);
				return (IR_FAIL);
			}
			/* Verify password */
			if ((req->loginPassword.len != 
			    currentUserInfo->password.len)
			    || (strncmp(req->loginPassword.sPtr, 
			    currentUserInfo->password.sPtr, 
			    req->loginPassword.len) != 0))
			{
				OpTm_UserLoginInfoFree(reply->loginInfo);
				OpCf_UpdateErrorReport(IR_ERROR_WARNING,
				    IRO_INVALID_PASSWORD, IR_FILE);
				return (IR_FAIL);
			}
			OpCf_IROMark(OpTm_UserLoginInfoFree, reply->loginInfo);	
			break;
		case IRO_UI_TARGETS:
			/* Verify user id */
			if (currentUserInfo->userId != req->myUserId)
			{
				OpCf_UpdateErrorReport(IR_ERROR_WARNING,
				    IRO_USER_UNKNOWN, IR_FILE);
				return (IR_FAIL);
			}

			/* Get first target from user's table */
			status = OpTm_GetTargetInfo(NULL, 
			    currentUserInfo->targetInfoFile, 
			    &targetFilePtr, &targetInfo);
			if (status == IR_FAIL)
			{
				return (IR_FAIL);
			}

			reply->targetInfo = RescList_Create(IR_FILE);
			if (reply->targetInfo == NULL)
			{
				OpCf_UpdateErrorReport(IR_ERROR_FATAL,
				    IRO_MEMORY_ERROR, IR_FILE);
				return (IR_FAIL);
			}
			/* For each user in list */
			while (targetInfo != NULL)
			{
				/* Get target info */
				tgInfo = (IR_USER_TARGET_INFO *)RescIR_Zmalloc(
				    IR_FILE, sizeof(IR_USER_TARGET_INFO));
				if (tgInfo == NULL)
				{
					OpCf_UpdateErrorReport(IR_ERROR_FATAL,
					    IRO_MEMORY_ERROR, IR_FILE);
					return (IR_FAIL);
				}
				RESC_ASN_MALLOCED(tgInfo);
				
				/* Copy user target info to reply */
				tgInfo->targetId = targetInfo->targetId;
				tgInfo->defSmallSetUpperBound =
				    targetInfo->defSmallSetUpperBound;
				tgInfo->defLargeSetLowerBound =
				    targetInfo->defLargeSetLowerBound;
				tgInfo->defPresentNumber =
				    targetInfo->defPresentNumber;
				tgInfo->defResultSetNamingSupported =
				    targetInfo->defResultSetNamingSupported;

				RESC_ASN_NO_MALLOC(&(tgInfo->displayFormats));
				RESC_ASN_INIT_VS(&(tgInfo->displayFormats));
				status = RescASN_CopyVS(IR_FILE, 
				    &tgInfo->displayFormats, 
				    &targetInfo->displayFormats);
				if (status == IR_FAIL)
				{
					OpCf_UpdateErrorReport(IR_ERROR_FATAL,
					    IRO_MEMORY_ERROR, IR_FILE);
					return (IR_FAIL);
				}

				RESC_ASN_NO_MALLOC(
				    &(tgInfo->defPreferedRecSyntax));
				RESC_ASN_INIT_OI(
				    &(tgInfo->defPreferedRecSyntax));
				status = RescASN_CopyOI(IR_FILE, 
				    &(tgInfo->defPreferedRecSyntax), 
				    &(targetInfo->defPreferedRecSyntax));
				if (status == IR_FAIL)
				{
					OpCf_UpdateErrorReport(IR_ERROR_FATAL,
					    IRO_MEMORY_ERROR, IR_FILE);
					return (IR_FAIL);
				}

				RESC_ASN_NO_MALLOC(&(tgInfo->targetName));
				RESC_ASN_INIT_VS(&(tgInfo->targetName));
				status = RescASN_CopyVS(IR_FILE, 
				    &(tgInfo->targetName), 
				    &(targetInfo->targetName));
				if (status == IR_FAIL)
				{
					OpCf_UpdateErrorReport(IR_ERROR_FATAL,
					    IRO_MEMORY_ERROR, IR_FILE);
					return (IR_FAIL);
				}

				RESC_ASN_NO_MALLOC(
				    &(tgInfo->userAuthentication));
				RESC_ASN_INIT_VS(&(tgInfo->userAuthentication));
				status = RescASN_CopyVS(IR_FILE, 
				    &(tgInfo->userAuthentication), 
				    &(targetInfo->userAuthentication));
				if (status == IR_FAIL)
				{
					OpCf_UpdateErrorReport(IR_ERROR_FATAL,
					    IRO_MEMORY_ERROR, IR_FILE);
					return (IR_FAIL);
				}

				RESC_ASN_NO_MALLOC(
				    &(tgInfo->defElementSetName));
				RESC_ASN_INIT_VS(&(tgInfo->defElementSetName));
				status = RescASN_CopyVS(IR_FILE, 
				    &(tgInfo->defElementSetName), 
				    &(targetInfo->defElementSetName));
				if (status == IR_FAIL)
				{
					OpCf_UpdateErrorReport(IR_ERROR_FATAL,
					    IRO_MEMORY_ERROR, IR_FILE);
					return (IR_FAIL);
				}

				status = RescList_Append(IR_FILE, 
				    reply->targetInfo, (void *)tgInfo, 
				    OpTm_UserTargetInfoElemFree, 
				    (void *)NULL);
				if (status == IR_FAIL)
				{
					OpCf_UpdateErrorReport(IR_ERROR_FATAL,
					    IRO_MEMORY_ERROR, IR_FILE);
					return (IR_FAIL);
				}
				
				OpTm_TargetInfoFree(targetInfo);
				/* Get next target from table */
				status = OpTm_GetNextTargetInfo(&targetFilePtr,
				    &targetInfo);
				if (status == IR_FAIL)
				{
					return (IR_FAIL);
				}
			}
			status = OpTm_CloseTargetInfoFile(&targetFilePtr);
			if (status == IR_FAIL)
			{
				return (IR_FAIL);
			}
			OpCf_IROMark(OpTm_UserTargetInfoFree, 
			    reply->targetInfo);	
			break;
		default:
			OpCf_UpdateErrorReport(IR_ERROR_WARNING,
			    IRO_QUERY_TYPE_NOT_SUPPORTED, IR_FILE);
			return (IR_FAIL);
	}

	return (IR_SUCCESS);
}

/****************************************************************************
 * FUNCTION: OpTm_UserLoginInfoFree 
 *
 * PURPOSE:
 *	This function is used to free user login information.
 *
 * INPUT:
 *	IRO_USER_LOGIN*	pointer to info structure
 *
 * OUTPUT:
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

void	OpTm_UserLoginInfoFree(IRO_USER_LOGIN *reply)
{
	if (reply != NULL)
	{
		if (RESC_ASN_IS_MALLOCED(reply))
		{
			RescIR_Free(IR_FILE, (CHAR *)reply);
		}
	}
}

/****************************************************************************
 * FUNCTION: OpTm_UserTargetInfoFree 
 *
 * PURPOSE:
 *	This function is used to free user target information.
 *
 * INPUT:
 *	IR_LIST*	pointer to info structure
 *
 * OUTPUT:
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

void	OpTm_UserTargetInfoFree(IR_LIST *reply)
{
	if (reply != NULL)
	{
		if (RESC_ASN_IS_MALLOCED(reply))
		{
			RescList_Destroy(IR_FILE, reply);
		}
	}
}

/****************************************************************************
 * FUNCTION: OpTm_UserTargetInfoClone 
 *
 * PURPOSE:
 *	This function is used to copy user gen information.
 *
 * INPUT:
 *	CHAR			pointer to source file
 *	IR_USER_TARGET_INFO	pointer to info structure
 *
 * OUTPUT:
 *	IR_USER_TARGET_INFO	pointer to info structure
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

#if 0
IR_USER_TARGET_INFO	*OpTm_UserTargetInfoClone(CHAR *srcFile, 
				IR_USER_TARGET_INFO *src)
{
	IR_USER_TARGET_INFO	*info;
	IR_STATUS		status;

	info = (IR_USER_TARGET_INFO *)RescIR_Zmalloc(IR_FILE, 
	    sizeof(IR_USER_TARGET_INFO));
	if (info == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
	 	   IRO_MEMORY_ERROR, IR_FILE);
	 	return (NULL);
	}
	RESC_ASN_MALLOCED(info);

	info->targetId = src->targetId;

	RESC_ASN_NO_MALLOC(&(info->targetName));
	RESC_ASN_INIT_VS(&(info->targetName));
	status = RescASN_CopyVS(IR_FILE, 
	    &(info->targetName), &(src->targetName));
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL,
		    IRO_MEMORY_ERROR, IR_FILE);
		return (NULL);
	}
	
	RESC_ASN_NO_MALLOC(&(info->userAuthentication));
	RESC_ASN_INIT_VS(&(info->userAuthentication));
	status = RescASN_CopyVS(IR_FILE, 
	    &(info->userAuthentication), &(src->userAuthentication));
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL,
		    IRO_MEMORY_ERROR, IR_FILE);
		return (NULL);
	}
	
	info->defSmallSetUpperBound = src->defSmallSetUpperBound;
	info->defLargeSetLowerBound = src->defLargeSetLowerBound;
	info->defPresentNumber = src->defPresentNumber;

	RESC_ASN_NO_MALLOC(&(info->defPreferedRecSyntax));
	RESC_ASN_INIT_OI(&(info->defPreferedRecSyntax));
	status = RescASN_CopyOI(IR_FILE, 
	    &(info->defPreferedRecSyntax), &(src->defPreferedRecSyntax));
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL,
		    IRO_MEMORY_ERROR, IR_FILE);
		return (NULL);
	}
	
	RESC_ASN_NO_MALLOC(&(info->defElementSetName));
	RESC_ASN_INIT_VS(&(info->defElementSetName));
	status = RescASN_CopyVS(IR_FILE, 
	    &(info->defElementSetName), &(src->defElementSetName));
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL,
		    IRO_MEMORY_ERROR, IR_FILE);
		return (NULL);
	}
	
	info->defResultSetNamingSupported = src->defResultSetNamingSupported;

	RESC_ASN_NO_MALLOC(&(info->displayFormats));
	RESC_ASN_INIT_VS(&(info->displayFormats));
	status = RescASN_CopyVS(IR_FILE, 
	    &(info->displayFormats), &(src->displayFormats));
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL,
		    IRO_MEMORY_ERROR, IR_FILE);
		return (NULL);
	}
	
	return (info);
	
}
#endif

/****************************************************************************
 * FUNCTION: OpTm_UserGenInfoFree 
 *
 * PURPOSE:
 *	This function is used to free user gen information.
 *
 * INPUT:
 *	IR_LIST*	pointer to info structure
 *
 * OUTPUT:
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

void	OpTm_UserGenInfoFree(IR_LIST *reply)
{
	if (reply != NULL)
	{
		RescList_Destroy(IR_FILE, reply);
	}
}

/****************************************************************************
 * FUNCTION: OpTm_UserGenInfoClone 
 *
 * PURPOSE:
 *	This function is used to copy user gen information.
 *
 * INPUT:
 *	CHAR			pointer to source file
 *	IR_USER_GEN_INFO	pointer to info structure
 *
 * OUTPUT:
 *	IR_USER_GEN_INFO	pointer to info structure
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

IR_USER_GEN_INFO	*OpTm_UserGenInfoClone(CHAR *srcFile, 
				IR_USER_GEN_INFO *src)
{
	IR_USER_GEN_INFO	*info;
	IR_STATUS		status;

	info = (IR_USER_GEN_INFO *)RescIR_Zmalloc(IR_FILE, 
	    sizeof(IR_USER_GEN_INFO));
	if (info == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
	 	   IRO_MEMORY_ERROR, IR_FILE);
	 	return (NULL);
	}
	RESC_ASN_MALLOCED(info);

	RESC_ASN_NO_MALLOC(&(info->name));
	RESC_ASN_INIT_VS(&(info->name));
	status = RescASN_CopyVS(IR_FILE, 
	    &(info->name), &(src->name));
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL,
		    IRO_MEMORY_ERROR, IR_FILE);
		return (NULL);
	}
	
	info->ifLangPref = src->ifLangPref;

	RESC_ASN_NO_MALLOC(&(info->accountCreditInfo));
	RESC_ASN_INIT_OS(&(info->accountCreditInfo));
	status = RescASN_CopyOS(IR_FILE, 
	    &(info->accountCreditInfo), &(src->accountCreditInfo));
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL,
		    IRO_MEMORY_ERROR, IR_FILE);
		return (NULL);
	}
	
	return (info);
	
}

/****************************************************************************
 * FUNCTION: OpTm_GetUserInfo 
 *
 * PURPOSE:
 *	This function is used to return user information.
 *
 * INPUT:
 *	VISIBLE_STRING		key
 *
 * OUTPUT:
 *	OP_SESSION_INFO		*pointer to info structure
 *	IR_STATUS
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

IR_STATUS	OpTm_GetUserInfo(VISIBLE_STRING *key, 
			OP_SESSION_INFO **info)
{
	IR_STATUS	status;
	VISIBLE_STRING		*record;

	/* Open user info file */
	status = RescIO_OpenFile(IR_FILE, OpTm_WorkPath(), 
	    OpTm_UserInfoFileName(), &filePtr);
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING, IRO_FILE_OPEN_ERROR,
		    IR_FILE);
		return (IR_FAIL);
	}
	
	*info = NULL;
	/* Seek to record */
	status = RescIO_Seek(IR_FILE, &filePtr, RESC_IO_USER_INFO, key, NULL);
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING, IRO_FILE_READ_ERROR,
		    IR_FILE);
		return (IR_FAIL);
	}
	else if (RescIO_EOF(filePtr))
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING, IRO_USER_UNKNOWN,
		    IR_FILE);
		return (IR_FAIL);
	}

	/* Get record */
	status = RescIO_GetRecord(IR_FILE, filePtr, &record);
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING, IRO_FILE_READ_ERROR,
		    IR_FILE);
		return (IR_FAIL);
	}

	status = OpTm_ConvertRecordToUserInfo(record, info);
	if (status == IR_FAIL)
	{
		return (IR_FAIL);
	}
	RescASN_DestroyVS(IR_FILE, record);
	
	return (IR_SUCCESS);
}

/****************************************************************************
 * FUNCTION: OpTm_GetNextUserInfo 
 *
 * PURPOSE:
 *	This function is used to return user information.
 *
 * INPUT:
 *
 * OUTPUT:
 *	OP_SESSION_INFO		*pointer to info structure
 *	IR_STATUS
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

#if 0
IR_STATUS	OpTm_GetNextUserInfo(OP_SESSION_INFO **info)
{
	IR_STATUS	status;
	VISIBLE_STRING		*record;

	*info = NULL;
	/* Seek to start of next record */
	status = RescIO_Seek(IR_FILE, &filePtr, 
	    RESC_IO_USER_INFO, NULL, NULL);
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING, 
		    IRO_FILE_READ_ERROR, IR_FILE);
		return (IR_FAIL);
	}
	if (!RescIO_EOF(filePtr))
	{
		status = RescIO_GetRecord(IR_FILE, filePtr, &record);
		if (status == IR_FAIL)
		{
			OpCf_UpdateErrorReport(IR_ERROR_WARNING, 
			    IRO_FILE_READ_ERROR, IR_FILE);
			return (IR_FAIL);
		}

		status = OpTm_ConvertRecordToUserInfo(record, info);
		if (status == IR_FAIL)
		{
			return (IR_FAIL);
		}
		RescASN_DestroyVS(IR_FILE, record);
	}
	
	return (IR_SUCCESS);
}
#endif

/****************************************************************************
 * FUNCTION: OpTm_CloseUserInfoFile 
 *
 * PURPOSE:
 *	This function is used to close the user info file.
 *
 * INPUT:
 *
 * OUTPUT:
 *	IR_STATUS
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

IR_STATUS	OpTm_CloseUserInfoFile()
{
	IR_STATUS status;

	/* Close user info file */
	status = RescIO_CloseFile(IR_FILE, &filePtr);
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING, IRO_FILE_READ_ERROR,
		    IR_FILE);
		return (IR_FAIL);
	}

	return (IR_SUCCESS);
}

/****************************************************************************
 * FUNCTION: OpTm_UserTargetInfoElemFree 
 *
 * PURPOSE:
 *	This function is used to free user target information.
 *
 * INPUT:
 *	CHAR 			pointer to source file
 *	IR_USER_TARGET_INFO	pointer to info structure
 *
 * OUTPUT:
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

void	OpTm_UserTargetInfoElemFree(CHAR *srcFile, IR_USER_TARGET_INFO *info)
{
	if (info != NULL)
	{
		RescASN_DestroyVS(IR_FILE, &info->targetName);
		RescASN_DestroyVS(IR_FILE, &info->userAuthentication);
		RescASN_DestroyVS(IR_FILE, &info->defElementSetName);
		RescASN_DestroyVS(IR_FILE, &info->defPreferedRecSyntax);
		RescASN_DestroyVS(IR_FILE, &info->displayFormats);
		if (info->isMalloced)
		{
			RescIR_Free(IR_FILE, (CHAR *)info);
		}	
	}
}

/****************************************************************************
 * FUNCTION: OpTm_UserGenInfoElemFree 
 *
 * PURPOSE:
 *	This function is used to free user general information.
 *
 * INPUT:
 *	CHAR 			pointer to source file
 *	IR_USER_GEN_INFO	pointer to info structure
 *
 * OUTPUT:
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

void	OpTm_UserGenInfoElemFree(CHAR *srcFile, IR_USER_GEN_INFO *info)
{
	if (info != NULL)
	{
		RescASN_DestroyVS(IR_FILE, &(info->name));
		RescASN_DestroyOS(IR_FILE, &(info->accountCreditInfo));
		if (info->isMalloced)
		{
			RescIR_Free(IR_FILE, (CHAR *)info);
		}	
	}
}

/****************************************************************************
 * FUNCTION: OpTm_ConvertRecordToUserInfo 
 *
 * PURPOSE:
 *	This function is used to convert a record to the user info struct.
 *
 * INPUT:
 *	VISIBLE_STRING	pointer to the record 
 *
 * OUTPUT:
 *	OP_SESSION_INFO 	pointer to pointer to info	
 *	IR_STATUS
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

IR_STATUS OpTm_ConvertRecordToUserInfo(VISIBLE_STRING *record, 
			OP_SESSION_INFO **infoPtr)
{
	OP_SESSION_INFO		*info;
	IR_STATUS		status;
	CHAR			*elem;
	CHAR			*buf;
	USHORT			tag;
	ULONG			len;
	BOOLEAN			recEnd;
	ULONG			ePtr;

	/* Allocate space for info structure */
	info = (OP_SESSION_INFO *)RescIR_Zmalloc(IR_FILE, 
	    sizeof(OP_SESSION_INFO));
	if (info == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
	 	   IRO_MEMORY_ERROR, IR_FILE);
	 	return (IR_FAIL);
	}
	RESC_ASN_MALLOCED(info);
	OpTm_UserInfoInit(info);
	*infoPtr = info;

	/* Get first tag in record */
	ePtr = 0;
	buf = OpTm_GetElem(*record, 0, RESC_IO_TAG_LENGTH);
	if (buf == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
		    IRO_MEMORY_ERROR, IR_FILE);
	        return (IR_FAIL);
	}
	tag = (USHORT)atoi(buf);

	/* Get length of first element in record */
	buf = OpTm_GetElem(*record, RESC_IO_TAG_LENGTH, 
	    RESC_IO_LEN_LENGTH);
	if (buf == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
		    IRO_MEMORY_ERROR, IR_FILE);
	        return (IR_FAIL);
	}
	len = atol(buf);

	/* Get first element in record */
	elem = OpTm_GetElem(*record, 
	    (RESC_IO_TAG_LENGTH + RESC_IO_LEN_LENGTH), len);
	if (elem == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
		    IRO_MEMORY_ERROR, IR_FILE);
	        return (IR_FAIL);
	}

	/* While not end of record */
	recEnd = FALSE;
	while (!recEnd)
	{
		/* Add each record element to the info structure */
		switch (tag)
		{
			case OP_USER_NAME:
				status = RescASN_AddVS(IR_FILE, 
				    &info->userName, elem, (SHORT)len);
				break;
			case OP_PASSWORD:
				status = RescASN_AddVS(IR_FILE, 
				    &info->password, elem, (SHORT)len);
				break;
			case OP_PRIVILEGE:
				info->privilege = (USHORT)atoi(elem);
				break;
			case OP_PREF_LANG:
				info->prefLang = (USHORT)atoi(elem);
				break;
			case OP_CREDIT:
				status = RescASN_AddOS(IR_FILE, 
				    &info->accountCreditInfo, elem, (SHORT)len);
				break;
			case OP_TRGT_FILE:
				status = RescASN_AddVS(IR_FILE, 
				    &info->targetInfoFile, elem, (SHORT)len);
				break;
			default:
				OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
				    IRO_DATA_ERROR, IR_FILE);
				return (IR_FAIL);
		}
		if (status == IR_FAIL)
		{
			OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
			    IRO_MEMORY_ERROR, IR_FILE);
			return (IR_FAIL);
		}
	
		/* Get next tag */
		ePtr = ePtr + RESC_IO_TAG_LENGTH + RESC_IO_LEN_LENGTH + len;
		buf = OpTm_GetElem(*record, ePtr, RESC_IO_TAG_LENGTH);
		if (buf == NULL)
		{
			recEnd = TRUE;
		}
		else
		{
			/* if not end of record, get next element */
			tag = (USHORT)atoi(buf);
			buf = OpTm_GetElem(*record, (ePtr + RESC_IO_TAG_LENGTH),
			    RESC_IO_LEN_LENGTH);
			if (buf == NULL)
			{
				OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
				    IRO_MEMORY_ERROR, IR_FILE);
			        return (IR_FAIL);
			}
			len = atol(buf);
			elem = OpTm_GetElem(*record, 
			    (ePtr + RESC_IO_TAG_LENGTH + RESC_IO_LEN_LENGTH), 
			    len);
		}
	}

	info->useWorkSpace = FALSE;

	return (IR_SUCCESS);
}

/****************************************************************************
 * FUNCTION: OpTm_ConvertUserInfoToRecord 
 *
 * PURPOSE:
 *	This function is used to convert user info to a record.
 *
 * INPUT:
 *	OP_SESSION_INFO 	pointer to pointer to record	
 *
 * OUTPUT:
 *	VISIBLE_STRING	pointer to the user info 
 *	IR_STATUS
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

IR_STATUS OpTm_ConvertUserInfoToRecord(OP_SESSION_INFO *info,
				VISIBLE_STRING **recordPtr)
{
	VISIBLE_STRING		*record;
	IR_STATUS		status;
	CHAR			buf[RESC_IO_MAX_RECORD_SIZE];
	CHAR			tmpBuf[RESC_IO_MAX_RECORD_SIZE];
	CHAR			*rec;

	/* Create record structure */
	record = RescASN_CreateVS(IR_FILE);
	if (record == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
		    IRO_MEMORY_ERROR, IR_FILE);
		    return (IR_FAIL);
	}
	*recordPtr = record;
	rec = buf;

	/* Concatenate each element of the info structure to a buffer */
	OpTm_PutElem(rec, OP_USER_NAME, info->userName.sPtr, 
	    info->userName.len, (ULONG *)&rec);

	OpTm_PutElem(rec, OP_PASSWORD, info->password.sPtr, info->password.len,
	    (ULONG *)&rec);

	sprintf(tmpBuf, "%d", info->privilege);
	OpTm_PutElem(rec, OP_PRIVILEGE, tmpBuf, strlen(tmpBuf), (ULONG *)&rec);

	sprintf(tmpBuf, "%d", info->prefLang);
	OpTm_PutElem(rec, OP_PREF_LANG, tmpBuf, strlen(tmpBuf), (ULONG *)&rec);

	OpTm_PutElem(rec, OP_CREDIT, info->accountCreditInfo.sPtr, 
	    info->accountCreditInfo.len, (ULONG *)&rec);

	OpTm_PutElem(rec, OP_TRGT_FILE, info->targetInfoFile.sPtr, 
	    info->targetInfoFile.len, (ULONG *)&rec);

	/* Copy buffer to record */
	status = RescASN_AddVS(IR_FILE, record, buf, strlen(buf));
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
		    IRO_MEMORY_ERROR, IR_FILE);
		    return (IR_FAIL);
	}

	return (IR_SUCCESS);
}

/****************************************************************************
 * FUNCTION: OpTm_GetTargetFileName 
 *
 * PURPOSE:
 *	This function is used to return the target file name.
 *
 * INPUT:
 *	IR_USER_ID		userId	
 *
 * OUTPUT:
 *	VISIBLE_STRING	*pointer to file name 
 *	IR_STATUS
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

IR_STATUS	OpTm_GetTargetFileName(IR_USER_ID userId, 
			VISIBLE_STRING *targetInfoFileName)
{
	IR_STATUS	status;

	/* Verify user id against current user Id */
	if ((currentUserInfo == NULL) ||
	    (currentUserInfo->userId != userId))
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING,
		    IRO_USER_UNKNOWN, IR_FILE);
		return (IR_FAIL);
	}

	/* Return targetInfoFileName */
	RESC_ASN_NO_MALLOC(targetInfoFileName);
	RESC_ASN_INIT_VS(targetInfoFileName);
	if (targetInfoFileName == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
		    IRO_MEMORY_ERROR, IR_FILE);
		    return (IR_FAIL);
	}

	status = RescASN_CopyVS(IR_FILE, targetInfoFileName, 
	    &(currentUserInfo->targetInfoFile));
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL, 
		    IRO_MEMORY_ERROR, IR_FILE);
		    return (IR_FAIL);
	}

	return (IR_SUCCESS);
}

/****************************************************************************
 * FUNCTION: OpTm_UserSessionInfo 
 *
 * PURPOSE:
 *	This function is used to return the current user information.
 *
 * INPUT:
 *	IR_USER_ID		userId	
 *
 * OUTPUT:
 *	OP_SESSION_INFO	*pointer to info 
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

OP_SESSION_INFO	*OpTm_UserSessionInfo(IR_USER_ID userId) 
{
	if ((currentUserInfo == NULL) || (currentUserInfo->userId != userId))
	{
		return ((OP_SESSION_INFO *)NULL);
	}

	return(currentUserInfo);
}

/****************************************************************************
 * FUNCTION: OpTm_UserId 
 *
 * PURPOSE:
 *	This function is used to return the current user id.
 *
 * INPUT:
 *
 * OUTPUT:
 *	IR_USER_ID	id 
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

IR_USER_ID	OpTm_UserId() 
{
	if (currentUserInfo == NULL)
	{
		return(IR_UNKNOWN_ID_TYPE);
	}

	return(currentUserInfo->userId);
}

/****************************************************************************
 * FUNCTION: OpTm_GetElem 
 *
 * PURPOSE:
 *	This function is used to get the next element out of a record.
 *
 * INPUT:
 *	VISIBLE_STRING	record
 *	ULONG		start position
 *	ULONG		length of element
 *
 * OUTPUT:
 *	CHAR		pointer to element
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

CHAR	*OpTm_GetElem(VISIBLE_STRING record, ULONG start, ULONG len) 
{
	static CHAR	buf[IR_LEN_500];

	/* if end of record */
	if ((start + len) > record.len)
	{
		return ((CHAR *)NULL);
	}

	/* Copy element */
	if (len > IR_LEN_500)
	{
		CS_LOG_PRINTS(IR_FILE, "major kludge finds an error!!!\n");
		len = IR_LEN_500;
	}
	memcpy(buf, (record.sPtr + start), (SHORT)len);
	buf[len] = EOS;
	return (buf);
}
	
/****************************************************************************
 * FUNCTION: OpTm_PutElem 
 *
 * PURPOSE:
 *	This function is used to put the next element into a record.
 *
 * INPUT:
 *	CHAR		start position
 *	USHORT		tag
 *	CHAR		element
 *	USHORT		length of element
 *
 * OUTPUT:
 * 	ULONG		pointer to next address
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

void	OpTm_PutElem(CHAR *start, USHORT tag, CHAR *elem, ULONG len, 
	 		ULONG *end) 
{
	CHAR	buf[RESC_IO_LEN_LENGTH+1];
	CHAR	buf1[1000];
	CHAR	*s;
	CHAR	*ptr;

	if (len != 0)
	{
		/* Add tag to record */
		ptr = start;
		sprintf(buf, "%4d", tag);
		strcpy(start, buf);

		/* Add element to record to the tmp buffer */	
		s = buf1;
		while ((*elem != EOS) && (len > 0))
		{
			*s++ = *elem++;
			len--;
		}
		*s = EOS; 
		len = strlen(buf1);

		/* Add element length to record */
		sprintf(buf, "%10d", len);
		strcat(start, buf);

		/* Now copy temporary buf1 to the record */
		strcat(start, buf1);

		ptr = &start[strlen(start)];
		*end = (ULONG)ptr;
	}
}
	
/****************************************************************************
 * FUNCTION: OpTm_UserInfoInit 
 *
 * PURPOSE:
 *	This function is initialize a user info struct.
 *
 * INPUT:
 *	OP_SESSION_INFO		pointer to user info struct
 *
 * OUTPUT:
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/
void OpTm_UserInfoInit(OP_SESSION_INFO *info)
{
	RESC_ASN_NO_MALLOC(&(info->userName));
	RESC_ASN_INIT_VS(&(info->userName));
	RESC_ASN_NO_MALLOC(&(info->password));
	RESC_ASN_INIT_VS(&(info->password));
	RESC_ASN_NO_MALLOC(&(info->accountCreditInfo));
	RESC_ASN_INIT_OS(&(info->accountCreditInfo));
	RESC_ASN_NO_MALLOC(&(info->targetInfoFile));
	RESC_ASN_INIT_VS(&(info->targetInfoFile));
}
