
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: OpWs  Work Space Info
 *
 * DESCRIPTION:
 *	This unit provides the functions required to support the
 *	IRO UA/OP API work space specifications.
 *
 * CONTENTS:
 *	IRO_WorkSpaceInfo
 *
 * NOTES:
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 *
/***************************************************************************/

#include <ir.h>
#include <ir_api.h>
#include <ir_ali.h>
#include <resc.h>
#include <resclist.h>
#include <op/ir_op.h>

static void OpWs_WsRecordListFree(IR_LIST *wsList);
static IRO_WS_GEN_INFO *OpWs_WsGetAllGenInfo(void);
static void OpWs_WsGenInfoFree(IRO_WS_GEN_INFO *);
static void OpWs_CopyWsInfoToHdrInfo(IRO_WS_HEADER_INFO *hdr, 
		OP_WS_INFO *wsInfo);
static void OpWs_DestroyHdrInfo(CHAR *src, void *data);
static void *OpWs_CopyHdrInfo(CHAR *src, void *data);
static IRO_WS_HEADER_INFO *OpWs_WsCreateHdrInfo();


extern IR_LIST	*OpWs_WsList;

/****************************************************************************
 * FUNCTION:  IRO_WorkSpaceInfo
 *
 * PURPOSE:
 *	This function supports all of the IRO Work Space operations. These
 *	include all of the following:
 *
 *	IRO_WS_INFO_HEADER		- Obtain general info 
 *	IRO_WS_INFO_SEARCH		- Retrieve initiating search
 *	IRO_WS_INFO_REC_ALL		- Get all records from a ws
 *	IRO_WS_INFO_PARTIAL		- Get n records from a ws
 *	IRO_WS_INFO_SPECIFIC		- Get specific records from a ws
 *	IRO_WS_INFO_DIAG		- Get non-surr diag rec for a ws
 *	IRO_WS_INFO_GEN_ALL		- Obtain a list of work spaces 
 *
 * INPUT:
 *	none
 *
 * OUTPUT:
 *	none
 *
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

IR_STATUS	IRO_WorkSpaceInfo(IR_USER_ID userId, 
		    IRO_WS_INFO_TYPE reqType,
		    IRO_WORK_SPACE_REQ *req,
		    IRO_WORK_SPACE_REPLY *reply)
{

	OP_SESSION_INFO	*sessionInfo;
	OP_WS_INFO	*ws;
	IRO_WS_GEN_INFO	*genInfo;

CS_LOG_PRINTV(IR_FILE, "IRO_WorkSpaceInfo. Type = %d\n",
    reqType);

	/* Verify the user Id */
	if ((sessionInfo = OpTm_UserSessionInfo(userId)) == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING, 
		    IRO_UNKNOWN_USER_ID, IR_FILE);
		return (IR_FAIL);
	}

	switch(reqType)
	{
	case IRO_WS_INFO_REC_ALL:
		CS_LOG_PRINTV(IR_FILE, "wsId = %d\n", req->wsId);
		/* Get the work space */
		if ((ws = OpWs_WsGet(req->wsId)) == NULL)
		{
			OpCf_UpdateErrorReport(IR_ERROR_WARNING, 
		    	    IRO_WORKSPACE_DOES_NOT_EXIST, IR_FILE);
			return (IR_FAIL);
		}
		reply->records = &ws->records;
		return(IR_SUCCESS);
		break;
	case IRO_WS_INFO_GEN_ALL:
		genInfo = OpWs_WsGetAllGenInfo();
		reply->genInfo = genInfo;

		OpCf_IROMark(OpWs_WsGenInfoFree, genInfo);
CS_LOG_PRINTV(IR_FILE, "Returning gen info %lx\n", genInfo);
		return(IR_SUCCESS);
		break;
	default:
		OpCf_UpdateErrorReport(IR_ERROR_WARNING,
		    IRO_UNSUPPORTED_REQ_TYPE ,IR_FILE);
		return(IR_FAIL);
	}

}

static void OpWs_WsRecordListFree(IR_LIST *wsList)
{

	if (wsList)
	{
		RescList_Destroy(IR_FILE, wsList);
	}
}


static IRO_WS_GEN_INFO *OpWs_WsGetAllGenInfo()
{
	IRO_WS_GEN_INFO	*info;
	IRO_WS_HEADER_INFO *hdr;
	SHORT		index;
	OP_WS_INFO 	*wsInfo;


	info = (IRO_WS_GEN_INFO *)RescIR_Malloc(IR_FILE,
	    sizeof(IRO_WS_GEN_INFO));

	RESC_ASN_MALLOCED(info);
	RESC_ASN_NO_MALLOC(&info->wsHeaderInfo);
	RESC_LIST_INIT(&info->wsHeaderInfo);

	
	RESC_LIST_SET_FIRST(OpWs_WsList);
	for (index = 0; index < RESC_LIST_SIZE(OpWs_WsList);
	    index++)
	{
		wsInfo = RESC_LIST_GET(OpWs_WsList);

		hdr = OpWs_WsCreateHdrInfo();

		if (!hdr)
		{
			/* Bail out */
			return(info);
		}

CS_LOG_PRINTV(IR_FILE, "Getting genInfo for ws index %d\n", index);
		/* Copy wsInfo data to hdr structure */
		OpWs_CopyWsInfoToHdrInfo(hdr, wsInfo);

CS_LOG_PRINTS(IR_FILE, "Appending entry to geninfo list\n");
		/* Save hdr structure in genInfo list */
		RescList_Append(IR_FILE, &info->wsHeaderInfo,
		    hdr, OpWs_DestroyHdrInfo, OpWs_CopyHdrInfo);
CS_LOG_PRINTS(IR_FILE, "Setting entry to next\n");
		RESC_LIST_SET_NEXT(OpWs_WsList);
	}
	

	return(info);
}

static void OpWs_WsGenInfoFree(IRO_WS_GEN_INFO *genInfo)
{
	if (!genInfo)
	{
		return;
	}

	RescList_Destroy(IR_FILE, &genInfo->wsHeaderInfo);
	if (RESC_ASN_IS_MALLOCED(genInfo))
	{
		RescIR_Free(IR_FILE, (CHAR *)genInfo);
	}
}

static IRO_WS_HEADER_INFO *OpWs_WsCreateHdrInfo()
{
	IRO_WS_HEADER_INFO *new;

	new = (IRO_WS_HEADER_INFO *)RescIR_Zmalloc(IR_FILE, 
	    sizeof(IRO_WS_HEADER_INFO));

	if (!new)
	{
		CS_LOG_PRINTS(IR_FILE, "malloc error\n");
		return(new);
	}

	/* Initialize all elements in the new structure */
	RESC_ASN_MALLOCED(new);
	
	new->wsId = IR_UNKNOWN_ID_TYPE;
	
	RESC_ASN_NO_MALLOC(&new->name);
	RESC_ASN_INIT_VS(&new->name);

	RESC_ASN_NO_MALLOC(&new->nonSurrDiag);
	RESC_ASN_NO_MALLOC(&new->nonSurrDiag.diagnosticSetId);
	RESC_ASN_INIT_OI(&new->nonSurrDiag.diagnosticSetId);
	RESC_ASN_NO_MALLOC(&new->nonSurrDiag.addInfo);
	RESC_ASN_INIT_OI(&new->nonSurrDiag.addInfo);

	/* All other fields are cleared using the zmalloc */
}

static void OpWs_CopyWsInfoToHdrInfo(IRO_WS_HEADER_INFO *hdr, 
		OP_WS_INFO *wsInfo)
{
	hdr->wsId = wsInfo->wsId;
	RescASN_CopyVS(IR_FILE, &hdr->name, &wsInfo->resultSetName);
	hdr->assocId = wsInfo->assocId;
	hdr->totalTargetRecords = wsInfo->totalTargetRecords;
	hdr->numWSRecords = wsInfo->numWSRecords;
	hdr->nextTargetRec = wsInfo->nextTargetRec;
	hdr->searchStatus = wsInfo->searchStatus;
	hdr->resultSetStatus = wsInfo->resultSetStatus;
	hdr->presentStatus = wsInfo->presentStatus;
	hdr->targetResultsDeleted = wsInfo->targetResultsDeleted;
	hdr->presenting = wsInfo->presenting;
	hdr->deleting = wsInfo->deleting;

	RescASN_CopyOI(IR_FILE, &hdr->nonSurrDiag.diagnosticSetId,
	    &wsInfo->nonSurrDiag.diagnosticSetId);
	RescASN_CopyVS(IR_FILE, &hdr->nonSurrDiag.addInfo,
	    &wsInfo->nonSurrDiag.addInfo);
	hdr->nonSurrDiag.condition = wsInfo->nonSurrDiag.condition;

	/* 
	 * Use the wsInfo assocId to determine if 
	 * remote present and delete are supported.
	 */
	hdr->presentSrvcSupported = 
	    OpAm_AssocServiceAllowed(OPAM_PRESENT, wsInfo->assocId);
	hdr->deleteSrvcSupported = 
	    OpAm_AssocServiceAllowed(OPAM_DELETE, wsInfo->assocId);
};

static void OpWs_DestroyHdrInfo(CHAR *src, void *data)
{
	IRO_WS_HEADER_INFO	*info;

	if (!data)
	{
		return;
	}

	info = (IRO_WS_HEADER_INFO *)data;

	RescASN_DestroyVS(src, &info->name);
	RescASN_DestroyOI(src, &info->nonSurrDiag.diagnosticSetId);
	RescASN_DestroyOI(src, &info->nonSurrDiag.addInfo);

	if (RESC_ASN_IS_MALLOCED(info))
	{
		RescIR_Free(IR_FILE, (CHAR *)info);
	}
}

static void *OpWs_CopyHdrInfo(CHAR *src, void *data)
{
	IRO_WS_HEADER_INFO	*new;
	IRO_WS_HEADER_INFO	*from;

	if (!data)
	{
		CS_LOG_PRINTS(src, "Unknown source data\n");
		return((void *)NULL);
	}

	new = OpWs_WsCreateHdrInfo();
	if (!new)
	{
		CS_LOG_PRINTS(src, "Cannot malloc new\n");
		return(new);
	}
	from = (IRO_WS_HEADER_INFO *)data;

	new->wsId = from->wsId;
	RescASN_CopyVS(IR_FILE, &new->name, &from->name);
	new->assocId = from->assocId;
	new->totalTargetRecords = from->totalTargetRecords;
	new->numWSRecords = from->numWSRecords;
	new->nextTargetRec = from->nextTargetRec;
	new->searchStatus = from->searchStatus;
	new->resultSetStatus = from->resultSetStatus;
	new->presentStatus = from->presentStatus;
	new->targetResultsDeleted = from->targetResultsDeleted;

	RescASN_CopyOI(IR_FILE, &new->nonSurrDiag.diagnosticSetId,
	    &from->nonSurrDiag.diagnosticSetId);
	RescASN_CopyVS(IR_FILE, &new->nonSurrDiag.addInfo,
	    &from->nonSurrDiag.addInfo);
	new->nonSurrDiag.condition = from->nonSurrDiag.condition;

	new->presentSrvcSupported = from->presentSrvcSupported;
	new->deleteSrvcSupported = from->deleteSrvcSupported;

	return(new);
}
