
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/
/****************************************************************************
 * UNIT: OpWs  Work Space Maintenance
 *
 * DESCRIPTION:
 *	This unit contains functions used for the maintenance of the
 *	work space entries which are maintained from the time that a
 *	search is sent until the work space is explicitly deleted by the
 *	user application or until the end of the user session.
 *
 * CONTENTS:
 *	OpWs_WsInit
 *	OpWs_WsCreate
 *	OpWs_WsGet
 *	OpWs_WsDelete
 *	OpWs_WsGetAllAssoc
 *	local OpWs_WsFind
 *	local OpWs_WsDestroy
 *	local OpWs_WsCopy
 *
 * NOTES:
 *
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
/***************************************************************************/

#include <ir.h>
#include <ir_api.h>
#include <ir_ali.h>
#include <resc.h>
#include <resclist.h>
#include <op/ir_op.h>

IR_LIST	*OpWs_WsList;
static IR_WS_ID OpWs_GenId=0;

static SHORT OpWs_WsFind(IR_ASSOC_ID wsId);
static void  OpWs_WsDestroy(CHAR *srcFile, void *infoPtr);
static void  *OpWs_WsCopy(CHAR *srcFile, void *data);
static OP_WS_INFO *OpWs_WsNew();

/****************************************************************************
 * FUNCTION:  OpWs_WsInit
 *
 * PURPOSE:
 *	Initialize the workspace list.
 *
 * INPUT:
 *	none
 *
 * OUTPUT:
 *	none
 *
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

void OpWs_WsInit(void)
{
	OpWs_WsList = RescList_Create(IR_FILE);
	if (OpWs_WsList == (IR_LIST *)NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL, IRO_MEMORY_ERROR,
		    IR_FILE);
	}
}


/****************************************************************************
 * FUNCTION:  OpWs_WsCreate
 *
 * PURPOSE:
 *	This function will create a new entry in the list and install an
 *	empty OP_WS_INFO structure into the data field of the list element.
 *
 * INPUT:
 *	IR_USER_ID
 *
 * OUTPUT:
 *	OP_WS_INFO	*		- pointer to new entry.
 *
 *
 * NOTES:
 *	There are no notes. 
 *
 ***************************************************************************/

OP_WS_INFO *OpWs_WsCreate(IR_USER_ID userId)
{
	OP_WS_INFO *wsInfo;

	wsInfo = OpWs_WsNew();

	/* Create new Id */
	OpWs_GenId++;
	wsInfo->userId = userId;
	wsInfo->wsId = OpWs_GenId;

	RescList_Append(IR_FILE, OpWs_WsList, wsInfo, 
	    OpWs_WsDestroy, OpWs_WsCopy);

	return (wsInfo);
}

/****************************************************************************
 * FUNCTION:  OpWs_WsGet
 *
 * PURPOSE:
 *	This function will get a work space for a given userId and work space
 *	id.
 *
 * INPUT:
 *	IR_WS_ID			- work space id.
 *
 * OUTPUT:
 *	OP_WS_INFO	pointer
 *
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

OP_WS_INFO *OpWs_WsGet(IR_WS_ID wsId)
{
	SHORT		pos;
	OP_WS_INFO 	*wsInfo;

	pos = OpWs_WsFind(wsId);

	if (pos == IR_UNKNOWN_ID_TYPE)
	{
		return((OP_WS_INFO *)NULL);
	}

	RESC_LIST_SET_CUR(OpWs_WsList, pos);

	wsInfo = RESC_LIST_GET(OpWs_WsList);

	return (wsInfo);
}

/****************************************************************************
 * FUNCTION:  OpWs_WsDelete
 *
 * PURPOSE:
 *	This function is used to remove a workspace entry from the
 *	work space list.
 *
 * INPUT:
 *	IR_WS_ID		- entry to remove
 *
 * OUTPUT:
 *	IR_STATUS
 *
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

IR_STATUS 	OpWs_WsDelete(IR_WS_ID wsId)
{
	SHORT	pos;

	pos = OpWs_WsFind(wsId);

	if (pos == IR_UNKNOWN_ID_TYPE)
	{
		return(IR_SUCCESS);
	}

	/* Remove from link */
	RescList_Delete(IR_FILE, OpWs_WsList, pos);

	return (IR_SUCCESS);
}

/****************************************************************************
 * FUNCTION:  OpWs_WsListFree
 *
 * PURPOSE:
 *	Free assoc entries and assoc list
 *
 * INPUT:
 *	none
 *
 * OUTPUT:
 *	none
 *
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

void OpWs_WsListFree(void)
{

	RescList_Destroy(IR_FILE, OpWs_WsList);

}

/****************************************************************************
 * FUNCTION:  OpWs_WsGetAllAssoc
 *
 * PURPOSE:
 *	Generate an array of wsIds that belong to a specified association.
 *	The list will be an IR_UNKNOWN_ID_TYPE terminated list and must be
 *	freed by the calling procedure.
 *
 * INPUT:
 *	IR_ASSOC_ID
 *
 * OUTPUT:
 *	array of IR_WS_ID terminated by IR_UNKNOWN_ID_TYPE.
 *
 * NOTES:
 *	It is the responsibility of the calling procedure to free the
 *	malloced array.
 *
 ***************************************************************************/

IR_WS_ID *OpWs_WsGetAllAssoc(IR_ASSOC_ID assocId)
{
	SHORT	index;
	SHORT	resultCnt;
	OP_WS_INFO	*ws;
	IR_WS_ID	*result;


	/* Malloc an array large enough to hold all wsIds. */
	result = (IR_WS_ID *)RescIR_Malloc(IR_FILE, 
	    (RESC_LIST_SIZE(OpWs_WsList)+1) * sizeof(IR_WS_ID));

	RESC_LIST_SET_FIRST(OpWs_WsList);
	resultCnt = 0;
	for (index = 0;	index < RESC_LIST_SIZE(OpWs_WsList); index++)
	{
		ws = RESC_LIST_GET(OpWs_WsList);
		if (ws->assocId == assocId)
		{
			result[resultCnt++] = ws->wsId;
		}
		RESC_LIST_SET_NEXT(OpWs_WsList);
		
	}
	result[resultCnt] = IR_UNKNOWN_ID_TYPE;

	return(result);
}

/****************************************************************************
 * LOCAL FUNCTION:  OpWs_WsFind
 *
 * PURPOSE:
 *	This local function searches for the work space entry that corresponds
 *	to the wsId supplied. The list index of the matching entry
 *	is returned.
 *
 * INPUT:
 *	IR_WS_ID
 *
 * OUTPUT:
 *	index into list. (IR_UNKNOWN_ID_TYPE) if not found.
 *
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

static SHORT	OpWs_WsFind(IR_WS_ID wsId)
{
	SHORT		pos;
	OP_WS_INFO	*wsInfo;

	if (RESC_LIST_SIZE(OpWs_WsList) == 0)
	{
		return(IR_UNKNOWN_ID_TYPE);
	}

	RESC_LIST_SET_FIRST(OpWs_WsList);
	for (pos = 0; pos < RESC_LIST_SIZE(OpWs_WsList); pos++)
	{
		wsInfo = RESC_LIST_GET(OpWs_WsList);

		if (wsInfo->wsId == wsId)
		{
			/* This is the one. */
			break;
		}
		RESC_LIST_SET_NEXT(OpWs_WsList);
	} 

	if (pos == RESC_LIST_SIZE(OpWs_WsList))
	{
		return(IR_UNKNOWN_ID_TYPE);
	}
	else
	{
		/* IR List counts start at zero. */
		return(pos);
	}
}

/****************************************************************************
 * LOCAL FUNCTION:  OpWs_WsDestroy
 *
 * PURPOSE:
 *	This local function will destroy a work space entry. It will free all
 *	malloced data.
 *
 * INPUT:
 *	pointer to work space entry.
 *
 * OUTPUT:
 *	void
 *
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

void  OpWs_WsDestroy(CHAR *srcFile, OP_WS_INFO *wsPtr)
{

	if (wsPtr == (OP_WS_INFO *)NULL)
	{
		return;
	}
	
	RescASN_DestroyVS(IR_FILE, &wsPtr->resultSetName);
	RescASN_DestroyOS(IR_FILE, &wsPtr->nonSurrDiag.diagnosticSetId);
	RescASN_DestroyVS(IR_FILE, &wsPtr->nonSurrDiag.addInfo);
	RescList_Destroy(IR_FILE, &wsPtr->records);


	/* Destroy the work-space structure */
	if (RESC_ASN_IS_MALLOCED(wsPtr))
	{
		RescIR_Free(IR_FILE, (CHAR *)wsPtr);
	}
}

/****************************************************************************
 * LOCAL FUNCTION:  OpWs_WsCopy
 *
 * PURPOSE:
 *	This function is used to create a copy of a op work space info
 *	structure. Most often this function is used to copy RescList_
 *	elements.
 *
 * INPUT:
 *	pointer to source of caller
 *	pointer to wsInfo structure to be copied
 *
 * OUTPUT:
 *	pointer to new wsInfo struct.
 *
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

void  *OpWs_WsCopy(CHAR *srcFile, void *data)
{
	OP_WS_INFO *old;
	OP_WS_INFO *new;

	if (data == NULL)
	{
		return((void *)NULL);
	}
	old = (OP_WS_INFO *)data;

	new = OpWs_WsNew();
	
	new->assocId = old->assocId;
	new->totalTargetRecords = old->totalTargetRecords;
	new->numWSRecords = old->numWSRecords;
	new->nextTargetRec = old->nextTargetRec;
	new->numRecReturned = old->numRecReturned;
	new->searchStatus = old->searchStatus;
	new->resultSetStatus = old->resultSetStatus;
	new->presentStatus = old->presentStatus;

	RescASN_CopyVS(IR_FILE, &new->resultSetName, 
	    &old->resultSetName);
	RescASN_CopyOI(IR_FILE, &new->nonSurrDiag.diagnosticSetId,
	    &old->nonSurrDiag.diagnosticSetId);
	RescASN_CopyVS(IR_FILE, &new->nonSurrDiag.addInfo,
	    &old->nonSurrDiag.addInfo);
	new->nonSurrDiag.condition = old->nonSurrDiag.condition;
	new->targetResultsDeleted = old->targetResultsDeleted;
	new->presenting = old->presenting;
	new->deleting = old->deleting;

	return(new);
}

/****************************************************************************
 * LOCAL FUNCTION:  OpWs_WsNew
 *
 * PURPOSE:
 *	Create and initialize a new op work space info structure.
 *
 * INPUT:
 *	none
 *
 * OUTPUT:
 *	OP_WS_INFO *
 *
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

static OP_WS_INFO	*OpWs_WsNew()
{
	OP_WS_INFO *wsInfo;

	wsInfo = (OP_WS_INFO *)RescIR_Zmalloc(IR_FILE,
	    sizeof(OP_WS_INFO));
	if (wsInfo == NULL)
	{
		return ((OP_WS_INFO *)NULL);
	}

	RESC_ASN_MALLOCED(wsInfo);

	/* Set new parameters */
	wsInfo->assocId = IR_UNKNOWN_ID_TYPE;
	wsInfo->totalTargetRecords = 0;
	wsInfo->numWSRecords = 0;
	wsInfo->nextTargetRec = 0;
	wsInfo->numRecReturned = 0;
	wsInfo->searchStatus = FALSE;
	wsInfo->resultSetStatus = IR_RESULT_SET_NO_STATUS;
	wsInfo->presentStatus = IR_PRSNT_NO_STATUS;
	wsInfo->targetResultsDeleted = FALSE;
	wsInfo->presenting = FALSE;
	wsInfo->deleting = FALSE;
	RESC_ASN_NO_MALLOC(&wsInfo->resultSetName);
	RESC_ASN_INIT_VS(&wsInfo->resultSetName);
	RESC_ASN_NO_MALLOC(&wsInfo->nonSurrDiag);
	RESC_ASN_NO_MALLOC(&wsInfo->nonSurrDiag.diagnosticSetId);
	RESC_ASN_INIT_OI(&wsInfo->nonSurrDiag.diagnosticSetId);
	RESC_ASN_NO_MALLOC(&wsInfo->nonSurrDiag.addInfo);
	RESC_ASN_INIT_VS(&wsInfo->nonSurrDiag.addInfo);
	RESC_ASN_NO_MALLOC(&wsInfo->records);
	RESC_LIST_INIT(&wsInfo->records);
	
	return(wsInfo);
}
