
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: RescIO.c
 *
 * DESCRIPTION:
 *	This unit contains the functions used retrieve records from a file 
 *
 * CONTENTS:
 *	RescIO_OpenFile
 *	RescIO_CloseFile
 *	RescIO_Seek
 *	RescIO_GetRecord
 *	RescIO_DeleteRecord
 *	RescIO_AddRecord
 *	RescIO_EOF
 *	RescIO_GetRecPtr
 *	RescIO_SetRecPtr
 *
 * NOTES:
 *
 * $Header$
 * $Log$
 *
/***************************************************************************/
#include <ir.h>
#include <resc.h>
#include <rescio.h>
#include <asn_tool.h>
#include <stdlib.h>

/****************************************************************************
 * FUNCTION:  RescIO_OpenFile
 *
 * PURPOSE:
 *	This function is used to open a file.
 *
 * INPUT:
 *	CHAR			pointer to calling function
 * 	VISIBLE_STRING		working path
 *	VISIBLE_STRING		fileName
 *	RESC_IO_FILE_STRUCT	pointer to file structure
 *
 * OUTPUT:
 *	IR_STATUS
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

IR_STATUS	RescIO_OpenFile(CHAR *srcFile, VISIBLE_STRING workPath,
		    VISIBLE_STRING fileName, RESC_IO_FILE_STRUCT *file)
{
	IR_STATUS	status;
	CHAR		fName[RESC_IO_MAX_FILE_NAME_SIZE+1];
	CHAR		pName[RESC_IO_MAX_FILE_NAME_SIZE+1];

	/* Concatenate path and file name */
	memcpy(fName, fileName.sPtr, fileName.len);
	fName[fileName.len] = EOS;
	memcpy(pName, workPath.sPtr, workPath.len);
	pName[workPath.len] = EOS;

	strcat(pName, fName);

	/* Create file structure */
	RESC_ASN_NO_MALLOC(file);

	/* Add filename to file structure */
	RESC_ASN_NO_MALLOC(&(file->name));
	RESC_ASN_INIT_VS(&(file->name));
	status = RescASN_AddVS(IR_FILE, &(file->name), pName, 
	    (fileName.len + workPath.len));
	if (status == IR_FAIL)
	{
		CS_LOG_PRINTS(IR_FILE, "Unable to create visible string\n");
		return (IR_FAIL);
	}

	/* Open file */
	file->fp = fopen(pName, "a+");
	if (file->fp == NULL)
	{
		CS_LOG_PRINTV(IR_FILE, "Unable to open file %s\n", pName);
		return (IR_FAIL);
	}
	file->eof = FALSE;

	return (IR_SUCCESS);
	
}

/****************************************************************************
 * FUNCTION:  RescIO_CloseFile
 *
 * PURPOSE:
 *	This function is used to close a file.
 *
 * INPUT:
 *	CHAR			pointer to calling function
 *	RESC_IO_FILE_STRUCT	pointer to file structure
 *
 * OUTPUT:
 *	IR_STATUS
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

IR_STATUS	RescIO_CloseFile(CHAR *srcFile, RESC_IO_FILE_STRUCT *file)
{
	SHORT		stat;
	CHAR		fName[RESC_IO_MAX_FILE_NAME_SIZE];

	/* Close file */
	stat = fclose(file->fp);
	if (stat != 0)
	{
		memcpy(fName, file->name.sPtr, file->name.len);
		fName[file->name.len] = EOS;
		CS_LOG_PRINTV(IR_FILE, "Unable to close file %s\n", fName);
		return (IR_FAIL);
	}

	/* Delete file structure */
	RescASN_DestroyVS(IR_FILE, (void *)&(file->name));

	return (IR_SUCCESS);
}
	
/****************************************************************************
 * FUNCTION:  RescIO_Seek
 *
 * PURPOSE:
 *	This function is used to seek to a key in a file.
 *
 * INPUT:
 *	CHAR			pointer to calling function
 *	RESC_IO_FILE_STRUCT	pointer to file structure
 *	RESC_IO_TAG_TYPE	type of record searching for
 *	VISIBLE_STRING		pointer to key
 *	fpos_t			pointer to record status
 *
 * OUTPUT:
 *	IR_STATUS
 *
 * NOTES:
 *	The pointer to the record status can be used for overwriting the
 *	status when deleting the record.
 *
 ***************************************************************************/

IR_STATUS	RescIO_Seek(CHAR *srcFile, RESC_IO_FILE_STRUCT *file, 
		    RESC_IO_TAG_TYPE tag, VISIBLE_STRING *key, 
		    fpos_t *statusPtr)
{
	SHORT		stat;
	BOOLEAN		found;
	CHAR		recTag[RESC_IO_TAG_LENGTH+1];
	CHAR		recStatus[RESC_IO_STATUS_LENGTH+1];
	CHAR		recKey[RESC_IO_KEY_LENGTH+1];
	CHAR		recLen[RESC_IO_LEN_LENGTH+1];
	SHORT		rLen;
	CHAR		*str;
	CHAR		record[RESC_IO_MAX_RECORD_SIZE];
	CHAR		fName[RESC_IO_MAX_FILE_NAME_SIZE+1];
	CHAR		buf[2];

	found = FALSE;
	/* While record key match not found */
	while (!found)
	{
		/* Get tag */
		str = fgets(recTag, RESC_IO_TAG_LENGTH+1, file->fp); 
		if (feof(file->fp) != 0)
		{
			file->eof = TRUE;
			return (IR_SUCCESS);
		}
		if (str == NULL)
		{
			memcpy(fName, file->name.sPtr, file->name.len);
			fName[file->name.len] = EOS;
			CS_LOG_PRINTV(IR_FILE, 
			    "Unable to read from file %s\n", fName);
			return (IR_FAIL);
		}

		if (statusPtr != NULL)
		{
			/* Save position of record status */
			stat = fgetpos(file->fp, statusPtr);
			if (stat != 0)
			{
				memcpy(fName, file->name.sPtr, file->name.len);
				fName[file->name.len] = EOS;
				CS_LOG_PRINTV(IR_FILE, 
				    "Unable to read from file %s\n", fName);
				return (IR_FAIL);
			}
		}

		/* Get record status */
		str = fgets(recStatus, RESC_IO_STATUS_LENGTH+1, file->fp); 
		if (str == NULL)
		{
			memcpy(fName, file->name.sPtr, file->name.len);
			fName[file->name.len] = EOS;
			CS_LOG_PRINTV(IR_FILE, 
			    "Unable to read from file %s\n", fName);
			return (IR_FAIL);
		}

		/* Get record key */
		str = fgets(recKey, RESC_IO_KEY_LENGTH+1, file->fp); 
		if (str == NULL)
		{
			memcpy(fName, file->name.sPtr, file->name.len);
			fName[file->name.len] = EOS;
			CS_LOG_PRINTV(IR_FILE, 
			    "Unable to read from file %s\n", fName);
			return (IR_FAIL);
		}

		/* If status is ok */
		if ((strncmp(recTag, tag, RESC_IO_TAG_LENGTH) == 0) && 
		    (strncmp(recStatus, RESC_IO_REC_OK, 
		    RESC_IO_STATUS_LENGTH) == 0))
		{
			/* compare keys */
			if (key == NULL)
			{
				found = TRUE;
			}
			else if (key->len > RESC_IO_KEY_LENGTH)
			{
				CS_LOG_PRINTV(IR_FILE, 
				    "Key too long %d\n", key->len);
				return (IR_FAIL);
			}
			else if (strncmp(key->sPtr, recKey, key->len) == 0)
			{
				found = TRUE;
			}
		}

		/* if record not found advance to next record */
		if (!found)
		{
			/* Get record length */
			str = fgets(recLen, RESC_IO_LEN_LENGTH+1, file->fp); 
			if (str == NULL)
			{
				memcpy(fName, file->name.sPtr, file->name.len);
				fName[file->name.len] = EOS;
				CS_LOG_PRINTV(IR_FILE, 
				    "Unable to read from file %s\n", fName);
				return (IR_FAIL);
			}

			rLen = atoi(recLen);
			/* Get record */
			str = fgets(record, rLen+1, file->fp); 
			if (str == NULL)
			{
				memcpy(fName, file->name.sPtr, file->name.len);
				fName[file->name.len] = EOS;
				CS_LOG_PRINTV(IR_FILE, 
				    "Unable to read from file %s\n", fName);
				return (IR_FAIL);
			}

			/* Get <CR> */
			str = fgets(buf, 2, file->fp); 
			if (str == NULL)
			{
				memcpy(fName, file->name.sPtr, file->name.len);
				fName[file->name.len] = EOS;
				CS_LOG_PRINTV(IR_FILE, 
				    "Unable to read from file %s\n", fName);
				return (IR_FAIL);
			}
		}
	}
	return (IR_SUCCESS);
}
	
/****************************************************************************
 * FUNCTION:  RescIO_GetRecord
 *
 * PURPOSE:
 *	This function is used to retrieve a record from a file.
 *
 * INPUT:
 *	CHAR			pointer to calling function
 *	RESC_IO_FILE_STRUCT	file structure
 *
 * OUTPUT:
 *	VISIBLE_STRING		pointer to pointer to record
 *	fpos_t			pointer to record status
 *	IR_STATUS
 *
 * NOTES:
 *	RescIO_Seek must be called prior to this function in order to set
 *	the file cursor.
 *
 ***************************************************************************/

IR_STATUS	RescIO_GetRecord(CHAR *srcFile, RESC_IO_FILE_STRUCT file,
				VISIBLE_STRING **record) 
{
	IR_STATUS	status;
	CHAR		recLen[RESC_IO_LEN_LENGTH+1];
	ULONG		rLen;
	CHAR		*str;
	CHAR		fName[RESC_IO_MAX_FILE_NAME_SIZE+1];
	CHAR		buf[RESC_IO_MAX_RECORD_SIZE+1];
	CHAR		*bufPtr;

	*record = NULL;

	/* Get record length */
	str = fgets(recLen, RESC_IO_LEN_LENGTH+1, file.fp); 
	if (str == NULL)
	{
		memcpy(fName, file.name.sPtr, file.name.len);
		fName[file.name.len] = EOS;
		CS_LOG_PRINTV(IR_FILE, 
		    "Unable to read from file %s\n", fName);
		return (IR_FAIL);
	}

	rLen = atol(recLen);
	if (rLen <= RESC_IO_MAX_RECORD_SIZE)
	{
		bufPtr = buf;
	}
	else
	{
		bufPtr = (CHAR *)RescIR_Malloc(IR_FILE, rLen+1);
		if (bufPtr == NULL)
		{
			CS_LOG_PRINTS(IR_FILE, "Memory allocation error\n");
			return (IR_FAIL);
		}
	}
	
	/* Get record */
	str = fgets(bufPtr, (int)rLen+1, file.fp); 
	if (str == NULL)
	{
		memcpy(fName, file.name.sPtr, file.name.len);
		fName[file.name.len] = EOS;
		CS_LOG_PRINTV(IR_FILE,
		    "Unable to read from file %s\n", fName);
		return (IR_FAIL);
	}

	*record = (VISIBLE_STRING *)RescASN_CreateVS(IR_FILE);

	/* Create record element */
	status = RescASN_AddVS(IR_FILE, *record, bufPtr, (SHORT)rLen+1);
	if (status == IR_FAIL)
	{
		CS_LOG_PRINTS(IR_FILE, 
		    "Unable to alloc memory for visible string\n");
		RescASN_DestroyVS(IR_FILE, (void *)record);
		record = NULL;	
		return (IR_FAIL);
	}

	/* Get <CR> */
	str = fgets(buf, 2, file.fp); 
	if (str == NULL)
	{
		memcpy(fName, file.name.sPtr, file.name.len);
		fName[file.name.len] = EOS;
		CS_LOG_PRINTV(IR_FILE, 
		    "Unable to read from file %s\n", fName);
		return (IR_FAIL);
	}

	return (IR_SUCCESS);
}
	
/****************************************************************************
 * FUNCTION:  RescIO_DeleteRecord
 *
 * PURPOSE:
 *	This function is used to delete a record from a file.
 *
 * INPUT:
 *	CHAR			pointer to calling function
 *	RESC_IO_FILE_STRUCT	file structure
 *	fpos_t			position of record status in file
 *
 * OUTPUT:
 *	IR_STATUS
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

IR_STATUS	RescIO_DeleteRecord(CHAR *srcFile, RESC_IO_FILE_STRUCT file,
				fpos_t recStatus) 
{
	SHORT		stat;
	CHAR		fName[RESC_IO_MAX_FILE_NAME_SIZE+1];

	
	/* Reset file cursor to record status */
	stat = fsetpos(file.fp, &recStatus);
	if (stat != 0)
	{
		memcpy(fName, file.name.sPtr, file.name.len);
		fName[file.name.len] = EOS;
		CS_LOG_PRINTV(IR_FILE,
		    "Unable to read from file %s\n", fName);
	}

	/* Change status to deleted */
	stat = fputs(RESC_IO_REC_DELETED, file.fp);
	if (stat < 0)
	{
		memcpy(fName, file.name.sPtr, file.name.len);
		fName[file.name.len] = EOS;
		CS_LOG_PRINTV(IR_FILE,
		    "Unable to write to file %s\n", fName);
	}

	return (IR_SUCCESS);
}
	
/****************************************************************************
 * FUNCTION:  RescIO_AddRecord
 *
 * PURPOSE:
 *	This function is used to append a record to a file.
 *
 * INPUT:
 *	CHAR			pointer to calling function
 *	RESC_IO_FILE_STRUCT	file structure
 *	VISIBLE_STRING		tag
 *	VISIBLE_STRING		key
 *	VISIBLE_STRING		record
 *
 * OUTPUT:
 *	IR_STATUS
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

IR_STATUS	RescIO_AddRecord(CHAR *srcFile, RESC_IO_FILE_STRUCT file,
				VISIBLE_STRING tag, VISIBLE_STRING key,
				VISIBLE_STRING record) 
{
	SHORT		stat;
	CHAR		fName[RESC_IO_MAX_FILE_NAME_SIZE];
	CHAR		buf[RESC_IO_MAX_RECORD_SIZE];
	SHORT		i;
	CHAR		*bufPtr;

	
	/* Go to end of file */
	stat = fseek(file.fp, 0, SEEK_END);
	if (stat != 0)
	{
		memcpy(fName, file.name.sPtr, file.name.len);
		fName[file.name.len] = EOS;
		CS_LOG_PRINTV(IR_FILE,
		    "Unable to read from file %s\n", fName);
	}
	
	/* Allocate space for temporary buffer */
	if (record.len <= RESC_IO_MAX_RECORD_SIZE)
	{
		bufPtr = buf;
	}
	else
	{
		bufPtr = (CHAR *)RescIR_Malloc(IR_FILE, record.len+1);
		if (bufPtr == NULL)
		{
			CS_LOG_PRINTS(IR_FILE, "Memory allocation error\n");
			return (IR_FAIL);
		}
	}
	
	/* Write tag to file */
	memcpy(bufPtr, tag.sPtr, tag.len);
	bufPtr[tag.len] = EOS;
	stat = fprintf(file.fp, bufPtr);
	if (stat == 0)
	{
		memcpy(fName, file.name.sPtr, file.name.len);
		fName[file.name.len] = EOS;
		CS_LOG_PRINTV(IR_FILE,
		    "Unable to write to file %s\n", fName);
	}

	/* Write status to file */
	memcpy(bufPtr, RESC_IO_REC_OK, RESC_IO_STATUS_LENGTH);
	bufPtr[RESC_IO_STATUS_LENGTH] = EOS;
	stat = fprintf(file.fp, bufPtr);
	if (stat == 0)
	{
		memcpy(fName, file.name.sPtr, file.name.len);
		fName[file.name.len] = EOS;
		CS_LOG_PRINTV(IR_FILE,
		    "Unable to write to file %s\n", fName);
	}

	/* Write key to file */
	memcpy(bufPtr, key.sPtr, key.len);
	bufPtr[key.len] = EOS;
	stat = fprintf(file.fp, bufPtr);
	if (stat == 0)
	{
		memcpy(fName, file.name.sPtr, file.name.len);
		fName[file.name.len] = EOS;
		CS_LOG_PRINTV(IR_FILE,
		    "Unable to write to file %s\n", fName);
	}

	/* Pad key record */
	for(i = key.len; i < RESC_IO_KEY_LENGTH; i++)
	{
		stat = fprintf(file.fp, " ");
		if (stat == 0)
		{
			memcpy(fName, file.name.sPtr, file.name.len);
			fName[file.name.len] = EOS;
			CS_LOG_PRINTV(IR_FILE,
			    "Unable to write to file %s\n", fName);
		}
	}

	/* Write record length to file */
	sprintf(bufPtr, "%d", record.len);
	stat = fprintf(file.fp, "%s", bufPtr);
	if (stat == 0)
	{
		memcpy(fName, file.name.sPtr, file.name.len);
		fName[file.name.len] = EOS;
		CS_LOG_PRINTV(IR_FILE,
		    "Unable to write to file %s\n", fName);
	}

	/* Pad record length record */
	for(i = strlen(bufPtr); i < RESC_IO_LEN_LENGTH; i++)
	{
		stat = fprintf(file.fp, " ");
		if (stat == 0)
		{
			memcpy(fName, file.name.sPtr, file.name.len);
			fName[file.name.len] = EOS;
			CS_LOG_PRINTV(IR_FILE,
			    "Unable to write to file %s\n", fName);
		}
	}

	/* Write record to file */
	memcpy(bufPtr, record.sPtr, record.len);
	bufPtr[record.len] = EOS;
	stat = fprintf(file.fp, "%s\n", bufPtr);
	if (stat == 0)
	{
		memcpy(fName, file.name.sPtr, file.name.len);
		fName[file.name.len] = EOS;
		CS_LOG_PRINTV(IR_FILE,
		    "Unable to write to file %s\n", fName);
	}

	return (IR_SUCCESS);
}
	
/****************************************************************************
 * FUNCTION:  RescIO_EOF
 *
 * PURPOSE:
 *	This function is used to determine if the end of file has been
 *	reached.
 *
 * INPUT:
 *	CHAR			pointer to calling function
 *	RESC_IO_FILE_STRUCT	pointer to file structure
 *
 * OUTPUT:
 *	BOOLEAN			eof indicator	
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

BOOLEAN		RescIO_EOF(RESC_IO_FILE_STRUCT file)
{
	return (file.eof);
}

/****************************************************************************
 * FUNCTION:  RescIO_ResetFile
 *
 * PURPOSE:
 *	This function is used to set the pointer in a file.
 *
 * INPUT:
 *	CHAR			pointer to calling function
 *	RESC_IO_FILE_STRUCT	file structure
 *	fpos_t			position of record in file
 *
 * OUTPUT:
 *	IR_STATUS
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

IR_STATUS	RescIO_SetRecPtr(CHAR *srcFile, RESC_IO_FILE_STRUCT file,
				fpos_t recPtr) 
{
	SHORT		stat;
	CHAR		fName[RESC_IO_MAX_FILE_NAME_SIZE+1];

	
	/* Reset file cursor to record status */
	stat = fsetpos(file.fp, &recPtr);
	if (stat != 0)
	{
		memcpy(fName, file.name.sPtr, file.name.len);
		fName[file.name.len] = EOS;
		CS_LOG_PRINTV(IR_FILE,
		    "Unable to read from file %s\n", fName);
	}
	
	return (IR_SUCCESS);
}

/****************************************************************************
 * FUNCTION:  RescIO_GetRecPtr
 *
 * PURPOSE:
 *	This function is used to get the pointer in a file.
 *
 * INPUT:
 *	CHAR			pointer to calling function
 *	RESC_IO_FILE_STRUCT	file structure
 *	fpos_t			position of record in file
 *
 * OUTPUT:
 *	IR_STATUS
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

IR_STATUS	RescIO_GetRecPtr(CHAR *srcFile, RESC_IO_FILE_STRUCT file,
				fpos_t *recPtr) 
{
	SHORT		stat;
	CHAR		fName[RESC_IO_MAX_FILE_NAME_SIZE+1];

	if (recPtr != NULL)
	{
		/* Save position of next record */
		stat = fgetpos(file.fp, recPtr);
		if (stat != 0)
		{
			memcpy(fName, file.name.sPtr, file.name.len);
			fName[file.name.len] = EOS;
			CS_LOG_PRINTV(IR_FILE, 
			    "Unable to read from file %s\n", fName);
			return (IR_FAIL);
		}
	}

	return (IR_SUCCESS);
}
