
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT:  Association Control Commnunication Event.
 *
 * DESCRIPTION:
 *	The Association Control Communication Event CSU reacts to 
 *	communication events concerning associations that have been 
 *	detected by a previous call to IRO_CommEvent.
 *
 * CONTENTS:
 *	UaAc_CommEvent
 *
 * NOTES:
 *	This CSU may only be invoked from with in the User Application
 * 	Layer of the IR Kernel-Origin
 *
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 *
/***************************************************************************/

#include <ir.h>
#include <ir_api.h>
#include <resc.h>
#include <resclist.h>
#include <ua.h>
#include <ua/uaepmsg.h>

/****************************************************************************
 * FUNCTION: UaAc_CommEvent
 *
 * PURPOSE:
 *	The User Application Association Control Communication Event
 *	function reacts to various sorts of communication events in
 *	order to inform the usrer and or update the menus.
 *
 * INPUT:
 * 	This function requires an UAAC_COMM_EVENT_DATA_TYPE, namely one
 *	of the following:
 *
 * 	    UAAC_ASSOC_CONNECT	- An Association has completed the
 *				  connection sequence.
 *	    UAAC_INIT_COMPLETE	- An Association has completed the
 *				  initialization sequence.
 *	    UAAC_ASSOC_ABORT	- An Association has been aborted.
 *	    UAAC_ASSOC_RELEASE  - An Association has been closed.
 *
 *	An IRO_COMM_EVENT_REPLY is also required and contains the
 *	information returned from a call to IRO_CommEvent.
 *
 * OUTPUT:
 *	This function returns an IR_STATUS with returns a non-zero
 *	value on error. A call to UaHec_ErrorControl is required
 *	to inform the user of the problem.
 *
 * NOTES:
 *	Nothing to note as of yet.
 ***************************************************************************/

IR_STATUS UaAc_CommEvent(
    UAAC_COMM_EVENT_DATA_TYPE dataType,   /* Type of communiction event. */
    IRO_COMM_EVENT_REPLY      *replyData) /* Related comm data. */
{
	IR_STATUS	returnCode;	/* Return value for function.  */
	BOOLEAN		cleanup;
   	IR_LIST		assocList;	/* Ouput list */
	IRO_ASSOC_STRUCT    *irStruct;

	returnCode = IR_SUCCESS;
	cleanup = FALSE;
	

	/*
	 * Determine the type of Communication event.
	 */
	switch (dataType)
	{
	case IR_COMM_DATA_CONNECT:
		RESC_ASN_NO_MALLOC(&assocList);
		RESC_LIST_INIT(&assocList);
		if ((UaAc_GetAssociationInfo(UAAC_GET_ONE_ASSOC,
			replyData->assocId, &assocList) == IR_SUCCESS) 
			&& (RESC_LIST_SIZE(&assocList) == 1))
		{
			RESC_LIST_SET_FIRST(&assocList);
			irStruct = RESC_LIST_GET(&assocList);
			UaHec_UpdateAssocName(replyData->assocId,
			    &irStruct->name);
			RescList_Destroy(IR_FILE, &assocList);
		}
		UaHec_UserMsg(IR_FILE, UAEP_MSG_TYPE_AC, MSG_ID_AC_CONNECTED,
		    (CHAR *)NO_STRING, (USHORT)replyData->assocId);
		break;
	case IR_COMM_DATA_INIT_SRVC:
		UaHec_UserMsg(IR_FILE, UAEP_MSG_TYPE_AC, MSG_ID_AC_INIT_COMPLETE,
		    (CHAR *)NO_STRING, (USHORT)replyData->assocId);
		break;
	
	case IR_COMM_DATA_INIT_FAIL:
		UaHec_UserMsg(IR_FILE, UAEP_MSG_TYPE_AC, MSG_ID_AC_INIT_FAIL,
		    (CHAR *)NO_STRING, (USHORT)replyData->assocId);
		break;
	case IR_COMM_DATA_REL_FAIL:
		UaHec_UserMsg(IR_FILE, UAEP_MSG_TYPE_AC, MSG_ID_AC_CONN_REL_FAIL,
		    (CHAR *)NO_STRING, (USHORT)replyData->assocId);
		cleanup = TRUE;
		break;
	case IR_COMM_DATA_ABORT_FAIL:
		UaHec_UserMsg(IR_FILE, UAEP_MSG_TYPE_AC, MSG_ID_AC_ABORT_FAIL,
		    (CHAR *)NO_STRING, (USHORT)replyData->assocId);
		cleanup = TRUE;
		break;
	case IR_COMM_DATA_PROTOCOL_ERR:
		UaHec_UserMsg(IR_FILE, UAEP_MSG_TYPE_AC, MSG_ID_AC_PROTOCOL_ERROR,
		    (CHAR *)NO_STRING, (USHORT)replyData->assocId);
		cleanup = TRUE;
		break;
	case IR_COMM_DATA_ABORT:
		UaHec_UserMsg(IR_FILE, UAEP_MSG_TYPE_AC, MSG_ID_AC_ABORT,
		    (CHAR *)NO_STRING, (USHORT)replyData->assocId);
		cleanup = TRUE;
		break;
	case IR_COMM_DATA_CONN_REFUSED:
		UaHec_UserMsg(IR_FILE, UAEP_MSG_TYPE_AC, MSG_ID_AC_CONN_REFUSED,
		    (CHAR *)NO_STRING, (USHORT)replyData->assocId);
		cleanup = TRUE;
		break;
	case IR_COMM_DATA_UNKNOWN_HOST:
	case IR_COMM_DATA_HOST_NOT_FOUND:
		UaHec_UserMsg(IR_FILE, UAEP_MSG_TYPE_AC, MSG_ID_AC_UNKNOWN_HOST,
		    (CHAR *)NO_STRING, (USHORT)replyData->assocId);
		cleanup = TRUE;
		break;
	case IR_COMM_DATA_RELEASE:
		UaHec_UserMsg(IR_FILE, UAEP_MSG_TYPE_AC, MSG_ID_AC_CONN_REL,
		    (CHAR *)NO_STRING, (USHORT)replyData->assocId);
		cleanup = TRUE;
		break;
	default:
		xvt_note("uaeappl.c Connection refused by target");

		break;
	}
	if (cleanup)
	{
		UaCs_DeleteAssocId(replyData->assocId);
		UaHec_DeleteAssocName(replyData->assocId);
		UaAc_RemoveTimedItem(replyData->assocId);
	}

	return (returnCode);
}
