
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: Association Information
 *
 * DESCRIPTION:
 *
 *	The Association Information CSU will get the association requested
 *	association information from the Origin Process.
 *
 *	The Update Status function will update the menu item based
 *	on the associations currently available.
 *
 * CONTENTS:
 *	UaAc_AssociationInfo
 *	UaAc_AssociationInfoFull
 *	UaAc_UpdateStatus
 *
 * NOTES:
 *	These functions should only be called from within the User
 *	application layer of the IR Kernel-Origin.
 *
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 *
/***************************************************************************/

#include <ir.h>
#include <ua.h>
#include <resc.h>
#include <resclist.h>
#include <ua/dialog.h>
#include <uaep.h>


/****************************************************************************
 * FUNCTION: UaAc_AssociationInfo
 *
 * PURPOSE:
 *
 *	This function is used to obtain a list of association information
 *	for one or more associations. The information returned is supplied
 *	in a brief format.
 *	
 * INPUT:
 *	This function Requires an UAAC_ASSOC_INFO_TYPE to determine
 *	the kinds of associations the caller wants information on.
 *	The following list describes the different requests and
 *	any additional iformation required.
 *
 *	Type			Description
 *	------------------    	--------------------------------------
 *	UAAC_GET_ALL_ASSOCS	- Obtain information for all existing
 *				  associations. No extra input required.
 *	UAAC_GET_ONE_ASSOC 	- Obtain information for a specific
 *				  association. An IR_ASSOC_ID for the 
 *				  association requested is required.
 *	UAAC_GET_IDLE_ASSOC	- Obtain information on all idle
 *				  associations. No extra input required.
 *	UAAC_GET_INIT_ASSOC 	- Obtain information on all assciations
 *				  that are in "INIT" state. No extra
 *				  input is required.
 *	UAAC_GET_SEARCH_ASSOC 	- Obtain information on all assciations
 *				  that are in "SEARCH" state. No extra
 *				  input is required.
 *	UAAC_GET_PRESENT_ASSOC 	- Obtain information on all assciations
 *				  that are in "PRESENT" state. No extra
 *				  input is required.
 *	UAAC_GET_DELETE_ASSOC 	- Obtain information on all assciations
 *				  that are in "DELETE" state. No extra
 *				  input is required.
 *	UAAC_GET_RSRC_RPT_ASSOC - Obtain information on all assciations
 *				  that are in "RESCOURCE REPORT" state. 
 *				  No extra input is required.
 *
 *	A pointer to a pointer of UAAC_ACCOC_LIST information is
 *	required  for output.
 *
 * OUTPUT:
 *	This function returns an IR_STATUS with returns a non-zero
 *	value on error. A call to UaHec_ErrorControl is required
 *	to inform the user of the problem.
 *
 *	On success it will return an IR_LIST of UAAC_ASSOC_INFO_TYPE 
 *	information. The array is terminated
 *	with an invalid element. There can be 0 or more elements in the
 *	array. It is the responsiblity of the calling function to
 *	free the list when it is nolonger required.
 *
 * NOTES:
 *	The data supplied in the assocList is a copy of the data provided
 *	from the IRO interface and is the responsibility of the calling
 *	function to destroy the list when it is done with it.
 *
 ***************************************************************************/

IR_STATUS UaAc_GetAssociationInfo(
    UAAC_ASSOC_INFO_TYPE 	type,		/* Type of associations */
    IR_ASSOC_ID			assocId,	/* Assoc Id optional */
    IR_LIST			*assocList)	/* Ouput list */
{

	IRO_ASSOC_REQ 			assocReq;	/* IRO Req. Info */
	IRO_ASSOC_REPLY			assocReply;	/* IRO Reply not used */
	IR_STATUS			returnCode;	/* Function Return */
	IR_ASSOC_ID			getAssoc;	/* Assoc to test for */
	BOOLEAN				getAll;		/* Get all elements. */
	IRO_ASSOC_STATE			getState;	/* State to get.  */
	SHORT				count;		/* loop counter */
	IRO_ASSOC_STRUCT		*assocStruct;	/* assoc list element */
	IRO_ASSOC_STRUCT		*tmpElem;	/* assoc list element */

	
	RESC_ASN_NO_MALLOC(&assocReq);
	RESC_ASN_NO_MALLOC(&assocReply);

	returnCode = IRO_Association(UaEp_InfoUserId(), 
	    IRO_ASSOC_GET_ASSOC_INFO,
	    &assocReq, 
	    &assocReply);
	if (returnCode != IR_SUCCESS)
	{
		UaHec_ReportIROError(IR_FILE,
		    IRO_ASSOCIATION_FUNC, IRO_ASSOC_GET_ASSOC_INFO);
		return(returnCode);
	}


	/*
 	 * Set up variables to uniquely identify the records required.
	 */
	switch (type)
	{
	case UAAC_GET_ALL_ASSOCS:
		/* Get all records */
		getAssoc = IR_UNKNOWN_ID_TYPE; 	/* not required. */
		getAll = TRUE;			/* We want every record */
		getState = IR_UNKNOWN_ID_TYPE; 	/* State not important. */
		break;

	case UAAC_GET_ONE_ASSOC:
		/* Get a specified record. */
		getAssoc = assocId;		/* Record to get. */
		getAll = FALSE;			/* Don't get all */
		getState = IR_UNKNOWN_ID_TYPE;  /* State not important */
		break;

	case UAAC_GET_IDLE_ASSOC:
		getAssoc = IR_UNKNOWN_ID_TYPE;	/* Not required */
		getAll = FALSE;			/* Don't get all */
		/* Get Idle state associations only. */
		getState = IRO_ASSOC_STATE_IDLE;
		break;
	

	case UAAC_GET_INIT_ASSOC:
		getAssoc = IR_UNKNOWN_ID_TYPE;	/* Not required */
		getAll = FALSE;			/* Don't get all */
		/* Get Initiation state associations only. */
		getState = IRO_ASSOC_STATE_INIT;
		break;

	case UAAC_GET_SEARCH_ASSOC:
		getAssoc = IR_UNKNOWN_ID_TYPE;	/* Not required */
		getAll = FALSE;			/* Don't get all */
		/* Get Search state associations only. */
		getState = IRO_ASSOC_STATE_SEARCH;
		break;

	case UAAC_GET_PRESENT_ASSOC:
		getAssoc = IR_UNKNOWN_ID_TYPE;	/* Not required */
		getAll = FALSE;			/* Don't get all */
		/* Get Present state associations only. */
		getState = IRO_ASSOC_STATE_PRESENT;
		break;

	case UAAC_GET_DELETE_ASSOC:
		getAssoc = IR_UNKNOWN_ID_TYPE;	/* Not required */
		getAll = FALSE;			/* Don't get all */
		/* Get Delete state associations only. */
		getState = IRO_ASSOC_STATE_DELETE;
		break;

	case UAAC_GET_RSRC_RPT_ASSOC:
		getAssoc = IR_UNKNOWN_ID_TYPE;	/* Not required */
		getAll = FALSE;			/* Don't get all */
		/* Get Resource Report state associations only. */
		getState = IRO_ASSOC_STATE_RSRC_RPRT;
		break;
	default:
		return(returnCode);
		break;
	}

	/*
	 * Place only the records requested into the reply data.
	 */
	RESC_LIST_SET_FIRST(assocReply.activeAssociations);
	for (count = 0; 
	    count < RESC_LIST_SIZE(assocReply.activeAssociations);
	    count++)
	{
		assocStruct = RESC_LIST_GET(assocReply.activeAssociations);
		if ((getAll) ||
		    (assocStruct->assocId == getAssoc) ||
		    (assocStruct->state == getState))
		{
			tmpElem = (IRO_ASSOC_STRUCT *)RescIR_Zmalloc(IR_FILE,
			    sizeof(IRO_ASSOC_STRUCT));
			RESC_ASN_MALLOCED(tmpElem);
			RESC_ASN_NO_MALLOC(&tmpElem->name);
			RESC_ASN_INIT_VS(&tmpElem->name);
			RESC_ASN_NO_MALLOC(&tmpElem->address);
			RESC_ASN_INIT_VS(&tmpElem->address);
			RESC_ASN_NO_MALLOC(&tmpElem->hostName);
			RESC_ASN_INIT_VS(&tmpElem->hostName);

			tmpElem->assocId = assocStruct->assocId;

			tmpElem->targetId = assocStruct->targetId;

			RescASN_CopyVS(IR_FILE, 
			    &tmpElem->name,
			    &assocStruct->name);

			RescASN_CopyVS(IR_FILE, 
			    &tmpElem->address,
			    &assocStruct->address);

			RescASN_CopyVS(IR_FILE, 
			    &tmpElem->hostName,
			    &assocStruct->hostName);

			tmpElem->state = assocStruct->state;
		
			/* 
			 * Use the same destroy and dup functions that
			 * were used by the IRO list element.
			 */
			RescList_Append(IR_FILE, assocList,
			    tmpElem, 
			    RESC_LIST_GET_DESTFN(assocReply.activeAssociations),
			    RESC_LIST_GET_DUPFN(assocReply.activeAssociations));

		}
		RESC_LIST_SET_NEXT(assocReply.activeAssociations);
	}

	return(returnCode);
}


/****************************************************************************
 * FUNCTION: UaAc_AssociationInfoFull
 *
 * PURPOSE:
 *
 *	This function is used to obtain a full account of association 
 *	information for one association. 
 *	
 * INPUT:
 *	This function requires a association identifier as input.
 *
 * OUTPUT:
 *
 * NOTES:
 *	The data supplied in the assocList is a copy of the data provided
 *	from the IRO interface and is the responsibility of the calling
 *	function to destroy the list when it is done with it.
 *
 ***************************************************************************/

IR_STATUS UaAc_GetAssociationInfoFull(
    IR_ASSOC_ID			assocId,	/* Assoc Id optional */
    IRO_ASSOC_FULL_INFO_STRUCT	*assocInfo)	/* Ouput structure */
{

	IRO_ASSOC_REQ 			assocReq;	/* IRO Req. Info */
	IRO_ASSOC_REPLY			assocReply;	/* IRO Reply not used */
	IR_STATUS			returnCode;	/* Function Return */
	IRO_ASSOC_STRUCT		*as;		/* assoc list element */
	IRO_COMM_INIT_RESP		*ir;		/* initialization info */
	
	RESC_ASN_NO_MALLOC(&assocReq);
	RESC_ASN_NO_MALLOC(&assocReply);

	assocReq.assocId = assocId;

	returnCode = IRO_Association(UaEp_InfoUserId(),
	    IRO_ASSOC_FULL_INFO,
	    &assocReq, 
	    &assocReply);
	if (returnCode != IR_SUCCESS)
	{
		UaHec_ReportIROError(IR_FILE,
		    IRO_ASSOCIATION_FUNC, IRO_ASSOC_FULL_INFO);
		return(returnCode);
	}

	/* Copy the IRO brief information to the return structure */
	as = &assocInfo->assocStruct;
	RESC_ASN_NO_MALLOC(as);
	RESC_ASN_NO_MALLOC(&as->name);
	RESC_ASN_INIT_VS(&as->name);
	RESC_ASN_NO_MALLOC(&as->address);
	RESC_ASN_INIT_VS(&as->address);
	RESC_ASN_NO_MALLOC(&as->hostName);
	RESC_ASN_INIT_VS(&as->hostName);

	as->assocId = assocReply.fullInfo->assocStruct.assocId;
	as->targetId = assocReply.fullInfo->assocStruct.targetId;
	as->state = assocReply.fullInfo->assocStruct.state;

	RescASN_CopyVS(IR_FILE, &as->name, 
	    &assocReply.fullInfo->assocStruct.name);
	RescASN_CopyVS(IR_FILE, &as->address, 
	    &assocReply.fullInfo->assocStruct.address);
	RescASN_CopyVS(IR_FILE, &as->hostName, 
	    &assocReply.fullInfo->assocStruct.hostName);

	/* 
	 * Copy the IRO Initialize response information 
	 * to the return strucuter.
 	 * The user information field is ignored.
	 */
	ir = &assocInfo->initResp;

	RESC_ASN_NO_MALLOC(ir);
	RESC_ASN_NO_MALLOC(&ir->protocolVersion);
	RESC_ASN_INIT_VS(&ir->protocolVersion);
	RESC_ASN_NO_MALLOC(&ir->implementationId);
	RESC_ASN_INIT_VS(&ir->implementationId);
	RESC_ASN_NO_MALLOC(&ir->implementationName);
	RESC_ASN_INIT_VS(&ir->implementationName);
	RESC_ASN_NO_MALLOC(&ir->implementationVersion);
	RESC_ASN_INIT_VS(&ir->implementationVersion);

	/* Initialize user info but do not use it */
	RESC_ASN_NO_MALLOC(&ir->userInfo);
	RESC_ASN_INIT_OS(&ir->userInfo);

	ir->search = assocReply.fullInfo->initResp.search;
	ir->present = assocReply.fullInfo->initResp.present;
	ir->delSet = assocReply.fullInfo->initResp.delSet;
	ir->resourceReport = assocReply.fullInfo->initResp.resourceReport;
	ir->triggerResourceCtrl = 
	    assocReply.fullInfo->initResp.triggerResourceCtrl;
	ir->resourceCtrl = assocReply.fullInfo->initResp.resourceCtrl;
	ir->accessCtrl = assocReply.fullInfo->initResp.accessCtrl;
	ir->preferredMessageSize = 
	    assocReply.fullInfo->initResp.preferredMessageSize;
	ir->maximumRecordSize = 
	    assocReply.fullInfo->initResp.maximumRecordSize;

	RescASN_CopyVS(IR_FILE, &ir->protocolVersion,
	    &assocReply.fullInfo->initResp.protocolVersion);
	RescASN_CopyVS(IR_FILE, &ir->implementationId,
	    &assocReply.fullInfo->initResp.implementationId);
	RescASN_CopyVS(IR_FILE, &ir->implementationName,
	    &assocReply.fullInfo->initResp.implementationName);
	RescASN_CopyVS(IR_FILE, &ir->implementationVersion,
	    &assocReply.fullInfo->initResp.implementationVersion);
	RescASN_CopyVS(IR_FILE, &ir->userInfo,
	    &assocReply.fullInfo->initResp.userInfo);

	return(IR_SUCCESS);
}
/****************************************************************************
 * FUNCTION: UaAc_UpdateStatus
 *
 * PURPOSE:
 *	This function is invoked to update the status and break menu item
 *	in all menus on the main window.
 *
 * INPUT:
 *	none.
 *
 * OUTPUT:
 *	none.
 *
 * NOTES:
 *	This function uses the UaAc_GetAssociationInfo function to obtain
 *	all state information for all associations.
 *
 ***************************************************************************/

void UaAc_UpdateStatus(void)
{
	IR_LIST			assocList;	/* IRO_ASSOC_STRUCTs */
	SHORT			i;

	RESC_ASN_NO_MALLOC(&assocList);
	RESC_LIST_INIT(&assocList);

	if (UaAc_GetAssociationInfo(UAAC_GET_ALL_ASSOCS,
	    IR_UNKNOWN_ID_TYPE,
	    &assocList) == IR_FAIL)
	{
		CS_LOG_PRINTS(IR_FILE, "*** ERROR *** Couldn't update assocation menu status items\n");
		return;
	}

	/* Check for any associations. */
	i = RESC_LIST_SIZE(&assocList);
	if (i > 0)
	{
		/* 
		 * There is at least one association. Set the status 
		 * and break menu items to on.
		 */
		UaEp_MenuSetStatus(M_CONNECT_BREAK, TRUE);
	}
	else
	{
		UaEp_MenuSetStatus(M_CONNECT_BREAK, FALSE);
	}

	RescList_Destroy(IR_FILE, &assocList);

}

/****************************************************************************
 * FUNCTION: UaAc_DestroyFullInfoStruct
 *
 * PURPOSE:
 *	This utility function is used to destroy all data malloced for
 *	the UaAc_GetAssociationFullInfo function to obtain information
 *	for a single association. 
 *
 * INPUT:
 *	IRO_ASSOC_FULL_INFO_STRUCT pointer
 *
 * OUTPUT:
 *	none.
 *
 * NOTES:
 *
 ***************************************************************************/

void UaAc_DestroyFullInfoStruct(IRO_ASSOC_FULL_INFO_STRUCT *infoStruct)
{
	if (infoStruct)
	{
		RescASN_DestroyVS(IR_FILE, &infoStruct->assocStruct.name);
		RescASN_DestroyVS(IR_FILE, &infoStruct->assocStruct.address);
		RescASN_DestroyVS(IR_FILE, &infoStruct->assocStruct.hostName);
		RescASN_DestroyVS(IR_FILE, 
		    &infoStruct->initResp.protocolVersion);
		RescASN_DestroyVS(IR_FILE, 
		    &infoStruct->initResp.implementationId);
		RescASN_DestroyVS(IR_FILE, 
		    &infoStruct->initResp.implementationName);
		RescASN_DestroyVS(IR_FILE, 
		    &infoStruct->initResp.implementationVersion);
		RescASN_DestroyOS(IR_FILE, &infoStruct->initResp.userInfo);
	}
	if (RESC_ASN_IS_MALLOCED(infoStruct))
	{
		RescIR_Free(IR_FILE, (CHAR *)infoStruct);
	}
}
