/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT:  Access and Resource Control Commnunication Event.
 *
 * DESCRIPTION:
 *	The Access and Resource Control Communication Event CSU reacts to 
 *	communication events concerning access statifaction requirments
 *	and resource control display information detected by a previous 
 *	call to IRO_CommEvent.
 *
 * CONTENTS:
 *	UaArc_CommEvent
 *
 * NOTES:
 *	This CSU may only be invoked from with in the User Application
 * 	Layer of the IR Kernel-Origin
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 *
/***************************************************************************/

#include <ir.h>
#include <uaep.h>
#include <ua.h>
#include <resc.h>
#include <string.h>
#include <debug.h>
#include <ir_api.h>
#include <resclist.h>

static void convertRsrcRprtToStr(IRO_COMM_RSRC_REPORT *, CHAR *, USHORT);

/****************************************************************************
 * FUNCTION: UaArc_CommEvent
 *
 * PURPOSE:
 *	The User Application Access and Resource Control Communication Event
 *	function reacts to various sorts of communication events in
 *	order to inform the usrer and or update the menus.
 *
 * INPUT:
 * 	This function requires an UAARC_COMM_EVENT_DATA_TYPE, namely one
 *	of the following:
 *
 *	    UAARC_ACCESS_CNTRL	   - An access control has been requested.
 *	    UAARC_RESOURCE_CNTRL   - Resource control info has arrived.
 *	    UAARC_RESOURCE_REPORT  - Resource report info has arrived.
 *
 *	An IRO_COMM_EVENT_REPLY is also required and contains the
 *	information returned from a call to IRO_CommEvent.
 *
 * OUTPUT:
 *	This function returns an IR_STATUS with returns a non-zero
 *	value on error. A call to UaHec_ErrorControl is required
 *	to inform the user of the problem.
 *
 * NOTES:
 *	Nothing to note as of yet.
 ***************************************************************************/

IR_STATUS UaArc_CommEvent(
    UAARC_COMM_EVENT_DATA_TYPE dataType,  /* Type of communiction event. */
    IRO_COMM_EVENT_REPLY *replyData)	 /* Related comm data. */
{
	IR_STATUS	returnCode;	/* Return value for function.  */
	UAEP_DIALOG	info;
	IR_ASSOC_ID	assocId;
	IRO_ACCESS_CNTRL_REQ accCtlReq;
	IRO_COMM_ACCS_CNTL_REQ *accsReq;
	IRO_COMM_RSRC_REPORT *rsrcRprt;
	IRO_COMM_RSRC_CNTL_REQ *rsrcCtlReq;
	IRO_RSRC_CNTRL_REQ	rsrcCtlResp;
	UAEP_SYSTEM_INFO_STRUCT *taskWin; /* Session information */
	CHAR			*noteString;
	CHAR			*s;
	VISIBLE_STRING		*targetName;
	CHAR			targetNameStr[IR_LEN_100];

	returnCode = IR_SUCCESS;
	assocId = replyData->assocId;

	/* Malloc the message display string */
	noteString = (CHAR *)RescIR_Malloc(IR_FILE, IR_LEN_1000);

	targetName = UaHec_GetAssocName(assocId);
	if (targetName)
	{
		VSToStrNullTerm(targetNameStr, targetName, IR_LEN_100);
	}
	else
	{
		sprintf(targetNameStr, "Unknown target for association %d\n",
		    assocId);
	}
	/*
	 * Determine the type of Communication event.
	 */
	switch (dataType)
	{
	case UAARC_ACCESS_CNTRL:
		/*
		 * Target is requesting information from the user.
		 * Prepare the dialog structure.
		 */
		memset(&info, (CHAR)NULL, sizeof(UAEP_DIALOG));
		accsReq = &replyData->commData.accessCntlReq;

		VSToStrNullTerm(info.data.rsrcCtl.targetData,
		    &accsReq->accessControlReq, EP_DLG_EDIT_SZ);
		info.data.rsrcCtl.assocId = assocId;

		CS_LOG_PRINTV(IR_FILE, 
		    "Processing access control request: %s\n",
		    info.data.rsrcCtl.targetData);

		create_res_dialog(WD_MODAL, DLG_ACCESS_CONTROL, 
		    EM_ALL, dlg_114_eh, (long)&info);

		CS_LOG_PRINTS(IR_FILE, 
		    "Executed the access control dialog. \n");
		/*
		 * This dialog can't be canncelled so don't
		 * bother checking
		 */
		RESC_ASN_NO_MALLOC(&accCtlReq);
		RESC_ASN_NO_MALLOC(&accCtlReq.accessResponse);
		RESC_ASN_INIT_VS(&accCtlReq.accessResponse);

		accCtlReq.assocId = assocId;
		
		RescASN_AddVS(IR_FILE, &accCtlReq.accessResponse,
		    info.data.rsrcCtl.userReply, 
		    strlen(info.data.rsrcCtl.userReply));

		taskWin = (UAEP_SYSTEM_INFO_STRUCT *)get_app_data(TASK_WIN);
		returnCode = IRO_AccessControl(taskWin->userId, &accCtlReq);

		RescASN_DestroyVS(IR_FILE, &accCtlReq.accessResponse);
		break;

 	case UAARC_RESOURCE_CNTRL:
		rsrcCtlReq = &replyData->commData.resourceCntlReq;

		strcpy(noteString, "Received resource control from target ");
		strcat(noteString, targetNameStr);
		if (rsrcCtlReq->responseRequired == TRUE)
		{
			strcat(noteString, "Sending resource control response indicating that the result set is wanted.");
			RESC_ASN_NO_MALLOC(&rsrcCtlResp);
			rsrcCtlResp.assocId = assocId;
			if (rsrcCtlReq->partialResultsAvail !=
			    IR_RESULT_SET_NO_STATUS)
			{
				rsrcCtlResp.resultSetWanted = TRUE;
				rsrcCtlResp.wantedFlagValid = TRUE;
			}
			rsrcCtlResp.continueFlag = TRUE;

			taskWin = (UAEP_SYSTEM_INFO_STRUCT *)
			    get_app_data(TASK_WIN);
			IRO_ResourceControl(taskWin->userId, &rsrcCtlResp);
		}

		if (rsrcCtlReq->resourceReport)
		{
			s = &noteString[strlen(noteString)];
			convertRsrcRprtToStr(rsrcCtlReq->resourceReport,
			    s, IR_LEN_500);
		}

		UaHec_Message("Resource Control Request", noteString, FALSE);

		break;
	case UAARC_RESOURCE_REPORT:
		strcpy(noteString, "Received Resource Report from target ");
		strcat(noteString, targetNameStr);
		rsrcRprt = &replyData->commData.resourceReport;
		s = &noteString[strlen(noteString)];
		convertRsrcRprtToStr(rsrcRprt, s, IR_LEN_500);
		UaHec_Message("Resource Report Response", noteString, FALSE);
		
		break;
	default:
		break;
	}

	RescIR_Free(IR_FILE, (CHAR *)noteString);
	return (returnCode);
}

static void convertRsrcRprtToStr(IRO_COMM_RSRC_REPORT *rsrcRprt, 
	    CHAR *str, USHORT len)
{
	CHAR 	tmpBuf[IR_LEN_100];
	SHORT	i;
	IRO_COMM_RSRC_RPRT_EST	*est;
	CHAR	*s;
	

	i = 0;
	s = str;
	*s = EOS;
	RESC_LIST_SET_FIRST(&rsrcRprt->estimates);
	while (i < RESC_LIST_SIZE(&rsrcRprt->estimates))
	{
		est = RESC_LIST_GET(&rsrcRprt->estimates);
		if (est->knownCurrency == TRUE)
		{
			sprintf(tmpBuf, 
			    "Estimate Type = %d, Estimate Value = %ld, CurrencyCode = %d \n",
			    est->estimateType, est->value, est->currency);
		}
		else
		{
			sprintf(tmpBuf, 
			    "Estimate Type = %d, Estimate Value = %ld \n",
			    est->estimateType, est->value);
		}
		strcat(s, tmpBuf);
		RESC_LIST_SET_NEXT(&rsrcRprt->estimates);
	}

	VSToStrNullTerm(tmpBuf, &rsrcRprt->message, IR_LEN_100);
	strcpy(s, tmpBuf);
}
