/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT:  Help and Error Control Z39.50 Diagnostic codes and status codes.
 *
 * DESCRIPTION:
 *	This unit contains the tables and functions required to support
 *	a textual description of Z39.50 diagnostic and status codes.
 *
 * CONTENTS:
 *	UaHec_Bib1DiagTbl	- Bib1 Diagnostic Set
 *	UaHec_Z3950ResultSetStat - Search/Present Result Set Status
 *	UaHec_Z3950PresentStat  - Present Status
 *	UaHec_Z3950DeletStat	- Delete Status
 *	UaHec_Z3950PartialResults - Resource Control Request 
 *	UaHec_Z3950RsrcReportStat - Resouce Report Status.
 *	UaHec_Z3950ObjectIds	- Z3950 Object Identifiers
 *
 *	UaHec_GetDiagMsg	- Get diagnostic message.
 *
 * NOTES:
 *	This CSU may only be invoked from with in the User Application
 * 	Layer of the IR Kernel-Origin
 *
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 *
/***************************************************************************/

#include <ir.h>
#include <ir_api.h>
#include <ua.h>
#include <ua/uaepmsg.h>
#include <resc.h>
#include <ir_oid.h>

typedef struct
{
	UAHEC_MSG_ID	msgId;		/* unique identifier */
	BOOLEAN		option;		/* option control */
	CHAR		*string;	/* Displayable string */
} MSG_TABLE;

static MSG_TABLE	UaHec_Bib1DiagTbl[]=
{
	{
		BIB1_DIAG_PSE, TRUE,
		"Permanent system error"
	},
	{
		BIB1_DIAG_TSE, TRUE,
		"Temporary system error"
	},
	{
		BIB1_DIAG_US, TRUE,
		"Unsupported search"
	},
	{
		BIB1_DIAG_TOESW, TRUE,
		"Terms only exclusion (stop) words"
	},
	{
		BIB1_DIAG_TMAW, TRUE,
		"Too many argument words"
	},
	{
		BIB1_DIAG_TMBO, TRUE,
		"Too many boolean operators"
	},
	{
		BIB1_DIAG_TMTW, TRUE,
		"Too many truncated words"
	},
	{
		BIB1_DIAG_TMIS, TRUE,
		"Too many incomplete subfields"
	},
	{
		BIB1_DIAG_TWTS, TRUE,
		"Truncated words too short"
	},
	{
		BIB1_DIAG_IFFRNST, TRUE,
		"Invalid format for record number(search term)"
	},
	{
		BIB1_DIAG_TMCISS, TRUE,
		"Too many characters in search statement"
	},
	{
		BIB1_DIAG_TMRR, TRUE,
		"Too many records retrieved"
	},
	{
		BIB1_DIAG_PROOR, TRUE,
		"Present request out-of-range"
	},
	{
		BIB1_DIAG_SEIPR, TRUE,
		"System error in presenting records"
	},
	{
		BIB1_DIAG_RNATBSI, TRUE,
		"Record not authorized to be sent intersystem"
	},
	{
		BIB1_DIAG_REPMS, TRUE,
		"Record exceeds Maximum-message-size"
	},
	{
		BIB1_DIAG_REMRS, TRUE,
		"Record exceeds Maximum-record-size"
	},
	{
		BIB1_DIAG_RSNSAAST, TRUE,
		"Result set not supported as a search term"
	},
	{
		BIB1_DIAG_OSRSASTMS, TRUE,
		"Only single result set as search term supported"
	},
	{
		BIB1_DIAG_OAOASRSASTS, TRUE,
		"Only ANDing of a single result set as search term supported"
	},
	{
		BIB1_DIAG_RSEARIO, TRUE,
		"Result set exists and replace indicator off"
	},
	{
		BIB1_DIAG_RSNNS, TRUE,
		"Result set naming not supported"
	},
	{
		BIB1_DIAG_COSDNS, TRUE,
		"Combination of specified databases not supported"
	},
	{
		BIB1_DIAG_ESNNS, TRUE,
		"Element set names not supported"
	},
	{
		BIB1_DIAG_SESNNVFSD, TRUE,
		"Specified element set name not valid for specified database"
	},
	{
		BIB1_DIAG_OASESNS, TRUE,
		"Only a single element set name supported"
	},
	{
		BIB1_DIAG_RSNLEUDBT, TRUE,
		"Result set no longer exists - unilaterally deleted by target"
	},
	{
		BIB1_DIAG_RSIIU, TRUE,
		"Result set is in use"
	},
	{
		BIB1_DIAG_OOTSDIL, TRUE,
		"One of the specified databases is locked"
	},
	{
		BIB1_DIAG_SRSDNE, TRUE,
		"Specified result set does not exist"
	},
	{
		BIB1_DIAG_RENRA, TRUE,
		"Resources exhausted - no results available"
	},
	{
		BIB1_DIAG_REUPRA, TRUE,
		"Resources exhausted - unpredictable partial results available"
	},
	{
		BIB1_DIAG_REVSORA, TRUE,
		"Resources exhausted - valid subset of results available"
	},
	{
		BIB1_DIAG_100_UE, TRUE,
		"Unspecified error"
	},
	{
		BIB1_DIAG_100_ACF, TRUE,
		"Access-control failure"
	},
	{
		BIB1_DIAG_100_SCRBCNBIRT, TRUE,
		"Security challenge required but could not be issued - request terminated"
	},
	{
		BIB1_DIAG_100_SCRBCNBIRNI, TRUE,
		"Security challenge required but could not be issued - record not included"
	},
	{
		BIB1_DIAG_100_SCFRNI, TRUE,
		"Security challenge failed - record not included"
	},
	{
		BIB1_DIAG_100_TBNCR, TRUE,
		"Terminated by negative continue response"
	},
	{
		BIB1_DIAG_100_NASATFTR, TRUE,
		"No abstract syntaxes agreed to for this record"
	},
	{
		BIB1_DIAG_100_QTNS, TRUE,
		"Query type not supported"
	},
	{
		BIB1_DIAG_100_MQ, TRUE,
		"Malformed query"
	},
	{
		BIB1_DIAG_100_DU, TRUE,
		"Database unavailable"
	},
	{
		BIB1_DIAG_100_OU, TRUE,
		"Operator unsupported"
	},
	{
		BIB1_DIAG_100_TMDS, TRUE,
		"Too many databases specified"
	},
	{
		BIB1_DIAG_100_TMRSC, TRUE,
		"Too many result sets created"
	},
	{
		BIB1_DIAG_100_UAT, TRUE,
		"Unsupported attribute type"
	},
	{
		BIB1_DIAG_100_UUA, TRUE,
		"Unsupported Use attribute"
	},
	{
		BIB1_DIAG_100_UVFUA, TRUE,
		"Unsupported value for Use attribute"
	},
	{
		BIB1_DIAG_100_UARBNS, TRUE,
		"Use attribute required but not supported"
	},
	{
		BIB1_DIAG_100_URA, TRUE,
		"Unsupported Relation attribute"
	},
	{
		BIB1_DIAG_100_USA, TRUE,
		"Unsupported Structure attribute"
	},
	{
		BIB1_DIAG_100_UPA, TRUE,
		"Unsupported Position attribute"
	},
	{
		BIB1_DIAG_100_UTA, TRUE,
		"Unsupported Truncation attribute"
	},
	{
		BIB1_DIAG_100_UAS, TRUE,
		"Unsupported Attribute Set"
	},
	{
		BIB1_DIAG_100_UCA, TRUE,
		"Unsupported Completeness attribute"
	},
	{
		BIB1_DIAG_100_UAC, TRUE,
		"Unsupported attribute combination"
	},
	{
		BIB1_DIAG_100_UCVFT, TRUE,
		"Unsupported coded value for term"
	},
	{
		BIB1_DIAG_100_MST, TRUE,
		"Malformed search term"
	},
	{
		BIB1_DIAG_100_ITVFA, TRUE,
		"Illegal term value for attribute"
	},
	{
		BIB1_DIAG_100_UFFUNV, TRUE,
		"Unparseable format for un-normalized value"
	},
	{
		BIB1_DIAG_100_IRSN, TRUE,
		"Illegal result set name"
	},
	{
		BIB1_DIAG_100_PSOSNS, TRUE,
		"Proximity search of sets not supported"
	},
	{
		BIB1_DIAG_100_IRSIPS, TRUE,
		"Illegal result set in proximity search"
	},
	{
		BIB1_DIAG_100_UPR, TRUE,
		"Unsupported proximity relation"
	},
	{
		BIB1_DIAG_100_UPUC, TRUE,
		"Unsupported proximity unit code"
	},
	{
		IR_UNKNOWN_ID_TYPE, TRUE,
		"*** Received unknown bib1 diagnostic code"
	}
};

static MSG_TABLE	UaHec_Z3950ResultSetStat[]=
{
	{
		Z3950_RSS_SUBSET, TRUE,
		"Result set status = SUBSET"
	},
	{
		Z3950_RSS_INTERIM, TRUE,
		"Result set status = INTERIM"
	},
	{
		Z3950_RSS_NONE, TRUE,
		"Result set status = NONE"
	},
	{
		IR_UNKNOWN_ID_TYPE, TRUE,
		"*** Received unknown result set status"
	}
};

static MSG_TABLE	UaHec_Z3950PresentStat[]=
{
	{
		Z3950_PS_SUCCESS, TRUE,
		"All of the expected database (or surrogate diagnostic) records are available"
	},
	{
		Z3950_PS_PARTIAL_1, TRUE,
		"Not all of the expected records can be returned because the request was terminated by access control"
	},
	{
		Z3950_PS_PARTIAL_2, TRUE,
		"Not all of the expected records can be returned because the message was terminated by maximum message size"
	},
	{
		Z3950_PS_PARTIAL_3, TRUE,
		"Not all of the expected records can be returned because the request was terminated by resource-control at origin"
	},
	{
		Z3950_PS_PARTIAL_4, TRUE,
		"Not all of the expected records can be returned because the request was terminated by resource-control at target"
	},
	{
		Z3950_PS_FAILURE, TRUE,
		"None of the expected database (or surrogate diagnostic) records can be returned. A single diagnostic is returned which is not a surrogate for a database record"
	},
	{
		IR_UNKNOWN_ID_TYPE, TRUE,
		"*** Received unknown present status"
	}
};

static MSG_TABLE	UaHec_Z3950DeleteStat[]=
{
	{
		Z3950_DS_SUCCESS, TRUE,
		"SUCCESS"
	},
	{
		Z3950_DS_RSDNE, TRUE,
		"Result set did not exist"
	},
	{
		Z3950_DS_PDBYT, TRUE,
		"Previously deleted by target"
	},
	{
		Z3950_DS_SPAT, TRUE,
		"System problem at target"
	},
	{
		Z3950_DS_ANA, TRUE,
		"Access not allowed"
	},
	{
		Z3950_DS_RCAO, TRUE,
		"Resource control at origin"
	},
	{
		Z3950_DS_RCAT, TRUE,
		"Resource control at target"
	},
	{
		Z3950_DS_BDNS, TRUE,
		"Bulk delete not supported"
	},
	{
		Z3950_DS_NARSDOBD, TRUE,
		"Not all result sets deleted on bulk delete"
	},
	{
		Z3950_DS_NARRSD, TRUE,
		"Not all requested result sets deleted"
	},
	{
		IR_UNKNOWN_ID_TYPE, TRUE,
		"*** Received unknown delete status"
	}
};

static MSG_TABLE	UaHec_Z3950PartialResults[]=
{
	{
		Z3950_RCPR_SUBSET, TRUE,
		"Resource control partial results = SUBSET"
	},
	{
		Z3950_RCPR_INTERIM, TRUE,
		"Resource control partial results = INTERIM"
	},
	{
		Z3950_RCPR_NONE, TRUE,
		"Resource control partial results = NONE"
	},
	{
		IR_UNKNOWN_ID_TYPE, TRUE,
		"*** Received unknown resource control partial results indicator"
	}
};

static MSG_TABLE	UaHec_Z3950RsrcReportStat[]=
{
	{
		Z3950_RRS_SUCCESS, TRUE,
		"Resource report status = SUCCESS"
	},
	{
		Z3950_RRS_PARTIAL, TRUE,
		"Resource report status = PARTIAL"
	},
	{
		Z3950_RRS_FAILURE_1, TRUE,
		"Resource report status = FAILURE_1"
	},
	{
		Z3950_RRS_FAILURE_2, TRUE,
		"Resource report status = FAILURE_2"
	},
	{
		Z3950_RRS_FAILURE_3, TRUE,
		"Resource report status = FAILURE_3"
	},
	{
		Z3950_RRS_FAILURE_4, TRUE,
		"Resource report status = FAILURE_4"
	},
	{
		IR_UNKNOWN_ID_TYPE, TRUE,
		"*** Received unknown resource report status"
	}
};

typedef struct
{
	CHAR	*oid;
	CHAR	*oidDesc;
} OID_TABLE;

static OID_TABLE	UaHec_Z3950ObjectIds[]=
{
	{
		"",
		"*** Searching for unknown object id"
	}
};


void UaHec_GetDiagMsg(SHORT diagType, SHORT diagCode, CHAR **msg)
{
	MSG_TABLE	*msgTbl;
	SHORT		i;

	msgTbl = (MSG_TABLE *)NULL;

	switch(diagType)
	{
	case MSG_DIAG_TYPE_BIB1:
		msgTbl = UaHec_Bib1DiagTbl;
		break;
	case MSG_DIAG_TYPE_RSS:
		msgTbl = UaHec_Z3950ResultSetStat;
		break;
	case MSG_DIAG_TYPE_PS:
		msgTbl = UaHec_Z3950PresentStat;
		break;
	case MSG_DIAG_TYPE_DS:
		msgTbl = UaHec_Z3950DeleteStat;
		break;
	case MSG_DIAG_TYPE_RCPR:
		msgTbl = UaHec_Z3950PartialResults;
		break;
	case MSG_DIAG_TYPE_RRS:
		msgTbl = UaHec_Z3950RsrcReportStat;
		break;
	default:
		*msg = "Unknown Diag Message type";
		return;
	}

	i = 0;
	while ((msgTbl[i].msgId != IR_UNKNOWN_ID_TYPE) &&
	    (msgTbl[i].msgId != diagCode))
	{
		i++;
	}
	*msg = msgTbl[i].string;

}
