/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: Send Search Request
 *
 * DESCRIPTION:
 *
 *	The Send Search Request CSU gets a completed search from 
 *	storage and sends attempts to send it to the indicated target.
 *
 * CONTENTS:
 *	UaSc_SendSearch
 *
 * NOTES:
 *	This CSU may only be invoked from with in the User Application
 * 	Layer of the IR Kernel-Origin
 *
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 *
/***************************************************************************/

#include <ir.h>
#include <ua.h>
#include <ua\rpnquery.h>
#include <resc.h>
#include <resclist.h>
#include <ir_oid.h>

/****************************************************************************
 * FUNCTION: UaSc_SendSearch
 *
 * PURPOSE:
 *	The User Application Search Control Send Search CSU obtains the
 *	active search from search maintenance and attempts to send it
 *	to the indicated target. The active search information must 
 *	contain "Query" and "Target" or "Result Set" information.  If
 *	This information is not present the request will be rejected.
 *
 * INPUT:
 *	There is no input to this function.
 *
 * OUTPUT:
 *	This function returns an IR_STATUS with returns a non-zero
 *	value on error. A call to UaHec_ErrorControl is required
 *	to inform the user of the problem.
 *
 * NOTES:
 *	Not yet required
 *
 ***************************************************************************/

IR_STATUS UaSc_SendSearch(void)
{
	UASC_MAINT_REQ	 mReq;	 /* Search Maintenance Request info. */
    	UASC_MAINT_REPLY mReply; /* Search Maintenance Reply info. */
	IRO_SEARCH_REQ	 search; /* Search Request info struct. */
	IR_STATUS	returnVal;

	/* Get the current database from storage. */
	UaSc_Maintenance(UASC_MAINT_GET_ACT, &mReq, &mReply);

	if (!mReply.act.canSearch)
	{
		/*
		 * We don't have enough information for the search.
		 */
		UaSc_ClearDB(&mReply.act.search.database);
		return(IR_SUCCESS);
	}

	CS_LOG_PRINTS(IR_FILE, "Beginning to convert the search\n");
	RESC_ASN_NO_MALLOC(&search);
	
	search.assocId =
	    mReply.act.search.database.assocId;
	search.smallSetUpperBound =
	    mReply.act.search.database.smallSetUpperBound;
	search.largeSetLowerBound =
	    mReply.act.search.database.largeSetPresentNum;
	search.mediumSetPresentNum =
	    mReply.act.search.database.mediumSetPresentNum;
	search.replaceIndicator =
	    mReply.act.search.database.replaceIndicator;

	RESC_ASN_NO_MALLOC(&search.resultSetName);
	RESC_ASN_INIT_VS(&search.resultSetName);
	RescASN_CopyVS(IR_FILE, &search.resultSetName,
	    &mReply.act.search.database.resultSetName);

	RESC_ASN_NO_MALLOC(&search.dbNameList);
	RESC_LIST_INIT(&search.dbNameList);
	RescList_Copy(IR_FILE, &search.dbNameList, 
	    &mReply.act.search.database.dbNameList);
	
	search.smallSetNameType = IR_ELEM_SET_NAME_NONE;
	
	RESC_ASN_NO_MALLOC(&search.smallSetList.generic);
	RESC_ASN_INIT_VS(&search.smallSetList.generic);
	RESC_ASN_NO_MALLOC(&search.smallSetList.dbSpecific);
	RESC_LIST_INIT(&search.smallSetList.dbSpecific);

	search.smallSetNameType = mReply.act.search.database.smallSetNameType;
	RescASN_CopyVS(IR_FILE, &search.smallSetList.generic,
		&mReply.act.search.database.smallSetNames.generic);
	RescList_Copy(IR_FILE, &search.smallSetList.dbSpecific,
		&mReply.act.search.database.smallSetNames.dbSpecific);


	RESC_ASN_NO_MALLOC(&search.mediumSetList.generic);
	RESC_ASN_INIT_VS(&search.mediumSetList.generic);
	RESC_ASN_NO_MALLOC(&search.mediumSetList.dbSpecific);
	RESC_LIST_INIT(&search.mediumSetList.dbSpecific);
	search.mediumSetNameType = mReply.act.search.database.mediumSetNameType;
	RescASN_CopyVS(IR_FILE, &search.mediumSetList.generic,
		&mReply.act.search.database.mediumSetNames.generic);
	RescList_Copy(IR_FILE, &search.mediumSetList.dbSpecific,
		&mReply.act.search.database.mediumSetNames.dbSpecific);

	
	RESC_ASN_NO_MALLOC(&search.recordSyntax);
	RESC_ASN_INIT_OI(&search.recordSyntax);
	RescASN_CopyOI(IR_FILE, &search.recordSyntax,
	    &mReply.act.search.database.recordSyntax);
	
	search.queryType = IR_QUERY_TYPE_1;
	/*
	 * Convert the query to the IR_RPN_QUERY_STRUCT.
	 */
	CS_LOG_PRINTS(IR_FILE, "About to convert local to IRO rpn query\n");
	RPN_StructToIrRPN(mReply.act.search.query, &search.query);

	CS_LOG_PRINTS(IR_FILE, "Converted local to IRO rpn query\n");
	if((returnVal = IRO_Search(UaEp_InfoUserId(), &search)) == IR_SUCCESS)
	{
		/*
		 * the search was successful. Clean up for another.
		 */
		UaSc_AddToQueryHist(mReply.act.search.query);
		RPN_Structure_Free(mReply.act.search.query);
		UaSc_Maintenance(UASC_MAINT_CLEAR_ACT, &mReq, &mReply);
	}
	else
	{
		/* The search failed don't clean up.*/
		RPN_Structure_Free(mReply.act.search.query);
	}
	/* Destroy the search request */
	UaSc_ClearDB(&mReply.act.search.database);
	RescUT_DestroyIROSearchReq(&search);
CS_LOG_PRINTS(IR_FILE, "Cleared IRO search strucutre\n");

	UaAc_CommStatusUpdate();
	return(returnVal);
}
